@@ -11,22 +11,28 @@
   '(("B" . mew-base64-encode-string)
     ("Q" . mew-q-encode-string)))
 
-(defvar mew-header-decode-switch
-  '(("B" . mew-base64-decode-string)
-    ("Q" . mew-q-decode-string)))
-
-(defvar mew-header-decode-error-switch
-  `(("B" . ,mew-error-invalid-b-encoding)
-    ("Q" . ,mew-error-invalid-q-encoding)))
-
 (defsubst mew-header-encode-get-func (b-or-q)
   (cdr (mew-assoc-case-equal b-or-q mew-header-encode-switch 0)))
 
+
+(defvar mew-header-decode-switch
+  `(("B"
+     mew-base64-decode-string
+     mew-base64-decode-error-string
+     ,mew-error-invalid-b-encoding)
+    ("Q"
+     mew-q-decode-string
+     nil
+     ,mew-error-invalid-q-encoding)))
+
 (defsubst mew-header-decode-get-func (b-or-q)
-  (cdr (mew-assoc-case-equal b-or-q mew-header-decode-switch 0)))
+  (nth 1 (mew-assoc-case-equal b-or-q mew-header-decode-switch 0)))
+
+(defsubst mew-header-decode-get-error-func (b-or-q)
+  (nth 2 (mew-assoc-case-equal b-or-q mew-header-decode-switch 0)))
 
 (defsubst mew-header-decode-get-error-str (b-or-q)
-  (cdr (mew-assoc-case-equal b-or-q mew-header-decode-error-switch 0)))
+  (nth 3 (mew-assoc-case-equal b-or-q mew-header-decode-switch 0)))
 
 (defconst mew-header-decode-regex
   "=\\?\\([^? \t]+\\)\\?\\(.\\)\\?\\([^? \t]+\\)\\?=")
@@ -81,7 +87,7 @@
   (let* ((estr (mew-cs-encode-string str hcs)))
     (concat "=?" charset "?" b-or-q "?" (funcall fun estr) "?=")))
 
-(defun mew-header-decode (charset b-or-q estr)
+(defun mew-header-decode (charset b-or-q estr &optional key)
   (let* ((fun (mew-header-decode-get-func b-or-q))
 	 (cs (mew-charset-to-cs charset))
 	 str)
@@ -93,6 +99,7 @@
       mew-error-unknown-charset)
      (fun ;; if cs is nil, mew-cs-decode-string does not cs-decode.
       (setq str (funcall fun estr))
+      (unless str (setq str (mew-header-decode-error key b-or-q estr)))
       (if (null str)
 	  (mew-header-decode-get-error-str b-or-q)
 	(when (and mew-decode-broken
@@ -107,12 +114,34 @@
 	  (mew-cs-decode-string str cs))))
      (t estr))))
 
+(defun mew-header-decode-error (key b-or-q estr)
+  (let ((func (mew-header-decode-get-error-func b-or-q)))
+    (if func (funcall func key b-or-q estr))))
+
+(defun mew-base64-decode-error-string (key b-or-q estr)
+  (let ((str estr) strlen r ret)
+    (when (string-match "^\\([^=]*\\)=*$" str)
+      (setq str (mew-match-string 1 str)))
+    (setq strlen (length str))
+    (setq r (% strlen 4))
+    (cond
+     ((= r 0)
+      (setq ret (mew-base64-decode-string str)))
+     ((= r 1)
+      )
+     ((= r 2)
+      (setq ret (mew-base64-decode-string (concat str "=="))))
+     ((= r 3)
+      (setq ret (mew-base64-decode-string (concat str "=")))))
+    (if ret (mew-decode-warning-fields key 'padding))
+    ret))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Quoted-printable encoding
 ;;;
 
-(defun mew-q-encode-string (str)
+(defun mew-q-encode-string (str &optional prefix)
   (let* ((len (length str))
 	 (ret (make-string (* len 3) ?a))
 	 (i 0) (j 0) char)
@@ -125,10 +154,11 @@
 	     (< char 126)
 	     (not (char-equal char ?=))
 	     (not (char-equal char ??))
-	     (not (char-equal char ?_))) ;; space
+	     (not (char-equal char ?_)) ;; space
+	     (or (not prefix) (not (memq char '(?# ?@)))))
 	(aset ret j char))
        (t
-	(aset ret j ?=)
+	(aset ret j (or prefix ?=))
 	(setq j (1+ j))
 	(aset ret j (aref "0123456789ABCDEF" (lsh char -4)))
 	(setq j (1+ j))
@@ -170,16 +200,16 @@
    ((and (<= ?A hex) (<= hex ?F)) (+ (- hex ?A) 10))
    ((and (<= ?a hex) (<= hex ?f)) (+ (- hex ?a) 10))))
 
-(defun mew-q-decode-string (qpstr &optional key)
+(defun mew-q-decode-string (qpstr &optional okey)
   (condition-case nil
       (let* ((len (length qpstr))
 	     (ret (make-string len ?a))
-	     (i 0) (j 0) char)
-	(setq key (or key ?=))
+	     (i 0) (j 0) char key)
+	(setq key (or okey ?=))
 	(while (< i len)
 	  (setq char (aref qpstr i))
 	  (cond
-	   ((char-equal char ?_)
+	   ((and (not okey) (char-equal char ?_))
 	    (aset ret j mew-sp))
 	   ((char-equal char key)
 	    (aset ret j (+ (* (mew-hexchar-to-int (aref qpstr (1+ i))) 16)
@@ -610,7 +640,8 @@
 	(end (match-end 0))
 	(css (mew-header-decode (mew-match-string 1)
 				(mew-match-string 2)
-				(mew-match-string 3))))
+				(mew-match-string 3)
+				key)))
     (setq css (mew-header-sanity-check-string2 css key))
     (delete-region beg end)
     (insert css)))
@@ -650,6 +681,9 @@
        ((eq err 'raw)
 	(setq level 2)
 	(setq wmsg (concat key " has raw text strings.\n")))
+       ((eq err 'padding)
+	(setq level 2)
+	(setq wmsg (concat key " has illegal padding in encoded text.\n")))
        ((eq err 'quoted)
 	(setq level 2)
 	(setq wmsg (concat key " has encoded-words in quoted text.\n"))))
@@ -666,7 +700,9 @@
        ((eq err 'ctl)
 	(concat "The '" param "' has invalid control codes.\n"))
        ((eq err 'enc)
-	(concat "The '" param "' parameter has encoded-word.\n")))
+	(concat "The '" param "' parameter has encoded-word.\n"))
+       ((eq err 'char)
+	(concat "The '" param "' has invalid charactors.\n")))
       (mew-xinfo-get-warning)))))
 
 (defun mew-header-decode-region (key rbeg rend &optional draftp)
@@ -938,11 +974,203 @@ That is, each line may be more than 75."
     (unless (string= value new) (mew-decode-warning-params param 'ctl))
     (list param new)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Punycode:: RFC 3492
+;;;
+
+(defconst mew-puny-base 36)
+(defconst mew-puny-tmin 1)
+(defconst mew-puny-tmax 26)
+(defconst mew-puny-damp 700)
+(defconst mew-puny-skew 38)
+(defconst mew-puny-initial-bias 72)
+(defconst mew-puny-initial-n 128)
+(defconst mew-puny-delimiter ?-)
+
+(defun mew-puny-adapt (delta numpoints firsttime)
+  (let ((k 0))
+    (if firsttime
+	(setq delta (/ delta mew-puny-damp))
+      (setq delta (/ delta 2)))
+    (setq delta (+ delta (/ delta numpoints)))
+    (while (> delta (/ (* (- mew-puny-base mew-puny-tmin) mew-puny-tmax) 2))
+      (setq delta (/ delta (- mew-puny-base mew-puny-tmin)))
+      (setq k (+ k mew-puny-base)))
+    (+ k (/ (* (1+ (- mew-puny-base mew-puny-tmin)) delta) (+ delta mew-puny-skew)))))
+
+(defun mew-puny-decode-digit (cp)
+  (if (< (- cp 48) 10)
+      (- cp 22)
+    (if (< (- cp 65) 26)
+	(- cp 65)
+      (if (< (- cp 97) 26)
+	  (- cp 97)
+	mew-puny-base))))
+
+(defun mew-puny-encode-digit (d)
+  (if (< d 26)
+      (+ d 22 75) ;; a-z
+    (+ d 22))) ;; 0-9
+
+(defun mew-puny-decode (input)
+  (when mew-cs-utf-16be
+    (condition-case nil
+	(mew-cs-decode-string
+	 (mew-puny-decode1 (substring input 4)) ;; xn--
+	 mew-cs-utf-16be)
+      (error nil))))
+
+(defun mew-puny-decode1 (input)
+  (let* ((n mew-puny-initial-n)
+	 (bias mew-puny-initial-bias)
+	 (len (length input))
+	 (in 0) (out 0)
+	 (i 0) (j 0) (b 0)
+	 digit thr oldi w k output ret)
+    (while (< j len)
+      (if (= (aref input j) mew-puny-delimiter) (setq b j))
+      (setq j (1+ j)))
+    (setq j 0)
+    (while (< j b)
+      (setq output (cons (aref input j) output))
+      (setq out (1+ out))
+      (setq j (1+ j)))
+    (setq output (nreverse output))
+    (if (> b 0) (setq in (1+ b)) (setq in 0))
+    (while (< in len)
+      (setq oldi i)
+      (setq w 1)
+      (setq k mew-puny-base)
+      (catch 'loop
+	(while t
+	  (if (>= in len) (error "punycode bad input"))
+	  (setq digit (mew-puny-decode-digit (aref input in)))
+	  (if (>= digit mew-puny-base) (error "punycode bad input"))
+	  (setq in (1+ in))
+	  (setq i (+ i (* digit w)))
+	  (if (<= k bias) 
+	      (setq thr mew-puny-tmin)
+	    (if (>= k (+ bias mew-puny-tmax))
+		(setq thr mew-puny-tmax)
+	      (setq thr (- k bias))))
+	  (if (< digit thr) (throw 'loop nil))
+	  (setq w (* w (- mew-puny-base thr)))
+	  (setq k (+ k mew-puny-base))))
+      (setq out (1+ out))
+      (setq bias (mew-puny-adapt (- i oldi) out (= oldi 0)))
+      (setq n (+ n (/ i out)))
+      (setq i (% i out))
+      (if (= i 0)
+	  (setq output (cons n (nthcdr i output)))
+	(setcdr (nthcdr (1- i) output) (cons n (nthcdr i output))))
+      (setq i (1+ i)))
+    (setq ret (make-string (* out 2) 0))
+    (setq j 0)
+    (while output
+      (aset ret j (/ (car output) 256))
+      (setq j (1+ j))
+      (aset ret j (% (car output) 256))
+      (setq j (1+ j))
+      (setq output (cdr output)))
+    ret))
+
+(defun mew-puny-encode (input)
+  (when mew-cs-utf-16be
+    (condition-case nil
+ 	(concat "xn--" (mew-puny-encode1 (mew-cs-encode-string input mew-cs-utf-16be)))
+      (error nil))))
+
+(defun mew-puny-encode1 (input)
+  (let* ((len (length input))
+	 (h-len (/ len 2))
+	 (n mew-puny-initial-n)
+	 (bias mew-puny-initial-bias)
+	 (delta 0) (out 0)
+	 (output (make-string (* len 4) 0))
+	 (j 0) h b m q k thr uni)
+    (while (< j len)
+      (setq uni (aref input j))
+      (setq j (1+ j))
+      (setq uni (+ (* uni 256) (aref input j)))
+      (setq j (1+ j))
+      (when (< uni 128) ;; basic
+	(aset output out uni)
+	(setq out (1+ out))))
+    (setq h out)
+    (setq b out)
+    (when (> b 0)
+      (aset output out mew-puny-delimiter)
+      (setq out (1+ out)))
+    (while (< h h-len)
+      (setq m 65536) ;; 17bits
+      (setq j 0)
+      (while (< j len)
+	(setq uni (aref input j))
+	(setq j (1+ j))
+	(setq uni (+ (* uni 256) (aref input j)))
+	(setq j (1+ j))
+	(if (and (>= uni n) (< uni m)) (setq m uni)))
+      (setq delta (+ delta (* (- m n) (1+ h))))
+      (setq n m)
+      (setq j 0)
+      (while (< j len)
+	(setq uni (aref input j))
+	(setq j (1+ j))
+	(setq uni (+ (* uni 256) (aref input j)))
+	(setq j (1+ j))
+	(when (< uni n)
+	  (setq delta (1+ delta))
+	  (if (= delta 0) (error "punycode overflow")))
+	(when (= uni n)
+	  (setq q delta)
+	  (setq k mew-puny-base)
+	  (catch 'loop
+	    (while t
+	      (if (<= k bias)
+		  (setq thr mew-puny-tmin)
+		(if (>= k (+ bias mew-puny-tmax))
+		    (setq thr mew-puny-tmax)
+		  (setq thr (- k bias))))
+	      (if (< q thr) (throw 'loop nil))
+	      (aset output out (mew-puny-encode-digit (+ thr (% (- q thr) (- mew-puny-base thr)))))
+	      (setq out (1+ out))
+	      (setq q (/ (- q thr) (- mew-puny-base thr)))
+	      (setq k (+ k mew-puny-base))))
+	  (aset output out (mew-puny-encode-digit q))
+	  (setq out (1+ out))
+	  (setq bias (mew-puny-adapt delta (1+ h) (= h b)))
+	  (setq delta 0)
+	  (setq h (1+ h))))
+      (setq delta (1+ delta))
+      (setq n (1+ n)))
+    (substring output 0 out)))
+
+(defun mew-puny-encode-url (url)
+  (let (beg end idn)
+    (with-temp-buffer
+      (insert url)
+      (goto-char (point-min))
+      (if (search-forward "://" nil t)
+	  (setq beg (point))
+	(setq beg (point-min)))
+      (if (search-forward "/" nil t)
+	  (setq end (1- (point)))
+	(setq end (point-max)))
+      (save-restriction
+	(narrow-to-region beg end)
+	(goto-char (point-min))
+	(while (re-search-forward "[^.]?[^. -][^.]*" nil t)
+	  (setq idn (mew-match-string 0))
+	  (delete-region (match-beginning 0) (match-end 0))
+	  (insert (mew-puny-encode idn))))
+      (mew-buffer-substring (point-min) (point-max)))))
+
 (provide 'mew-bq)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2005 Mew developing team.
+;; Copyright (C) 1997-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
