@@ -33,7 +33,7 @@
       (forward-line -1))
     (if (looking-at "\\([^:]*:\\)")
 	(mew-field-get-func (match-string 1) switch)
-      nil))) ;; what a case reachs here?
+      nil))) ;; what a case reaches here?
   
 ;;
 ;; Window management for completion candidates
@@ -41,7 +41,7 @@
 
 (defvar mew-complete-candidates nil)
 
-(defun mew-complete-window-delete ()
+(defun mew-complete-window-delete (&optional force)
   (when (mew-ainfo-get-win-cfg)
     ;; (mew-ainfo-get-win-cfg) remains when the last completion  
     ;; finished with multiple candidates.
@@ -52,12 +52,34 @@
     ;; (mew-ainfo-get-win-cfg), the cursor jumps to mini buffer.
     ;; This was a stupid bug of Mew. So, let's see if the complete
     ;; buffer is displayed or not.
-    (if (get-buffer-window mew-buffer-completions)
+    (if (or force (get-buffer-window mew-buffer-completions))
 	(set-window-configuration (mew-ainfo-get-win-cfg)))
     (mew-ainfo-set-win-cfg nil))
   (mew-remove-buffer mew-buffer-completions)
   (setq mew-complete-candidates nil))
 
+(defun mew-complete-insert-folder-function (choice buffer mini-p base-size)
+  (let ((start (mew-minibuf-point-min))
+	(proto (substring choice 0 1))
+	(pos (point)))
+    (while (not (or (= start (point))
+		    (not (char-before))
+		    (char-equal (char-before) ?,)))
+      (forward-char -1))
+    (if (and (member proto mew-folder-prefixes)
+	     (looking-at (concat "\\("
+				 (regexp-opt mew-config-cases t)
+				 ":\\)"
+				 (regexp-quote proto))))
+	(progn
+	  (delete-region (match-end 1) pos)
+	  (goto-char (match-end 1)))
+      (delete-region (point) pos))
+    (insert choice)
+    (remove-text-properties start (point-max) '(mouse-face nil))
+    (mew-complete-window-delete 'force)
+    t))
+
 (defun mew-complete-window-show (all)
   (unless (mew-ainfo-get-win-cfg)
     (mew-ainfo-set-win-cfg (current-window-configuration)))
@@ -70,8 +92,11 @@
 	      (set-window-start win 1)
 	    (scroll-other-window))))
     (setq mew-complete-candidates all)
-    (with-output-to-temp-buffer
-	mew-buffer-completions
+    (with-output-to-temp-buffer mew-buffer-completions
+      (when mew-inherit-complete-folder
+	(make-local-variable 'choose-completion-string-functions)
+	(add-hook 'choose-completion-string-functions
+		  'mew-complete-insert-folder-function))
       (display-completion-list all))))
 
 (defun mew-complete-backscroll ()
@@ -179,7 +204,8 @@ is inserted before the cursor, the short name is expanded to its address."
       (mew-input-folder-search-complete)
     (mew-draft-set-completion-ignore-case mew-complete-folder-ignore-case)
     (let ((word (mew-delete-backward-char))
-	  (completion-ignore-case mew-complete-folder-ignore-case))
+	  (completion-ignore-case mew-complete-folder-ignore-case)
+	  (mew-inherit-complete-folder t))
       (if (null word)
 	  (mew-complete-window-show (list "+"))
 	(if (and (mew-folder-absolutep word)
@@ -200,7 +226,8 @@ is inserted before the cursor, the short name is expanded to its address."
     (mew-complete-folder2)))
 
 (defun mew-input-folder-search-complete ()
-  (let (keys)
+  (let ((mew-inherit-complete-folder t)
+	keys)
     (save-excursion
       (set-buffer mew-input-folder-search-buf)
       (save-excursion
@@ -216,6 +243,7 @@ is inserted before the cursor, the short name is expanded to its address."
 (defun mew-complete-folder2 ()
   (let ((word (mew-delete-backward-char nil ", \t\n"))
 	(completion-ignore-case mew-complete-folder-ignore-case)
+	(mew-inherit-complete-folder t)
 	case folder)
     (cond
      ((null word)
@@ -273,7 +301,8 @@ is inserted before the cursor, the short name is expanded to its address."
   (if mew-input-folder-search-direction
       (mew-input-folder-search-complete)
     (let ((word (mew-delete-backward-char))
-	  (completion-ignore-case mew-complete-folder-ignore-case))
+	  (completion-ignore-case mew-complete-folder-ignore-case)
+	  (mew-inherit-complete-folder t))
       (if (null word)
 	  (mew-complete-window-show (list "%"))
 	(mew-complete
@@ -626,7 +655,11 @@ the name exists."
     (let ((inhibit-quit t))
       (mew-let-user-read)
       (delete-region savemax (point-max))
-      (if quit-flag (setq quit-flag nil	unread-command-events 7)))))
+      (when quit-flag
+	(setq quit-flag nil)
+	(setq unread-command-events (list (if mew-xemacs-p
+					      (character-to-event 7)
+					    7))))))) ;; 7 == C-g
 
 ;;
 ;; Extracting completion key
@@ -731,7 +764,7 @@ the name exists."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2003 Mew developing team.
+;; Copyright (C) 1997-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
