@@ -121,6 +121,9 @@
 (defun mew-fcc (&optional case)
   (mew-cfent-value case "fcc" mew-fcc))
 
+(defun mew-bcc (&optional case)
+  (mew-cfent-value case "bcc" mew-bcc))
+
 (defun mew-dcc (&optional case)
   (let ((dcc (mew-cfent-value case "dcc" mew-dcc)))
     (if (eq dcc 'me) (mew-mail-address case) dcc)))
@@ -159,8 +162,15 @@
 (defun mew-smtp-helo-domain (&optional case)
   (mew-cfent-value case "smtp-helo-domain" mew-smtp-helo-domain))
 
+(defun mew-smtp-user-only (&optional case)
+  (let ((user-domain (mew-smtp-user case)))
+    (if (string-match "^\\([^@]+\\)@" user-domain)
+	(mew-match-string 1 user-domain)
+      user-domain)))
+
 (defun mew-smtp-user (&optional case)
-  (mew-cfent-value case "smtp-user" mew-smtp-user))
+  (or (mew-cfent-value case "smtp-user" mew-smtp-user)
+      (mew-mail-address case)))
 
 (defun mew-smtp-auth-list (&optional case)
   (mew-cfent-value case "smtp-auth-list" mew-smtp-auth-list))
@@ -183,6 +193,12 @@
 	 (time (mew-time-ctz-to-msgid (current-time))))
     (concat "<" time "." random "." user "@" domain ">")))
 
+(defun mew-use-smtp-auth (&optional case)
+  (mew-cfent-value case "use-smtp-auth" mew-use-smtp-auth))
+
+(defun mew-smtp-auth-plain-authorize-id (&optional case)
+  (mew-cfent-value case "smtp-auth-plain-authorize-id" mew-smtp-auth-plain-authorize-id))
+
 ;;
 
 (defun mew-mailbox-type (&optional case)
@@ -260,6 +276,12 @@
 (defun mew-imap-ssl-port (&optional case)
   (mew-cfent-value case "imap-ssl-port" mew-imap-ssl-port))
 
+(defun mew-imap-proxy-server (&optional case)
+  (mew-cfent-value case "imap-proxy-server" mew-imap-proxy-server))
+
+(defun mew-imap-proxy-port (&optional case)
+  (mew-cfent-value case "imap-proxy-port" mew-imap-proxy-port))
+
 (defun mew-imap-user (&optional case)
   (mew-cfent-value case "imap-user" mew-imap-user))
 
@@ -323,6 +345,9 @@
 (defun mew-inbox-folder (&optional case)
   (mew-cfent-value case "inbox-folder" mew-inbox-folder))
 
+(defun mew-imap-inbox-folder (&optional case)
+  (mew-cfent-value case "imap-inbox-folder" mew-imap-inbox-folder))
+
 (defun mew-nntp-newsgroup (&optional case)
   (mew-cfent-value case "nntp-newsgroup" mew-nntp-newsgroup))
 
@@ -367,6 +392,20 @@
 
 ;;
 
+(defun mew-warn-addresses (&optional case)
+  (mew-cfent-value case "warn-addresses" mew-warn-addresses))
+
+(defun mew-safe-addresses (&optional case)
+  (mew-cfent-value case "safe-addresses" mew-safe-addresses))
+
+(defun mew-warn-domains (&optional case)
+  (mew-cfent-value case "warn-domains" mew-warn-domains))
+
+(defun mew-safe-domains (&optional case)
+  (mew-cfent-value case "safe-domains" mew-safe-domains))
+
+;;
+
 (defun mew-proto (&optional case)
   (mew-cfent-value case "proto" mew-proto))
 
@@ -408,6 +447,11 @@
 (defun mew-inbox-action-alist (&optional case)
   (mew-cfent-value case "inbox-action-alist" mew-inbox-action-alist))
 
+;;
+
+(defun mew-draft-privacy-method (&optional case)
+  (mew-cfent-value case "draft-privacy-method" mew-draft-privacy-method))
+
 ;;;
 ;;; Setting case
 ;;;
@@ -466,7 +510,7 @@ fields in the header according to the new value."
   (save-excursion
     (goto-char (point-min))
     (let ((new-case (mew-tinfo-get-case))
-	  from dcc eoh)
+	  from fcc dcc eoh)
       ;; (mew-header-end) cannot be used here
       (mew-header-goto-end)
       (setq eoh (point))
@@ -475,15 +519,17 @@ fields in the header according to the new value."
        ((mew-draft-resent-p eoh)
 	(setq from mew-resent-from:)
 	;; (setq cc mew-resent-cc:)
+	(setq fcc mew-resent-fcc:)
 	(setq dcc mew-resent-dcc:))
        (t
 	(setq from mew-from:)
 	;; (setq cc mew-cc:)
+	(setq fcc mew-fcc:)
 	(setq dcc mew-dcc:)))
       (mew-header-replace-value from (mew-from new-case))
       ;; (mew-header-replace-value cc (mew-cc new-case))
       (mew-header-replace-value dcc (mew-dcc new-case))
-      (mew-header-replace-value mew-fcc: (mew-fcc new-case))
+      (mew-header-replace-value fcc (mew-fcc new-case))
       (mew-header-replace-value mew-reply-to: (mew-reply-to new-case))
       (mew-header-replace-value mew-organization: (mew-organization new-case))
       (mew-header-delete-lines (mapcar 'car (mew-header-alist old-case)))
@@ -522,7 +568,7 @@ fields in the header according to the new value."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2005 Mew developing team.
+;; Copyright (C) 1999-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
