@@ -10,6 +10,13 @@
 ;; Constants which are used in two or more files must be here.
 ;; Constants used only in a file should be defined in the file.
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Suffix
+;;;
+
+(defconst mew-suffix ".mew")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Readability
@@ -106,8 +113,12 @@
 (defconst mew-ct-apk "Application/Pgp-Keys")
 (defconst mew-ct-pgs "application/pgp-signature") ;; due to the RFC 1847 bug
 (defconst mew-ct-pge "application/pgp-encrypted") ;; due to the RFC 1847 bug
-(defconst mew-ct-sms "application/pkcs7-signature")
-(defconst mew-ct-xsms "application/x-pkcs7-signature")
+(defconst mew-ct-smm "Application/Pkcs7-Mime")
+(defconst mew-ct-xsmm "Application/X-Pkcs7-Mime")
+(defconst mew-ct-smm-enc "enveloped-data")
+(defconst mew-ct-smm-sig "signed-data")
+(defconst mew-ct-sms "application/pkcs7-signature")  ;; due to the RFC 1847 bug
+(defconst mew-ct-xsms "application/x-pkcs7-signature")  ;; due to the RFC 1847 bug
 
 (defconst mew-us-ascii "us-ascii")
 (defconst mew-utf-8 "utf-8")
@@ -135,10 +146,10 @@
 (defconst mew-decode-composite-value
   `(,mew-7bit ,mew-8bit ,mew-bin))
 
-(defsubst mew-cte-p (cte)
+(defun mew-cte-p (cte)
   (member (downcase cte) mew-decode-value))
 
-(defsubst mew-cte-composite-p (cte)
+(defun mew-cte-composite-p (cte)
   (member (downcase cte) mew-decode-composite-value))
 
 ;; must be list of list.
@@ -204,17 +215,13 @@
 ;;;
 
 (defconst mew-regex-files "^[^.]\\|^.[^.]") ;; xxx "[^.]" is enough?
-(defconst mew-regex-message-files  "^[1-9][0-9]*$")
-(defconst mew-regex-message-files2 "^[1-9][0-9]*")
-(defconst mew-regex-message-files3 "^[0-9]+$")
-(defconst mew-regex-message-files4 "^0[1-9][0-9]*$")
 
 ;;;
 ;;; Sumsyn
 ;;;
 
 (defconst mew-regex-mark "^\\([^ \n]\\)")
-(defsubst mew-sumsyn-mark () (string-to-char (mew-match-string 1)))
+(defun mew-sumsyn-mark () (string-to-char (mew-match-string 1)))
 
 ;; fld msg 
 (defconst mew-regex-sumsyn-short   "\r \\([^ \n]*\\) \\([0-9]+\\)")
@@ -223,25 +230,32 @@
 (defconst mew-regex-sumsyn-long    "\r \\([^ \n]*\\) \\([0-9]+\\) \\([^ \n]*\\) \\([^ \n]*\\) \\([^ \n]*\\) \\([0-9]*\\)")
 
 ;; fld msg 
-(defsubst mew-regex-sumsyn-msg (msg)
+(defun mew-regex-sumsyn-msg (msg)
                            (format "\r \\([^ \n]*\\) \\(%s\\) " msg))
 
+(defun mew-summary-search-msg (msg)
+  (let ((regex (mew-regex-sumsyn-msg msg)))
+    (when (or (re-search-forward  regex nil t)
+	      (re-search-backward regex nil t))
+      (beginning-of-line)
+      t)))
+
 (defconst mew-regex-sumsyn-valid   "\r \\([^ \n]*\\) \\([1-9][0-9]*\\)")
 (defconst mew-regex-sumsyn-invalid "\r [^ \n]* 0[1-9][0-9]*")
 
-(defsubst mew-msg-validp (msg)
+(defun mew-msg-validp (msg)
   (and msg (string-match mew-regex-message-files msg)))
 
-(defsubst mew-msg-invalidp (msg)
+(defun mew-msg-invalidp (msg)
   (and msg (string-match "^0" msg)))
 
-(defsubst mew-msg-truncatedp (siz)
+(defun mew-msg-truncatedp (siz)
   (and siz (string-match "^0" siz)))
 
 ;; fld msg my-id par-id uid siz
-(defsubst mew-regex-sumsyn-my-id  (my-id)
+(defun mew-regex-sumsyn-my-id  (my-id)
   (format "\r [^ \n]* [0-9]+ %s [^ \n]*" (regexp-quote my-id)))
-(defsubst mew-regex-sumsyn-par-id (par-id)
+(defun mew-regex-sumsyn-par-id (par-id)
   (format "\r [^ \n]* [0-9]+ [^ \n]* %s" (regexp-quote par-id)))
 
 ;;;
@@ -252,7 +266,7 @@
 (defconst mew-regex-part      "^.... \\([.0-9]+\\) ")
 (defconst mew-regex-part-icon "<\\([.0-9]+\\)>")
 
-(defsubst mew-regex-jmp-part (part) (format "^.... \\(%s\\) " part))
+(defun mew-regex-jmp-part (part) (format "^.... \\(%s\\) " part))
 
 ;; [\033\200-\377] does not work due to the multibyte
 (defconst mew-regex-esc-or-nonascii "[^\000-\032\034-\177]") ;; \033 = esc
@@ -268,13 +282,15 @@
 
 ;; Can skip \n. Match the first id. For example:
 ;; In-Reply-To: Your message of "Mon, 5 Oct 1998 13:33:14 +0900"
-;; 	<19981005133317N.kazu@iijlab.net>
+;; 	<19981005133317N.kazu@example.org>
 (defconst mew-regex-id "<[^>\n\t]+>")
 
 (defconst mew-regex-case "^\\([^-+$%*,:][^,:]+\\):\\(.*\\)")
 (defconst mew-regex-case1 "^\\([^-+$%*,:][^,:]+\\(,[^,:]+\\)*\\):\\(.*\\)")
 (defconst mew-regex-case2 "\\([^-+$%*,:][^,:]+\\(,[^,:]+\\)*\\):")
 
+(defconst mew-regex-punycode "xn--\\([-0-9a-zA-z]+\\)")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Folder
@@ -318,12 +334,19 @@
 (defconst mew-inherit-refile-proto nil)
 (defconst mew-inherit-grep-cmd nil)
 (defconst mew-inherit-after-marking nil)
+(defconst mew-inherit-pick-tokens nil)
+(defconst mew-inherit-pick-ret nil)
+(defconst mew-inherit-pick-mewlp nil)
+(defconst mew-inherit-pick-omit-and nil)
+(defconst mew-inherit-pick-omit-and2 nil)
+(defconst mew-inherit-submission nil)
+(defconst mew-inherit-complete-folder nil)
 
 (provide 'mew-const)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2005 Mew developing team.
+;; Copyright (C) 2000-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
