@@ -12,9 +12,22 @@
 ;;; Decode info
 ;;;
 
-;; decode-body:
-;; t means decode with MIME encoding and convert charset.
-;; symbol means decode with MIME encoding only.
+;; limit:
+;;   Depth of RFC 822 messages.
+;;   The cnt starts with 0.
+;;   Decode if cnt <= limit
+;;   nil means that limit is 0.
+;;
+;; decode-text:
+;;   If t, decode a text MIME body.
+;;   Otherwise, it is treated as 'no-cs-conv
+;;
+;; decode-binary:
+;;   If t, decode a binary (non-text) MIME body.
+;;
+;; use-alt:
+;;   If t, one part in Multipart/Alternative is used.
+;;   Otherwise, it is treated as Multipart/Mixed.
 
 (defvar mew-dinfo-list '("limit" "decode-text" "decode-binary" "use-alt"))
 
@@ -56,9 +69,9 @@
   `((,mew-ct-pge mew-pgp-decrypt mew-pgp-ver mew-prog-pgp)))
 
 (defvar mew-decode-multipart-signed-switch
-  `((,mew-ct-pgs mew-pgp-verify    mew-pgp-ver   mew-prog-pgp)
-    (,mew-ct-sms mew-smime-verify  mew-smime-ver mew-prog-smime)
-    (,mew-ct-xsms mew-smime-verify mew-smime-ver mew-prog-smime)))
+  `((,mew-ct-pgs  mew-pgp-verify          mew-pgp-ver   mew-prog-pgp)
+    (,mew-ct-sms  mew-smime-detach-verify mew-smime-ver mew-prog-smime)
+    (,mew-ct-xsms mew-smime-detach-verify mew-smime-ver mew-prog-smime)))
 
 ;;
 
@@ -236,7 +249,8 @@ Return the existence MIME-Version: and the value of Subject:."
 	 (visibles (make-list (length mew-field-spec) nil))
 	 (prop mew-use-highlight-header)
 	 key beg med subj from contents others
-	 key-face val-face N nspec visiblep mimep fld ext ext-str ext-face)
+	 key-face val-face N nspec visiblep mimep
+	 size ext ext-str ext-face)
     (mew-decode-narrow-to-header
      (while (not (eobp))
        (if (not (looking-at mew-keyval))
@@ -249,15 +263,15 @@ Return the existence MIME-Version: and the value of Subject:."
 	 (setq nspec (mew-nspec-by-key key))
 	 (setq N (mew-nspec-n nspec))
 	 (setq visiblep (mew-nspec-visiblep nspec))
+	 (if (fboundp visiblep)
+	     (setq visiblep (funcall visiblep (mew-buffer-substring med (1- (point))))))
 	 (cond
 	  ((string= key mew-x-mew:)
 	   ;; deleting X-Mew: on the RFC822 header
 	   (delete-region beg (point)))
 	  ((string= key mew-x-mew-uidl:)
-	   (when (and (eq cnt 1)
-		      (setq fld (mew-cinfo-get-fld))
-		      (not (and mew-trash-folder (string= fld mew-trash-folder)))
-		      (not (mew-folder-remotep (mew-case:folder-folder fld))))
+	   (setq size (mew-scan-uid-size (mew-buffer-substring med (1- (point)))))
+	   (when (and (eq cnt 1) (mew-msg-truncatedp size))
 	     (mew-xinfo-set-action
 	      (mew-substitute-for-summary 
 	       "Too large, truncated (the 'T' mark). To get the entire message, type '\\[mew-summary-retrieve-message]'"))))
@@ -587,7 +601,7 @@ Return a part syntax after moving the beginning of the content body."
 	 (ct (mew-syntax-get-value ctl 'cap))
 	 (textp (mew-ct-textp ct))
 	 (cte (or (mew-syntax-get-cte syntax) mew-7bit))
-	 (multi-err t) (encap nil) type)
+	 (multi-err t) (encap nil) type hend format delsp)
     ;; the beginning of the content body
     (cond
      ((not (mew-cte-p cte))
@@ -618,6 +632,10 @@ Return a part syntax after moving the beginning of the content body."
 	  ;; xxx how about message/partial?
 	  (mew-syntax-set-ct syntax mew-type-apo)
 	  ))))
+     ((or (string= ct mew-ct-smm) (string= ct mew-ct-xsmm))
+      (mew-decode-mime-body nil ct cte)
+      (mew-syntax-set-end syntax (point-max))
+      (setq syntax (mew-decode-smime syntax cnt)))
      ;; Multipart, decoding is not required
      ((mew-ct-multipartp ct)
       (if (mew-cte-composite-p cte)
@@ -633,6 +651,7 @@ Return a part syntax after moving the beginning of the content body."
 	 ((string= mew-ct-mls ct)
 	  (setq syntax (mew-decode-multipart-signed syntax cnt)))
 	 ((string= mew-ct-mle ct)
+	  ;; xxx making use of password cache?
 	  (if mew-inherit-prefetching
 	      (signal 'quit "")
 	    (setq syntax (mew-decode-multipart-encrypted syntax cnt))))
@@ -660,13 +679,23 @@ Return a part syntax after moving the beginning of the content body."
     ;; or single in multipart/alternative
     ;; or single in multipart/security
     (setq type (mew-syntax-get-value (mew-syntax-get-ct syntax) 'cap))
-    (unless (mew-syntax-get-end syntax)
+    (setq hend (mew-syntax-get-end syntax))
+    (unless hend
       (mew-syntax-set-end syntax (point-max)))
     (when (and (string= type mew-ct-txt)
 	       ;; highlight in the cache buffers only
 	       (string-match mew-buffer-cache-prefix (buffer-name)))
-      (mew-highlight-body-region (mew-syntax-get-begin syntax)
-				 (mew-syntax-get-end syntax)))
+      (setq format (mew-syntax-get-param ctl "format"))
+      (when (and format (mew-case-equal format "flowed"))
+	(setq delsp (mew-syntax-get-param ctl "delsp"))
+	(if (and delsp (mew-case-equal delsp "yes"))
+	    (setq delsp t)
+	  (setq delsp nil))
+	(mew-decode-flowed (mew-syntax-get-begin syntax) (point-max) delsp)
+	(setq hend nil))
+      (mew-highlight-body-region (mew-syntax-get-begin syntax) (point-max)))
+    (unless hend
+      (mew-syntax-set-end syntax (point-max))) ;; ajusting
     (if encap
 	;; Mew allows text/plain and multipart/* for body.
 	;; If other CT: is embedded under message, it should be
@@ -680,6 +709,39 @@ Return a part syntax after moving the beginning of the content body."
       ;; return value
       syntax)))
 
+(defun mew-decode-flowed-level ()
+  (cond
+   ((looking-at "^$") -1)
+   ((looking-at "^>*") (- (match-end 0) (match-beginning 0)))))
+
+(defun mew-decode-flowed (beg end delsp)
+  "Decoding wrapped lines encoded with RFC 3676"
+  (save-restriction
+    (narrow-to-region beg end)
+    (goto-char (point-min))
+    (let ((adj (if delsp 2 1))
+	  (level (mew-decode-flowed-level))
+	  next-level softbreak)
+      (end-of-line)
+      ;; char-before causes an error at the beginning of buffer
+      (if (and (not (bobp)) (= (char-before) mew-sp))
+	  (setq softbreak (1+ (point)))
+	(setq softbreak nil))
+      (while (= (forward-line) 0)
+	(setq next-level (mew-decode-flowed-level))
+	(if (/= level next-level)
+	    (setq level next-level)
+	  (when softbreak
+	    (delete-region (- softbreak adj) softbreak)
+	    (while (and (char-after) (char-equal (char-after) ?>))
+	      (delete-char 1))
+	    (while (and (char-after) (char-equal (char-after) mew-sp))
+	      (delete-char 1))))
+	(end-of-line)
+	(if (= (char-before) mew-sp)
+	    (setq softbreak (1+ (point)))
+	  (setq softbreak nil))))))
+
 ;;;
 ;;; Decoding multipart
 ;;;
@@ -687,8 +749,14 @@ Return a part syntax after moving the beginning of the content body."
 (defun mew-decode-multipart-boundary-regex (boundary)
   ;; for broken MUAs which uses non-ASCII boundaries
   ;; This works only on Emacs 21.
-  (setq boundary (mew-set-string-multibyte boundary))
-  (concat "^--" (regexp-quote boundary) "\\(--\\|\\)[ \t]*$"))
+  (let ((mboundary (mew-set-string-multibyte boundary)))
+    ;; Some MUAs use invalid charactor like "<>".
+    (when (or (string-match "[!-&*;<>@[-^`{-~]" mboundary)
+	      (not (string= mboundary boundary)))
+      (if mew-decode-broken
+	  (mew-decode-warning-params "boundary" 'char)
+	(mew-decode-error "Boundary has invalid charactor")))
+    (concat "^--" (regexp-quote mboundary) "\\(--\\|\\)[ \t]*$")))
 
 (defsubst mew-decode-multipart-boundary-cont ()
   (string= (mew-match-string 1) ""))
@@ -961,7 +1029,7 @@ Return a part syntax after moving the beginning of the content body."
 	  (setq start (match-end 0))
 	  (when (string-match "^\r?$" key)
 	    (save-restriction
-	      (if (string-match mew-bin cte)
+	      (if (string-match cte mew-bin)
 		  (narrow-to-region (point-min) (1+ start))
 		(narrow-to-region (point-min) (point-max)))
 	      (goto-char (point-min))
@@ -976,7 +1044,7 @@ Return a part syntax after moving the beginning of the content body."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
