@@ -25,7 +25,7 @@
 
 (defvar mew-encode-multipart-signed-switch
   `((,mew-ct-pgs mew-pgp-sign mew-pgp-canonicalize)
-    (,mew-ct-sms mew-smime-sign)))
+    (,mew-ct-sms mew-smime-detach-sign)))
 
 ;;
 
@@ -181,8 +181,8 @@
 	    (setq val (mew-buffer-substring start (1- (point))))
 	    (delete-region start (1- (point)))
 	    (backward-char 1)
-	    ;;
-	    (setq vals (mapcar 'mew-chop (mew-split-quoted val ?, ?: ?\;)))
+	    ;; single quote is allowed for local-part
+	    (setq vals (mapcar 'mew-chop (mew-split-quoted val ?, ?: ?\; 'no-single)))
 	    (while vals
 	      (setq val (car vals))
 	      (setq vals (cdr vals))
@@ -371,6 +371,10 @@
 	(error
 	 (mew-encode-undo)
 	 (message "%s" (mew-tinfo-get-encode-err))
+	 (setq ret nil))
+	(quit
+	 (mew-encode-undo)
+	 (message "quit")
 	 (setq ret nil))))
     ret))
 
@@ -406,7 +410,7 @@
       (mew-encode-error (format "'%s' is not in the right form!" recipients))))
     ;; Header modifications which are not remained.
     (mew-header-delete-lines (list mew-fcc: mew-resent-fcc:)) ;; anyway
-    (mew-smtp-set-dcc pnm (mew-encode-delete-dcc resentp))
+    (mew-encode-delete-dcc resentp)
     (mew-smtp-set-bcc pnm (mew-encode-delete-bcc resentp))
     (mew-smtp-set-fcc pnm fcc)
     ;;
@@ -447,6 +451,10 @@
 	(error
 	 (mew-encode-undo)
 	 (message "%s" (mew-tinfo-get-encode-err))
+	 (setq ret nil))
+	(quit
+	 (mew-encode-undo)
+	 (message "quit")
 	 (setq ret nil))))
     ret))
 
@@ -508,6 +516,10 @@
 
 (defun mew-encode-make-single ()
   ;; Just after the header
+  ;; Using Multipart/Signed if the body is a single part.
+  (if (equal (mew-syntax-get-privacy mew-encode-syntax)
+	     `((,mew-ct-smm ,mew-ct-smm-sig)))
+      (mew-syntax-set-privacy mew-encode-syntax `((,mew-ct-mls ,mew-ct-sms))))
   (mew-encode-singlepart mew-encode-syntax nil nil t 'cover))
 
 (defun mew-encode-charset-8bitp (charset)
@@ -531,7 +543,7 @@
 (defvar mew-ask-encoding t)
 (defvar mew-default-encoding mew-b64)
 
-(defun mew-encode-mime-body (ctl cte file)
+(defun mew-encode-mime-body (ctl cte file no-encoding)
   ;; If file is 't', target is buffered.
   ;; text should be buffered
   ;; 	- specified charset is a rare case
@@ -589,13 +601,19 @@
 	    ;; unknown charset
 	    (setq cte mew-default-encoding))
 	(cond
+	 (no-encoding
+	  (if (mew-encode-charset-8bitp charset)
+	      (setq cte mew-8bit)
+	    (setq cte mew-7bit)))
 	 (mew-inherit-7bit
 	  (if (mew-case-equal cte mew-8bit)
 	      (setq cte mew-default-encoding)))
 	 (mew-use-8bit
 	  (if (mew-encode-charset-8bitp charset)
 	      (setq cte mew-8bit))))
-	(when (or (mew-case-equal cte mew-7bit) (mew-case-equal cte mew-8bit))
+	(when (and (not no-encoding)
+		   (or (mew-case-equal cte mew-7bit)
+		       (mew-case-equal cte mew-8bit)))
 	  (save-excursion
 	    (goto-char beg)
 	    (mew-set-buffer-multibyte nil)
@@ -615,11 +633,17 @@
 	  (setq cte mew-8bit)
 	(setq cte mew-7bit)))
      (t
+      (if (and no-encoding cte (mew-case-equal cte mew-b64))
+	  (setq cte mew-bin))
       ;; There are 7bit ascii bodies such as 
       ;; application/pgp-encrypted and message/external-body.
       ;; If 7bit or 8bit, it should be linebase.
       (if (null cte) (setq cte mew-7bit))))
     (cond
+     ((mew-case-equal cte mew-bin)
+      (mew-frwlet
+       mew-cs-binary mew-cs-dummy
+       (mew-insert-file-contents file)))
      ((or (mew-case-equal cte mew-7bit) (mew-case-equal cte mew-8bit))
       ;; Certainly linebase here.
       (unless textp
@@ -664,6 +688,22 @@
       (mew-encode-error (concat mew-prog-mime-encode " does not exist"))))
     (list (if charset (list "charset" charset)) cte)))
 
+(defun mew-encode-no-mime-encoding (privacy)
+  (let ((first (car privacy)))
+    (and (listp first)
+	 (or (string= (car first) mew-ct-mle)
+	     (string= (car first) mew-ct-smm)))))
+
+(defun mew-broken-name (cdpl)
+  (let ((mew-encode-word-max-length 1000) ;; xxx
+	(file (nth 1 (assoc "filename" cdpl)))
+	name)
+    (when (and file (string-match mew-regex-nonascii file))
+      (setq name (car (mew-header-encode-string file)))
+      ;; name must not be double-quoted here.
+      ;; mew-heaer-insert will do this later
+      (list "name" name))))
+
 (defun mew-encode-singlepart (syntax &optional path depth buffered coverp)
   ;; path is nil if called make-single or security multipart
   ;; buffered is t if called make-single
@@ -676,10 +716,11 @@
 	 (cd (mew-syntax-get-cd syntax))
 	 (cdpl (mew-syntax-get-cdp syntax))
 	 (privacy (mew-syntax-get-privacy syntax))
+	 (no-encoding (mew-encode-no-mime-encoding privacy))
 	 (mew-inherit-7bit (mew-encode-limit-7bitp privacy))
 	 (beg (point))
-	 charset-cte charset bodybeg cst ask-cst)
-    (setq charset-cte (mew-encode-mime-body ctl cte (or buffered file)))
+	 charset-cte charset bodybeg cst ask-cst broken-name)
+    (setq charset-cte (mew-encode-mime-body ctl cte (or buffered file) no-encoding))
     (goto-char beg)
     (mew-set '(charset cte) charset-cte)
     (setq ctl (mew-delete "icharset" ctl))
@@ -687,6 +728,11 @@
       (setq ctl (mew-syntax-get-params ctl))
       (setq ctl (mew-delete "charset" ctl))
       (setq ctl (cons ct (cons charset ctl))))
+    ;;
+    (setq ctl (mew-delete "name" ctl))
+    (setq broken-name (mew-broken-name cdpl))
+    (if broken-name (setq ctl (nconc ctl (list broken-name))))
+    ;;
     (mew-header-insert mew-ct: ctl)
     (mew-header-insert mew-cte: cte)
     (and cd (mew-header-insert mew-cd: cd))
@@ -702,7 +748,7 @@
 	(mew-header-delete-lines mew-field-delete-for-forwarding)))
     (when privacy
       (mew-encode-security-multipart
-       beg privacy depth (mew-syntax-get-decrypters syntax)))
+       beg privacy depth (mew-syntax-get-decrypters syntax) cte))
     (goto-char (point-max))
     (when (and coverp (setq cst (nth 1 charset)))
       (cond
@@ -780,7 +826,7 @@
 	   (mew-replace-character (current-time-string) mew-sp ?_) ?: ?_)
 	  (mew-random-string 3 t)))
 
-(defun mew-encode-multipart (syntax path depth &optional buffered)
+(defun mew-encode-multipart (syntax path depth &optional buffered cte)
   (let* ((boundary
 	  (mew-boundary-get ;; 0 is nil for Next_Part
 	   (if (> depth 0) (format "BOUNDARY%s" (number-to-string depth)))))
@@ -822,7 +868,7 @@
       (mew-header-insert mew-cte: (if (> 8bit-cnt 0) mew-8bit mew-7bit)))
     (when privacy 
       (mew-encode-security-multipart
-       beg privacy depth (mew-syntax-get-decrypters syntax)))
+       beg privacy depth (mew-syntax-get-decrypters syntax) cte))
     (goto-char (point-max))
     (> 8bit-cnt 0)))
 
@@ -831,7 +877,7 @@
 ;;; Privacy services
 ;;;
 
-(defun mew-encode-security-multipart (beg privacy depth decrypters)
+(defun mew-encode-security-multipart (beg privacy depth decrypters cte)
   (save-restriction
     (narrow-to-region beg (point-max))
     (let (proto ct)
@@ -841,19 +887,23 @@
 	(setq ct (mew-capitalize ct))
 	(setq privacy (cdr privacy))
 	(cond 
-	 ((string= mew-ct-mle ct)
-	  (mew-encode-multipart-encrypted ct proto depth decrypters))
-	 ((string= mew-ct-mls ct)
-	  (mew-encode-multipart-signed ct proto depth)))))))
+	 ((string= ct mew-ct-mle)
+	  (mew-encode-multipart-encrypted ct proto depth decrypters cte))
+	 ((string= ct mew-ct-mls)
+	  (mew-encode-multipart-signed ct proto depth))
+	 ((string= ct mew-ct-smm)
+	  (mew-encode-smime proto cte decrypters)))))))
 
 (defun mew-security-multipart-boundary (depth)
    (if depth
        (mew-boundary-get (format "Security_Multipart%s" (number-to-string depth)))
      (mew-boundary-get "Security_Multipart")))
 
-(defun mew-save-transfer-form (beg end retain)
+(defun mew-save-transfer-form (beg end retain &optional cte)
   ;; called in the narrowed region
-  (let ((sbeg beg) (send end) (draft-buf (current-buffer)) tmpbuf file)
+  (let ((sbeg beg) (send end) (draft-buf (current-buffer))
+	(ocs mew-cs-text-for-net)
+	tmpbuf file)
     (if retain
 	(progn
 	  (setq tmpbuf (generate-new-buffer mew-buffer-prefix))
@@ -861,13 +911,23 @@
 	  (mew-erase-buffer)
 	  (mew-insert-buffer-substring draft-buf beg end)
 	  (setq sbeg (point-min) send (point-max))))
-    (unless mew-cs-text-for-net
-      (goto-char sbeg) ;; just in case
-      (mew-lf-to-crlf)
-      (setq send (point-max)))
+    (goto-char sbeg) ;; just in case
+    (if (and cte (mew-case-equal cte mew-bin)
+	     (re-search-forward mew-eoh))
+	(progn
+	  (setq ocs mew-cs-binary)
+	  (forward-line)
+	  (save-restriction
+	    (narrow-to-region sbeg (point))
+	    (goto-char sbeg)
+	    (mew-lf-to-crlf)))
+      (unless mew-cs-text-for-net
+	(goto-char sbeg) ;; just in case
+	(mew-lf-to-crlf)))
+    (setq send (point-max))
     (setq file (mew-make-temp-name))
     (mew-frwlet
-     mew-cs-dummy mew-cs-text-for-net
+     mew-cs-dummy ocs
      (write-region sbeg send file nil 'no-msg))
     (if retain
 	(mew-remove-buffer tmpbuf)
@@ -875,16 +935,17 @@
     (set-buffer draft-buf)
     file)) ;; return value
 
-(defun mew-encode-multipart-encrypted (ct proto depth decrypters)
+(defun mew-encode-multipart-encrypted (ct proto depth decrypters cte)
   ;; called in the narrowed region
   (let* ((boundary (mew-security-multipart-boundary depth))
 	 (switch mew-encode-multipart-encrypted-switch) ;; save length
 	 (func (mew-encode-get-security-func proto switch))
 	 file1 file2 file3 cte2 cte3 fc errmsg)
-    (if mew-encrypt-to-myself
+    (if (and mew-encrypt-to-myself
+	     (not (member mew-inherit-encode-signer decrypters)))
 	(setq decrypters (cons mew-inherit-encode-signer decrypters)))
     ;; Write the part converting line breaks.
-    (setq file1 (mew-save-transfer-form (point-min) (point-max) nil))
+    (setq file1 (mew-save-transfer-form (point-min) (point-max) nil cte))
     ;; The narrowed region stores nothing
     ;; Call the protocol function
     (condition-case nil
@@ -908,7 +969,7 @@
       (mew-header-insert mew-ct: (list ct
 				       (list "protocol" proto)
 				       (list "boundary" boundary)))
-      (mew-header-insert mew-cte: "7bit")
+      (mew-header-insert mew-cte: mew-7bit)
       (insert (format "\n--%s\n" boundary))
       ;; Insert control keys
       (mew-encode-singlepart 
@@ -957,7 +1018,7 @@
 				       (list "protocol" proto)
 				       (list "micalg" micalg)
 				       (list "boundary" boundary)))
-      (mew-header-insert mew-cte: "7bit")
+      (mew-header-insert mew-cte: mew-7bit)
       (insert (format "\n--%s\n" boundary))
       (goto-char (point-max))
       ;; After the signed part
@@ -1026,7 +1087,7 @@
       (goto-char hd-end)
       (mew-header-insert mew-mv: (concat mew-mv:-num " " mew-field-comment)))
      (t
-      (goto-char (marker-position hd-end))
+      (goto-char hd-end)
       (forward-line)
       (setq body-beg (point))
       (setq body-end (point-max))
@@ -1205,7 +1266,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
