@@ -13,9 +13,11 @@
 ;;;
 
 (defun mew-mark-kill-refile (src msg)
-  (let ((myself (member src (mew-refile-get msg))))
+  (let ((refinfo (mew-refile-get msg))
+	myself)
+    (setq myself (or (null refinfo) (member src refinfo)))
     (if myself (mew-summary-refile-remove-body))
-    (not myself))) ;; return value
+    (not myself)))
 
 (defun mew-mark-exec-refile (src msgs)
   "Refile MSGs from the SRC folder."
@@ -104,7 +106,7 @@ DELS represents the messages to be deleted."
   (let ((suffix (mew-folder-suffix src))
 	file infofile)
     (while dels
-      (setq file (mew-expand-folder src (car dels)))
+      (setq file (mew-expand-msg src (car dels)))
       (setq dels (cdr dels))
       ;; if file does not exist, the marked line will be deleted anyway.
       (mew-delete-file file)
@@ -114,8 +116,8 @@ DELS represents the messages to be deleted."
 
 (defun mew-i2s (num cache)
   (if cache
-      (concat "0" (int-to-string num)) ;; "0" is the mark for invalid messages
-    (int-to-string num)))
+      (concat "0" (number-to-string num)) ;; "0" is the mark for invalid messages
+    (number-to-string num)))
 
 (defun mew-summary-mv-msgs (src dsts-msgs)
   (let* ((dsts (car dsts-msgs)) ;; (+foo +bar)
@@ -132,20 +134,20 @@ DELS represents the messages to be deleted."
 	  (while msgs
 	    (setq msg (car msgs))
 	    (setq msgs (cdr msgs))
-	    (if (file-regular-p (mew-expand-folder src msg))
+	    (if (file-regular-p (mew-expand-msg src msg))
 		(setq msgs- (cons msg msgs-))))
 	  (setq msgs- (nreverse msgs-))
 	  (setq msgs msgs-))
       (setq dst (mew-case-folder mew-inherit-exec-case (car dsts)))
       (setq dsts (cdr dsts))
-      (setq num (string-to-int (mew-folder-new-message dst 'num-only cache)))
+      (setq num (string-to-number (mew-folder-new-message dst 'num-only cache)))
       (while msgs
-	(setq srcfile (mew-expand-folder src (car msgs)))
+	(setq srcfile (mew-expand-msg src (car msgs)))
 	(setq msgs (cdr msgs))
 	(when (and (file-exists-p srcfile) (file-writable-p srcfile))
 	  (setq strnum (mew-i2s num cache))
 	  (setq msgs- (cons strnum msgs-))
-	  (setq dstfile (mew-expand-folder dst strnum))
+	  (setq dstfile (mew-expand-new-msg dst strnum))
 	  (setq num (1+ num))
 	  (rename-file srcfile dstfile 'override)
 	  (when suffix
@@ -157,13 +159,13 @@ DELS represents the messages to be deleted."
     (while dsts
       (setq dst (mew-case-folder mew-inherit-exec-case (car dsts)))
       (setq dsts (cdr dsts))
-      (setq num (string-to-int (mew-folder-new-message dst 'num-only cache)))
+      (setq num (string-to-number (mew-folder-new-message dst 'num-only cache)))
       (setq msgs msgs-)
       (while msgs
-	(setq srcfile (mew-expand-folder src (car msgs)))
+	(setq srcfile (mew-expand-msg src (car msgs)))
 	(setq msgs (cdr msgs))
 	(setq strnum (mew-i2s num cache))
-	(setq dstfile (mew-expand-folder dst strnum))
+	(setq dstfile (mew-expand-new-msg dst strnum))
 	(setq num (1+ num))
 	(mew-link srcfile dstfile))
       (mew-touch-folder dst))))
@@ -175,12 +177,12 @@ DELS represents the messages to be deleted."
 	 (suffix (mew-folder-suffix src))
 	 num srcfile dstfile infofile)
     (unless (file-directory-p trashdir) (make-directory trashdir))
-    (setq num (string-to-int (mew-folder-new-message trash 'num-only cache)))
+    (setq num (string-to-number (mew-folder-new-message trash 'num-only cache)))
     ;; must be here after ensuring that +trash exists.
     (while dels
-      (setq srcfile (mew-expand-folder src (car dels))) ;; xxx
+      (setq srcfile (mew-expand-msg src (car dels))) ;; xxx
       (setq dels (cdr dels))
-      (setq dstfile (mew-expand-folder trash (mew-i2s num cache)))
+      (setq dstfile (mew-expand-new-msg trash (mew-i2s num cache)))
       (setq num (1+ num))
       (if (file-regular-p srcfile)
 	  ;; if not, the marked line will be deleted anyway.
@@ -267,27 +269,20 @@ destination is the same as that of the current message are processed."
 ;;; Exec region
 ;;;
 
-(defun mew-summary-move-and-display (msg)
-  (let ((here (point)))
-    (goto-char (point-min))
-    (if (not (re-search-forward (mew-regex-sumsyn-msg msg) nil t))
-	(goto-char here)
-      (beginning-of-line)
-      (mew-summary-display nil))))
+(defun mew-summary-move-and-display (msg &optional redisplay)
+  (when (mew-summary-search-msg msg)
+    (mew-summary-display redisplay)))
 
 (defun mew-summary-go-back-summary (key)
-  (if (not (mew-thread-p))
-      (message "Move to a physical folder first")
-    (let* ((bnm (mew-summary-folder-name 'ext))
-	   (fld (mew-thread-to-folder bnm))
-	   (msg (mew-summary-message-number)))
-      (if (not (and fld (get-buffer fld)))
-	  (message "No physical folder")
-	(mew-summary-visit-folder fld nil 'no-ls)
-	(if msg
-	    (mew-summary-move-and-display msg)
-	  (goto-char (point-max)))
-	(message "Now in %s. Type '%s' again" fld key)))))
+  (let ((msg (mew-summary-message-number))
+	(fld (mew-summary-physical-folder)))
+    (if (not (and fld (get-buffer fld)))
+	(message "No physical folder")
+      (mew-summary-visit-folder fld nil 'no-ls)
+      (if msg
+	  (mew-summary-move-and-display msg)
+	(goto-char (point-max)))
+      (message "Now in %s. Type '%s' again" fld key))))
 
 (defun mew-summary-exec-region (beg end &optional func)
   (let ((folder (mew-sinfo-get-folder)))
@@ -297,7 +292,7 @@ destination is the same as that of the current message are processed."
      ((mew-folder-remotep folder)
       (message "Refiling and deleting...")
       (force-mode-line-update)
-      (mew-summary-exec-remote beg end))
+      (mew-summary-exec-remote beg end func))
      ((mew-virtual-p)
       (mew-summary-go-back-summary
        (mew-substitute-for-summary "\\[mew-summary-exec]")))
@@ -341,11 +336,10 @@ destination is the same as that of the current message are processed."
 	  (let* ((marks (mew-mark-get-all-marks))
 		 (src (mew-summary-folder-name 'ext))
 		 (lbuf (mew-refile-log-buffer src))
-		 (m (make-marker))
+		 (m (copy-marker end))
 		 (cnt 0)
 		 (case-fold-search nil)
 		 msg msgs err-folders mark func-exec func-sanity killp regex)
-	    (set-marker m end)
 	    (while marks
 	      (setq mark (car marks))
 	      (setq marks (cdr marks))
@@ -410,9 +404,12 @@ destination is the same as that of the current message are processed."
     (beginning-of-line)
     (setq start (point))
     (forward-line)
-    (mew-elet (put-text-property start (point) 'invisible t))))
+    (mew-elet
+     (put-text-property start (point) 'invisible t)
+     ;; For Emacs only. To make invisible text act as we want
+     (put-text-property start (point) 'front-sticky t))))
 
-(defun mew-summary-exec-remote-get-info (beg end modify)
+(defun mew-summary-exec-remote-get-info (beg end modify &optional func-decide)
   ;; for POP or IMAP
   ;; If modify, marked lines get invisible. They are removed
   ;; IMAP's sentinel.
@@ -420,10 +417,12 @@ destination is the same as that of the current message are processed."
   ;; by mew-summary-exec-local().
   (let* ((case (mew-sinfo-get-case))
 	 (folder (mew-sinfo-get-folder))
-	 (m (make-marker))
+	 (m (copy-marker end))
+	 (case-fold-search nil)
+	 (rttl 0) (dttl 0)
 	 trash trash-list lomov
 	 siz msg uid ref flds del regex trss rmvs kils refs movs
-	 refinfo refile-back mark rttl dttl)
+	 refinfo refile-back)
     (when (mew-folder-imapp folder)
       ;; see also mew-mark-exec-delete.
       (setq trash (mew-imap-trash-folder case))
@@ -440,63 +439,85 @@ destination is the same as that of the current message are processed."
 	(if (string= folder trash)
 	    (setq trash nil)))))
     (save-excursion
-      (set-marker m end)
       (mew-decode-syntax-delete)
-      (setq regex (mew-mark-regex mew-mark-refile))
-      (goto-char beg)
-      (while (re-search-forward regex m t)
-	(re-search-forward mew-regex-sumsyn-long)
-	(setq msg (mew-sumsyn-message-number))
-	(setq uid (mew-sumsyn-message-uid))
-	(setq siz (mew-sumsyn-message-size))
-	(setq refinfo (mew-refile-get msg))
-	(setq refile-back (cons refinfo refile-back))
-	(setq flds (cdr refinfo))
-	;;
-	(setq del nil)
-	(if (member folder flds)
-	    (setq flds (delete folder flds)) ;; destructive
-	  (setq del t)
-	  (when modify
-	    (if lomov
-		(setq movs (cons msg movs))
-	      (setq kils (cons msg kils)))))
-	(when flds
-	  (setq ref (cons uid (cons siz (cons del flds))))
-	  (setq refs (cons ref refs)))
-	(if modify
-	    (if del
-		(mew-mark-invisible-line)
-	      (mew-summary-refile-remove-body)
-	      (mew-mark-remove))))
-      (setq refs (nreverse refs))
-      (setq rttl (length refs))
-      (setq movs (nreverse movs))
-      (if modify (mew-sinfo-set-refile-back refile-back))
-      (when lomov
-	(let ((mew-inherit-exec-case case))
-	  (mew-mark-sanity-refile movs)))
       ;;
-      (setq regex (mew-mark-list-regex (list mew-mark-delete mew-mark-unlink)))
-      (goto-char beg)
-      (setq dttl 0)
-      (while (re-search-forward regex m t)
-	(setq mark (match-string 0))
-	(re-search-forward mew-regex-sumsyn-long)
-	(setq msg (mew-sumsyn-message-number))
-	(setq uid (mew-sumsyn-message-uid))
-	(setq siz (mew-sumsyn-message-size))
-	(if (and trash (char-equal (string-to-char mark) mew-mark-delete))
-	    (setq trss (cons (list uid siz t trash) trss))
-	  (setq rmvs (cons uid rmvs)))
-	(when modify
-	  (setq kils (cons msg kils))
-	  (mew-mark-invisible-line))
-	(setq dttl (1+ dttl)))
+      ;; refile
+      ;;
+      (when (mew-markdb-by-mark mew-mark-refile)
+	(setq regex (mew-mark-regex mew-mark-refile))
+	(goto-char beg)
+	(while (re-search-forward regex m t)
+	  (re-search-forward mew-regex-sumsyn-long)
+	  (setq msg (mew-sumsyn-message-number))
+	  (setq uid (mew-sumsyn-message-uid))
+	  (setq siz (mew-sumsyn-message-size))
+	  (when (or (not func-decide) (funcall func-decide msg))	
+	    (setq refinfo (mew-refile-get msg))
+	    (setq refile-back (cons refinfo refile-back))
+	    (setq flds (cdr refinfo))
+	    ;;
+	    (setq del nil)
+	    (if (member folder flds)
+		(setq flds (delete folder flds)) ;; destructive
+	      (setq del t)
+	      (when modify
+		(if lomov
+		    (setq movs (cons msg movs))
+		  (setq kils (cons msg kils)))))
+	    (when flds
+	      (setq ref (cons uid (cons siz (cons del flds))))
+	      (setq refs (cons ref refs)))
+	    (if modify
+		(if del
+		    (mew-mark-invisible-line)
+		  (mew-summary-refile-remove-body)
+		  (mew-mark-remove)))))
+	(setq refs (nreverse refs))
+	(setq rttl (length refs))
+	(setq movs (nreverse movs))
+	(if modify (mew-sinfo-set-refile-back refile-back))
+	(when lomov
+	  (let ((mew-inherit-exec-case case))
+	    (mew-mark-sanity-refile movs))))
+      ;;
+      ;; delete
+      ;;
+      (when (mew-markdb-by-mark mew-mark-delete)
+	(setq regex (mew-mark-regex mew-mark-delete))
+	(goto-char beg)
+	(while (re-search-forward regex m t)
+	  (re-search-forward mew-regex-sumsyn-long)
+	  (setq msg (mew-sumsyn-message-number))
+	  (setq uid (mew-sumsyn-message-uid))
+	  (setq siz (mew-sumsyn-message-size))
+	  (if trash
+	      (setq trss (cons (list uid siz t trash) trss))
+	    (setq rmvs (cons uid rmvs)))
+	  (when modify
+	    (setq kils (cons msg kils))
+	    (mew-mark-invisible-line))
+	  (setq dttl (1+ dttl))))
+      ;;
+      ;; unlink
+      ;;
+      (when (mew-markdb-by-mark mew-mark-unlink)
+	(setq regex (mew-mark-regex mew-mark-unlink))
+	(goto-char beg)
+	(while (re-search-forward regex m t)
+	  (re-search-forward mew-regex-sumsyn-long)
+	  (setq msg (mew-sumsyn-message-number))
+	  (setq uid (mew-sumsyn-message-uid))
+	  (setq siz (mew-sumsyn-message-size))
+	  (setq rmvs (cons uid rmvs))
+	  (when modify
+	    (setq kils (cons msg kils))
+	    (mew-mark-invisible-line))
+	  (setq dttl (1+ dttl))))
+      ;;
       (setq rmvs (nreverse rmvs))
       (setq trss (nreverse trss))
-      (when trss
-	(setq refs (nconc trss refs)))
+      (if trss (setq refs (nconc trss refs)))
+      ;;
       (set-buffer-modified-p nil)
       (cond
        ((and (null rmvs) (null refs)) nil)
@@ -506,7 +527,7 @@ destination is the same as that of the current message are processed."
        (t
 	(list rmvs nil refs nil 0 0))))))
 
-(defun mew-summary-exec-remote (beg end)
+(defun mew-summary-exec-remote (beg end &optional func-decide)
   (if (mew-folder-nntpp (mew-sinfo-get-folder))
       (message "This command cannot be used in this folder")
     (when (mew-summary-exclusive-p)
@@ -514,10 +535,12 @@ destination is the same as that of the current message are processed."
 	     (folder (mew-sinfo-get-folder))
 	     (bnm (mew-summary-folder-name 'ext))
 	     (lbuf (mew-refile-log-buffer bnm))
-	     (info (mew-summary-exec-remote-get-info beg end 'modify)))
+	     (info (mew-summary-exec-remote-get-info beg end 'modify func-decide)))
 	(if (not info)
 	    (message "No messages to be processed")
 	  (mew-summary-reset)
+	  (while (eq (get-text-property (point) 'invisible) t)
+	    (forward-line))
 	  (cond
 	   ((mew-folder-popp folder)
 	    (mew-pop-retrieve case 'exec bnm info))
@@ -541,9 +564,8 @@ destination is the same as that of the current message are processed."
 (defun mew-summary-exec-offline (&optional arg)
   "\\<mew-summary-mode-map>Process marked messages *offline*. Messages to be refiled in 
 a remote folder are moved to the corresponding folder but they are
-marked invalid. A number of an invalid message starts with '0'.
-Invalid messages will be overridden when the remote folder
-is scanned online. 
+marked invalid. Invalid message are marked with '#'.
+Invalid messages will be overridden when the remote folder is scanned online. 
 A job to delete or refile messages, which is created by this command, 
 is queued in a queue folder (%queue for IMAP). To flush jobs in 
 the queue, type '\\[mew-summary-send-message]' in the queue online."
@@ -639,7 +661,7 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2003 Mew developing team.
+;; Copyright (C) 1997-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
