@@ -44,6 +44,15 @@
       (mew-thread-p)
       (mew-virtual-for-one-summary)))
 
+(defsubst mew-pickable-folder ()
+  (cond
+   ((mew-summary-p)
+    (mew-summary-folder-name 'ext))
+   ((mew-thread-p)
+    (mew-vinfo-get-parent-folder))
+   ((mew-virtual-for-one-summary)
+    (car (mew-vinfo-get-flds)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; List functions
@@ -356,7 +365,7 @@ Words are separated by '/' and '-'."
       (insert str))))
 
 (defsubst mew-insert-message (fld msg rcs size)
-  (let ((file (mew-expand-folder fld msg)))
+  (let ((file (mew-expand-msg fld msg)))
     (cond
      ((not (file-readable-p file))
       (error "%s does not exist" (mew-concat-folder fld msg)))
@@ -396,33 +405,49 @@ Words are separated by '/' and '-'."
 	(setq ret (cons (substring str start) ret)))
     (nreverse ret)))
 
-(defun mew-split-quoted (str sepchar &optional qopen qclose)
+(defun mew-remove-single-quote (str)
+  (let* ((len (length str))
+	 (ret (make-string len mew-sp))
+	 (i 0) (j 0))
+    (while (< i len)
+      (unless (char-equal (aref str i) ?')
+	(aset ret j (aref str i))
+	(setq j (1+ j)))
+      (setq i (1+ i)))
+    (substring ret 0 j)))
+
+(defun mew-split-quoted (str sepchar &optional qopen qclose no-single)
   "Return a list of strings splitting STR with SEPCHAR.
 SEPCHARs in double-quoted strings are ignored.
 If QUOTEDCHAR is provided, SEPCHARs between QOPEN and QCLOSE are
 also ignored."
- (let ((qlevel 0) (len (length str)) (start 0) (i 0) dblq ret c)
-   (if (and qopen (not qclose)) (setq qclose qopen))
-   (while (< i len)
-     (setq c (aref str i))
-     (cond 
-      ((char-equal ?\\ c)
-       (setq i (1+ i)))
-      ((char-equal ?\" c)
-       (setq dblq (not dblq)))
-      ((and qopen (char-equal c qopen))
-       (setq qlevel (1+ qlevel)))
-      ((and qclose (char-equal c qclose))
-       (setq qlevel (1- qlevel)))
-      ((char-equal c sepchar)
-       (if (or dblq (>= qlevel 1))
-	   ()
-	 (setq ret (cons (substring str start i) ret))
-	 (setq start (1+ i)))))
-     (setq i (1+ i)))
-   (if (/= start len)
-       (setq ret (cons (substring str start) ret)))
-   (nreverse ret)))
+  (let ((qlevel 0) (len (length str)) (start 0) (i 0) dblq sub ret c)
+    (if (and qopen (not qclose)) (setq qclose qopen))
+    (while (< i len)
+      (setq c (aref str i))
+      (cond 
+       ((char-equal ?\\ c)
+	(setq i (1+ i)))
+       ((or (char-equal ?\" c) (and (not no-single) (char-equal ?' c)))
+	(setq dblq (not dblq)))
+       ((and qopen (char-equal c qopen))
+	(setq qlevel (1+ qlevel)))
+       ((and qclose (char-equal c qclose))
+	(setq qlevel (1- qlevel)))
+       ((char-equal c sepchar)
+	(unless (or dblq (>= qlevel 1))
+	  (setq sub (substring str start i))
+	  (unless no-single
+	    (setq sub (mew-remove-single-quote sub)))
+	  (setq ret (cons sub ret))
+	  (setq start (1+ i)))))
+      (setq i (1+ i)))
+    (when (/= start len)
+      (setq sub (substring str start))
+      (unless no-single
+	(setq sub (mew-remove-single-quote sub)))
+      (setq ret (cons sub ret)))
+    (nreverse ret)))
 
 (defun mew-chop (str)
   "Split off preceding and following white spaces."
@@ -613,7 +638,7 @@ If case is \"default\", it is not prepended."
      (t
       (if (null case)
 	  (setq folder case:folder)
-	(if (string-equal case mew-case-default)
+	(if (string= case mew-case-default)
 	    (progn
 	      (setq case nil)
 	      (setq case:folder folder))))
@@ -650,7 +675,7 @@ If case is \"default\", it is not prepended."
 			  (mew-imap-utf-7-encode-string subdir))))
      (t folder))))
 
-(defun mew-expand-folder (folder &optional message)
+(defun mew-expand-folder (folder)
   "Expanding FOLDER to its absolute path"
   (when (stringp folder)
     (let (dir)
@@ -660,9 +685,35 @@ If case is \"default\", it is not prepended."
 	;; absolute path
 	;; "C:/" -> t, "C:" -> nil , "CC:/" -> nil
 	(setq dir (expand-file-name folder)))
-      (if message
-	  (expand-file-name message dir)
-	dir))))
+      dir)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Suffix support
+;;;
+
+(defvar mew-use-suffix nil)
+
+(defun mew-msg-new-filename (file)
+  (if mew-use-suffix
+      (format "%s%s" file mew-suffix)
+    file))
+
+(defun mew-msg-get-filename (file)
+  (let ((sfile (format "%s%s" file mew-suffix)))
+    (if (file-exists-p sfile)
+	sfile
+      file)))
+
+(defun mew-expand-msg (folder message)
+  (let ((file (expand-file-name message (mew-expand-folder folder))))
+    (mew-msg-get-filename file)))
+
+(defun mew-expand-new-msg (folder message)
+  (mew-msg-new-filename (expand-file-name message (mew-expand-folder folder))))
+
+(defun mew-expand-file (folder file)
+  (expand-file-name file (mew-expand-folder folder)))
 
 (defun mew-concat-folder (folder subfolder)
   (concat (file-name-as-directory folder) subfolder))
@@ -672,7 +723,7 @@ If case is \"default\", it is not prepended."
       (concat (mew-imap-file-name-as-directory folder case) subfolder)
     (concat (file-name-as-directory folder) subfolder)))
 
-(defsubst mew-dir-messages (dir &optional regex)
+(defsubst mew-dir-messages (dir &optional regex full)
   ;; directory_files uses ENCODE_FILE() for DIR and
   ;;                      DECODE_FILE() for results(FILES).
   ;; Both ENCODE_FILE() and DECODE_FILE() are quite slow.
@@ -690,7 +741,7 @@ If case is \"default\", it is not prepended."
     (setq edir (mew-cs-encode-string edir cs))
     (or regex (setq regex mew-regex-message-files))
     (mew-alet
-     (directory-files edir nil regex 'no-sort))))
+     (directory-files edir full regex 'no-sort))))
 
 (defun mew-folder-messages (folder)
   (let* ((dir (mew-expand-folder folder))
@@ -709,12 +760,12 @@ If case is \"default\", it is not prepended."
     (when (file-directory-p dir)
       (setq msgs (mew-dir-messages dir regex))
       (while msgs
-	(setq cur (string-to-number (car msgs)))
+	(setq cur (string-to-number (file-name-sans-extension (car msgs))))
 	(if (> cur max) (setq max cur))
 	(setq msgs (cdr msgs)))
       (setq max (1+ max))
       (setq maxfile (number-to-string max))
-      (setq maxpath (mew-expand-folder folder maxfile))
+      (setq maxpath (mew-expand-new-msg folder maxfile))
       (while (file-exists-p maxpath) ;; xxx
 	;; If NFS is used, readdir() may fail. Emacs 20 does not
 	;; rescan the directory again. So, we need to rescan in
@@ -723,29 +774,29 @@ If case is \"default\", it is not prepended."
 	(setq msgs (mew-dir-messages dir regex))
 	(setq max 0)
 	(while msgs
-	  (setq cur (string-to-number (car msgs)))
+	  (setq cur (string-to-number (file-name-sans-extension (car msgs))))
 	  (if (> cur max) (setq max cur))
 	  (setq msgs (cdr msgs)))
 	(setq max (1+ max))
 	(setq maxfile (number-to-string max))
-	(setq maxpath (mew-expand-folder folder maxfile)))
+	(setq maxpath (mew-expand-new-msg folder maxfile)))
       (while (get-file-buffer maxpath) ;; xxx
 	;; file not exist but there is a buffer.
 	(setq max (1+ max))
 	(setq maxfile (number-to-string max))
-	(setq maxpath (mew-expand-folder folder maxfile)))
+	(setq maxpath (mew-expand-new-msg folder maxfile)))
       (while (file-exists-p maxpath)
 	(setq maxfile (read-string (format "%s/%s exists. Input a message number: " max folder)))
 	(while (not (string-match mew-regex-message-files maxfile))
 	  (setq maxfile (read-string "Input NUMBER: ")))
-	(setq maxpath (mew-expand-folder folder maxfile)))
+	(setq maxpath (mew-expand-new-msg folder maxfile)))
       (if num-only
-	  maxfile
+	  (mew-msg-new-filename maxfile)
 	maxpath))))
 
 (defun mew-touch-folder (fld)
   (when (stringp mew-summary-touch-file)
-    (let ((file (mew-expand-folder fld mew-summary-touch-file)))
+    (let ((file (mew-expand-file fld mew-summary-touch-file)))
       (when (file-writable-p file)
 	(write-region "touched by Mew." nil file nil 'no-msg)
 	(mew-set-file-modes file)))))
@@ -1609,7 +1660,7 @@ by side-effect."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2005 Mew developing team.
+;; Copyright (C) 1997-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
