@@ -134,22 +134,185 @@
 ;;; Image
 ;;;
 
+(defcustom mew-image-display-resize t
+  "*If non-nil, image will be displayed with fitting to frame size"
+  :type 'boolean)
+
+(defcustom mew-image-display-resize-care-height t
+  "*If nil, image will be displayed with fitting to only frame width"
+  :type 'boolean)
+
+(defcustom mew-image-width-margin 45
+  "*A value for width margin to display image when resizing"
+  :type 'integer)
+
+(defcustom mew-image-height-margin 200
+  "A value for height margin to display image when resizing"
+  :type 'integer)
+
+(defvar mew-image-l-endian ?\x49)
+(defvar mew-image-b-endian ?\x4d)
+
 (defun mew-image-inline-p (format)
   ;; display-graphic-p
   (and window-system (image-type-available-p format)))
 
+(defun mew-img-get-n (op len)
+  (let* ((n 0) (size 0))
+    (if (eq op ?\x49)   ; I(?\x49) or M(?\x4d)
+	(while (< n len)
+	  (setq size (+ size (* (char-after) (expt ?\x100 n))))
+	  (forward-char)
+	  (setq n (+ n 1)))
+	(while (< 0 len)
+	  (setq len (- len 1))
+	  (setq size (+ size (* (char-after) (expt ?\x100 len))))
+	  (forward-char)))
+    size))
+
+(defun mew-jpeg-size ()
+  (let (c size width height)
+    (save-excursion
+      (catch 'loop
+	(while t
+	  (setq c (char-after))
+	  (forward-char)
+	  (unless (eq c ?\xff)
+	    (throw 'loop nil))
+	  (setq c (char-after))
+	  (forward-char)
+	  (cond
+	   ((or (eq c ?\xd8)                      ;; SOI
+		(eq c ?\xd9)                      ;; EOI
+		(and (>= c ?\xd0) (<= c ?\xd7)))) ;; RSTm
+	   ((eq c ?\xc0)
+	    (forward-char 3)
+	    (setq height (mew-img-get-n mew-image-b-endian 2))
+	    (setq width (mew-img-get-n mew-image-b-endian 2))
+	    (backward-char)
+	    (throw 'loop nil))
+	   ((and (>= c ?\xc1) (<= c ?\xfe))
+	    (setq size (mew-img-get-n mew-image-b-endian 2))
+	    (forward-char (- size 2)))
+	   (t
+	    (throw 'loop nil))))))
+    (cons width height)))
+
+(defun mew-png-size ()
+  (let (width height)
+    (save-excursion
+      (forward-char 18)
+      ;; length is four bytes
+      ;; but we takes lower two bytes
+      (setq width (mew-img-get-n mew-image-b-endian 2))
+      (forward-char 2)
+      (setq height (mew-img-get-n mew-image-b-endian 2))
+      (cons width height))))
+
+(defun mew-gif-size ()
+  (let (width height)
+    (save-excursion
+      (forward-char 6)
+      (setq width (mew-img-get-n mew-image-l-endian 2))
+      (setq height (mew-img-get-n mew-image-l-endian 2))
+      (cons width height))))
+
+(defun mew-tiff-size ()
+  (let (endian size entry tag width height)
+    (save-excursion
+      (setq endian (char-after))
+      (forward-char 4)
+      (setq size (mew-img-get-n endian 4))
+      (forward-char (- size 8)) ;; jump to IFD
+      ;; IFD
+      (setq entry (mew-img-get-n endian 2))
+      (catch 'loop
+	(while t
+	  (setq entry (- entry 1))
+	  (setq tag (mew-img-get-n endian 2))
+	  (cond
+	   ((eq tag 256)
+	    (forward-char 6)
+	    (setq width (mew-img-get-n endian 2))
+	    (forward-char 2))
+	   ((eq tag 257)
+	    (forward-char 6)
+	    (setq height (mew-img-get-n endian 2))
+	    (forward-char 2))
+	   (t
+	    (forward-char 10)))
+	  (unless (< 1 entry)
+	    (throw 'loop nil)))))
+    (cons width height)))
+
+(defvar mew-image-alist
+  '((jpeg mew-jpeg-size "jpegtopnm" "pnmtojpeg")
+    (png  mew-png-size  "pngtopnm"  "pnmtopng")
+    (gif  mew-gif-size  "giftopnm"  "pnmtogif")
+    (tiff mew-tiff-size "tifftopnm" "pnmtotiff")))
+
+(defun mew-image-format-ent (format)
+  (assoc format mew-image-alist))
+
+(defun mew-image-get-func (ent)
+  (nth 1 ent))
+
+(defun mew-image-get-prog (ent)
+  (nth 2 ent))
+
+(defun mew-image-get-prog2 (ent)
+  (nth 3 ent))
+
 (defun mew-mime-image (cache begin end format)
   (message "Loading image...")
   (set-buffer (mew-buffer-message))
-  (mew-elet
-   (condition-case nil
-       (insert-image
-	(create-image (save-excursion
-			(set-buffer cache)
-			(string-as-unibyte
-			 (mew-buffer-substring begin end)))
-		      format t))
-     (error ())))
+  (let* ((width (- (frame-pixel-width (selected-frame)) mew-image-width-margin))
+	 (height (- (frame-pixel-height (selected-frame)) mew-image-height-margin))
+	 (ent (mew-image-format-ent format))
+	 (prog (mew-image-get-prog ent))
+	 (prog2 (mew-image-get-prog2 ent))
+	 (func-size (mew-image-get-func ent))
+	 image-width image-height image)
+    (when (and mew-image-display-resize func-size (fboundp func-size))
+      (save-excursion
+	(set-buffer cache)
+	(goto-char begin)
+	(setq image-width (car (funcall func-size)))
+	(setq image-height (cdr (funcall func-size)))))
+    (if (and image-width image-height
+	     (or (< width image-width)
+		 (and mew-image-display-resize-care-height (< height image-height)))
+	     (mew-which-exec prog))
+	(with-temp-buffer
+	  (mew-plet
+	   (message "Resizing image...")
+	   (insert-buffer-substring cache begin end)
+	   (mew-set-buffer-multibyte nil)
+	   (call-process-region (point-min) (point-max) prog
+				t '(t nil) nil)
+	   (if mew-image-display-resize-care-height
+	       (call-process-region (point-min) (point-max) "pnmscale"
+				    t '(t nil) nil
+				    "-xysize"
+				    (format "%d" width)
+				    (format "%d" height))
+	     (call-process-region (point-min) (point-max) "pnmscale"
+				  t '(t nil) nil
+				  "-xsize" (format "%d" width)))
+	   (if (and (string< emacs-version "22")
+		    (mew-which-exec prog2))
+	       (call-process-region (point-min) (point-max) prog2
+				    t '(t nil) nil)
+	     (setq format 'pbm))
+	   (setq image (mew-buffer-substring (point-min) (point-max)))
+	   (message "Resizing image...done")))
+      (save-excursion
+	(set-buffer cache)
+	(setq image (mew-buffer-substring begin end))))
+    (mew-elet
+     (condition-case nil
+	 (insert-image (create-image (string-as-unibyte image) format t))
+       (error ()))))
   (goto-char (point-min))
   (message "Loading image...done"))
 
@@ -217,19 +380,14 @@
 ;;; Click hack
 ;;;
 
-(defun mew-browse-url-at-mouse (event)
-  (interactive "e")
-  (let ((buf (current-buffer)))
-    (browse-url-at-mouse event)
-    (pop-to-buffer buf)))
-
 (define-key mew-message-mode-map [mouse-2] 'mew-browse-url-at-mouse)
+(define-key mew-message-mode-map [M-down-mouse-1] 'mew-browse-url-at-mouse) ;; for Mac
 
 (provide 'mew-gemacs)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2005 Mew developing team.
+;; Copyright (C) 1997-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
