@@ -38,8 +38,13 @@
 		(save-excursion (beginning-of-line) (point)))))
 
 (defun mew-unhighlight-cursor-line ()
-  (if (overlayp (mew-sinfo-get-cursor-line))
-      (move-overlay (mew-sinfo-get-cursor-line) 1 1)))
+  (when (overlayp (mew-sinfo-get-cursor-line))
+    (move-overlay (mew-sinfo-get-cursor-line) 1 1)
+    ;; (goto-char (point-max)) works wrong if underline exists.
+    ;; Redisplays it so that it works well.
+    ;; (sit-for 0)
+    ;; This code rocks Summary mode when typing "x", sigh.
+    ))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -77,7 +82,7 @@
 (defun mew-highlight-header-region (BEG END)
   "A function to highlight header in Message and Draft mode."
   (when mew-use-highlight-header
-    (let (key beg med nspec overlay key-face val-face)
+    (let (key beg med nspec overlay key-face val-face start)
       (save-excursion
 	(mew-elet
 	 (mew-unhighlight-region BEG END)
@@ -87,6 +92,7 @@
 	   (while (not (eobp))
 	     (if (not (looking-at mew-keyval))
 		 (forward-line)
+	       (setq start (point))
 	       (setq key (mew-match-string 1))
 	       (setq beg (match-beginning 0))
 	       (setq med (match-end 0))
@@ -103,8 +109,38 @@
 	       (while (looking-at mew-lwsp+)
 		 (forward-line)
 		 (setq overlay (mew-overlay-make (match-end 0) (1- (point))))
-		 (overlay-put overlay 'face val-face))))))))))
-	       
+		 (overlay-put overlay 'face val-face))
+	       (if (member (mew-capitalize key)
+			   mew-draft-address-warning-fields)
+		   (mew-highlight-header-address-region start (point)))))))))))
+
+(defun mew-highlight-header-address-region (beg end)
+  (let ((case (mew-tinfo-get-case))
+	addr domain overlay begovl endovl warnp)
+    (save-excursion
+      (goto-char beg)
+      (while (re-search-forward "[^\n\t, <>:;]+@\\([^\n\t, <>:;]+\\)" end t)
+	(setq addr (downcase (mew-match-string 0)))
+	(setq domain (downcase (mew-match-string 1)))
+	(setq begovl (match-beginning 0))
+	(setq endovl (match-end 0))
+	(cond
+	 ((member addr (mew-safe-addresses case))
+	  (setq warnp nil))
+	 ((member addr (mew-warn-addresses case))
+	  (setq warnp t))
+	 ((member domain (mew-safe-domains case))
+	  (setq warnp nil))
+	 ((member domain (mew-warn-domains case))
+	  (setq warnp t))
+	 ((or (mew-safe-addresses case) (mew-safe-domains case))
+	  (setq warnp t))
+	 (t
+	  (setq warnp nil)))
+	(when warnp
+	  (setq overlay (mew-overlay-make begovl endovl))
+	  (overlay-put overlay 'face 'mew-face-header-warning)
+	  (overlay-put overlay 'priority 2)))))) ;; a bug of Meadow
 
 (defun mew-highlight-header ()
   (save-restriction
@@ -129,7 +165,7 @@
 	   (url-regex mew-regex-url)
 	   (fancy-num 0)
 	   (fancy-length (length mew-highlight-body-cite-faces))
-	   beg end face fancy-alst fancy-prefix)
+	   beg end face fancy-alst fancy-prefix idn url)
       (save-excursion
 	(mew-elet
 	 (when draft
@@ -164,11 +200,23 @@
 	 (when mew-use-highlight-url
 	   (goto-char BEG)
 	   (while (and (<= (point) END) (re-search-forward url-regex END t))
-	     (setq beg (match-beginning 0))
-	     (setq end (match-end 0))
-	     (put-text-property beg end 'face 'mew-face-body-url)
-	     (put-text-property
-	      beg end 'mouse-face mew-highlight-url-mouse-face))))))))
+	     (save-restriction
+	       (setq beg (match-beginning 0))
+	       (setq end (match-end 0))
+	       (narrow-to-region beg end)
+	       (goto-char (point-min))
+	       (setq url (mew-buffer-substring (point-min) (point-max)))
+	       (while (and mew-use-punycode (re-search-forward mew-regex-punycode nil t))
+		 (setq idn (mew-puny-decode (mew-buffer-substring (match-beginning 0) (match-end 0))))
+		 (when idn
+		   (delete-region (match-beginning 0) (match-end 0))
+		   (insert idn)))
+	       (goto-char (point-max))
+	       (put-text-property (point-min) (point-max) 'mew-url url)
+	       (put-text-property (point-min) (point-max) 'face 'mew-face-body-url)
+	       (put-text-property
+		(point-min) (point-max) 'mouse-face mew-highlight-url-mouse-face)
+	       (setq END (- END (- end (point-max))))))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -314,7 +362,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2005 Mew developing team.
+;; Copyright (C) 1997-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
