@@ -38,7 +38,7 @@
     "wrk" "jobs" "namespace" "authl"
     "spam" "spam-field" "spam-word" "spam-pattern" 
     "new-mailbox" "my-prefix" "specials" "bytes"
-    "thread-info" "status-buf"))
+    "thread-info" "disp-info" "status-buf"))
 
 (mew-info-defun "mew-imap-" mew-imap-info-list)
 
@@ -151,7 +151,7 @@
     (when (and (eq directive 'jobs) jobs)
       (let* ((case (mew-imap-get-case pnm))
 	     (bnm (mew-imap-get-bnm pnm))
-	     (file (mew-expand-folder bnm (car jobs)))
+	     (file (mew-expand-msg bnm (car jobs)))
 	     (work (concat file mew-queue-work-suffix))
 	     (data (concat file mew-imapq-info-suffix))
 	     (data-work (concat data mew-queue-work-suffix))
@@ -210,11 +210,11 @@
 		      "Checking" nil (mew-imap-secure-p pnm))
       (cond
        ((eq directive 'biff)
-	(setq max (mew-lisp-load (mew-expand-folder bnm mew-imap-msgid-file))))
+	(setq max (mew-lisp-load (mew-expand-file bnm mew-imap-msgid-file))))
        ((eq directive 'scan)
 	(if (eq range nil) ;; update
 	    (setq max (mew-lisp-load
-		       (mew-expand-folder bnm mew-imap-msgid-file))))))
+		       (mew-expand-file bnm mew-imap-msgid-file))))))
       (if max
 	  (setq max (string-to-number max))
 	(setq max 0))
@@ -420,9 +420,11 @@
     (mew-imap-process-send-string pro pnm "CREATE \"%s\"" dst)))
 
 (defun mew-imap-command-wmbx (pro pnm)
-  (mew-imap-set-error pnm "Mailbox name is wrong!")
-  (mew-imap-set-status pnm "logout2")
-  (mew-imap-command-logout2 pro pnm))
+  (let* ((rtr (mew-imap-get-rfl pnm))
+	 (dst (nth 0 rtr)))
+    (mew-imap-set-error pnm (format "Cannot create '%s' or copy to it" dst))
+    (mew-imap-set-status pnm "logout2")
+    (mew-imap-command-logout2 pro pnm)))
 
 (defun mew-imap-command-pre-dels (pro pnm)
   (let ((dtt (length (mew-imap-get-dels pnm)))
@@ -560,17 +562,19 @@
 	(mew-imap-command-logout pro pnm)))
      ((or (eq directive 'exec) (eq directive 'jobs) (eq directive 'get))
       (mew-imap-set-truncated pnm nil)
+      ;; RFC822 turns the Seen flag "on"
       (mew-imap-process-send-string pro pnm "UID FETCH %s RFC822" uid))
      ((and (eq directive 'scan) (not get-body))
       (mew-imap-set-truncated pnm t)
-      (mew-imap-process-send-string pro pnm "UID FETCH %s RFC822.HEADER" uid))
+      ;; RFC822 keeps the Seen flag "off"
+      (mew-imap-process-send-string pro pnm "UID FETCH %s BODY.PEEK[HEADER]" uid))
      ((or (= lim 0) (< (string-to-number siz) lim))
       (mew-imap-set-truncated pnm nil)
       (mew-imap-process-send-string pro pnm "UID FETCH %s RFC822" uid))
      (t
       (mew-imap-set-truncated pnm t)
       (mew-imap-process-send-string
-       pro pnm "UID FETCH %s RFC822.HEADER" uid)))))
+       pro pnm "UID FETCH %s BODY.PEEK[HEADER]" uid)))))
 
 (defun mew-imap-command-post-fetch (pro pnm)
   (let* ((directive (mew-imap-get-directive pnm))
@@ -583,7 +587,7 @@
 	 (fld-msg (nth 3 rtr)) ;; 'exec, 'jobs, 'scan, ...
 	 (truncated (mew-imap-get-truncated pnm))
 	 (case (mew-imap-get-case pnm))
-	 fld msg vec file msg-file lmsg mark)
+	 fld msg vec file msg-file lmsg mark folder)
     (cond
      ((null fld-msg)
       (setq fld (mew-imap-get-bnm pnm)))
@@ -592,13 +596,14 @@
      ((listp fld-msg)
       (mew-set '(fld msg) fld-msg)
       (setq lmsg msg)))
+    (setq folder (mew-case:folder-folder fld))
     ;; line delimiters
     (goto-char (point-min))
     (mew-crlf-to-lf)
     (cond
      ((eq directive 'scan)
       (setq msg uid) 
-      (setq file (mew-expand-folder fld msg)))
+      (setq file (mew-expand-new-msg fld msg)))
      (t
       (setq msg-file (mew-net-get-new-message
 		      pnm fld msg 'mew-imap-get-msgdb 'mew-imap-set-msgdb))
@@ -611,30 +616,34 @@
 						mew-case-default))))
      ((eq directive 'scan)
       (mew-header-insert-xmu uid siz nil))
-     ((and (eq directive 'get) (mew-folder-imapp (mew-case:folder-folder fld)))
+     ((and (eq directive 'get) (mew-folder-imapp folder))
       (mew-header-insert-xmu uid siz nil)))
     (catch 'write-error
       (condition-case nil
-	  (mew-frwlet 
-	   mew-cs-dummy mew-cs-text-for-write
-	   (write-region (point-min) (point-max) file nil 'no-msg))
+	  (let ((write-region-inhibit-fsync mew-use-async-write))
+	    (mew-frwlet 
+	     mew-cs-dummy mew-cs-text-for-write
+	     (write-region (point-min) (point-max) file nil 'no-msg)))
 	(error
 	 (mew-imap-set-status pnm "logout")
 	 (mew-imap-command-logout pro pnm)
 	 (throw 'write-error nil)))
-      (mew-set-file-modes file)
       (mew-imap-set-rcnt pnm (1+ (mew-imap-get-rcnt pnm)))
-      ;;
-      (mew-set-buffer-multibyte t)
-      (setq vec (mew-scan-header))
-      (mew-scan-set-folder vec fld)
-      (mew-scan-set-message vec msg)
-      (if (and (eq directive 'scan)
-	       (string= (mew-case:folder-folder fld) mew-imap-inbox-folder))
-	  (setq mark (mew-scan-inbox-action vec case)))
-      (mew-scan-body vec)
-      (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line fld vec width lmsg mark)
+      (when (file-exists-p file)
+	(mew-set-file-modes file)
+	(unless (string= folder (mew-imap-trash-folder case))
+	  (mew-set-file-type file))
+	;;
+	(mew-set-buffer-multibyte t)
+	(setq vec (mew-scan-header))
+	(mew-scan-set-folder vec fld)
+	(mew-scan-set-message vec msg)
+	(if (and (eq directive 'scan)
+		 (string= folder mew-imap-inbox-folder))
+	    (setq mark (mew-scan-inbox-action vec case)))
+	(mew-scan-body vec)
+	(mew-set-buffer-multibyte nil)
+	(mew-scan-insert-line fld vec width lmsg mark))
       (mew-imap-set-rtrs pnm (cdr rtrs))
       (cond
        ((eq directive 'inc)
@@ -714,9 +723,11 @@
 		  "Listing"
 		  nil
 		  (mew-imap-secure-p pnm))
-  (let ((my-prefix (mew-imap-get-my-prefix pnm)))
+  (let* ((my-prefix (mew-imap-get-my-prefix pnm))
+	 (case (mew-imap-get-case pnm))
+	 (prefix-list (mew-imap-prefix-list case)))
     (mew-imap-message pnm "Collecting mailbox list...")
-    (mew-imap-process-send-string pro pnm "LIST \"%s\" *" (or my-prefix ""))))
+    (mew-imap-process-send-string pro pnm "LIST \"%s\" *" (or (and prefix-list my-prefix) ""))))
 
 (defun mew-imap-command-post-list (pro pnm)
   (let* ((case (mew-imap-get-case pnm))
@@ -1124,7 +1135,7 @@
 	pro tm)
     (condition-case emsg
 	(progn
-	  (setq tm (mew-timer mew-imap-timeout-time 'mew-imap-timeout))
+	  (setq tm (run-at-time mew-imap-timeout-time nil 'mew-imap-timeout))
 	  (or no-msg (message "Connecting to the IMAP server..."))
 	  (setq pro (open-network-stream pnm nil server sprt))
 	  (process-kill-without-query pro)
@@ -1154,6 +1165,8 @@
 	 (sshsrv (mew-imap-ssh-server case))
 	 (sslp (mew-imap-ssl case))
 	 (sslport (mew-imap-ssl-port case))
+	 (proxysrv (mew-imap-proxy-server case))
+	 (proxyport (mew-imap-proxy-port case))
 	 ;; dirty but necessary for migration
 	 (mailbox (unless (eq directive 'list)
 		    (mew-imap-utf-7-encode-string
@@ -1162,7 +1175,7 @@
 	 (buf (get-buffer-create (mew-imap-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
 	 process sshname sshpro sslname sslpro lport info jobs tls
-	 thread-info to-buf)
+	 thread-info disp-info virtual)
     (if (mew-imap-get-process pnm)
 	(message "Another IMAP process is running. Try later")
       (cond
@@ -1180,7 +1193,9 @@
  	  (setq sslname (process-name sslpro))
 	  (setq lport (mew-ssl-pnm-to-lport sslname))
  	  (when lport
- 	    (setq process (mew-imap-open pnm "localhost" lport no-msg)))))
+ 	    (setq process (mew-imap-open pnm mew-ssl-localhost lport no-msg)))))
+       (proxysrv
+	(setq process (mew-imap-open pnm proxysrv proxyport no-msg)))
        (t
 	(setq process (mew-imap-open pnm server port no-msg))))
       (if (null process)
@@ -1190,6 +1205,7 @@
 	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
 	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
+	(mew-sinfo-set-scan-md5 nil)
 	(mew-info-clean-up pnm)
 	(mew-imap-set-no-msg pnm no-msg) ;; must come here
 	(mew-imap-message pnm "Communicating with the IMAP server...")
@@ -1228,11 +1244,13 @@
 	  (mew-imap-set-refs pnm (nth 0 args))
 	  (setq thread-info (nth 1 args))
 	  (mew-imap-set-thread-info pnm thread-info)
-	  (setq to-buf (nth 0 thread-info))
-	  (when to-buf
-	    (mew-imap-set-status-buf pnm to-buf)
+	  (setq disp-info (nth 2 args))
+	  (mew-imap-set-disp-info pnm disp-info)
+	  (setq virtual (mew-net-thread-info-get-virtual thread-info))
+	  (when virtual
+	    (mew-imap-set-status-buf pnm virtual)
 	    (save-excursion
-	      (set-buffer to-buf)
+	      (set-buffer virtual)
 	      (mew-summary-lock process "IMAPing" (or sshpro sslpro)))))
 	 ((eq directive 'scan)
 	  (mew-imap-set-range pnm (nth 0 args))
@@ -1387,136 +1405,140 @@
 	 (msgid (mew-imap-get-msgid pnm))
 	 (done (mew-imap-get-done pnm))
 	 (error (mew-imap-get-error pnm))
-	 (file (mew-expand-folder bnm mew-imap-msgid-file))
+	 (file (mew-expand-file bnm mew-imap-msgid-file))
 	 (buf (process-buffer process))
-	 (thread-info (mew-imap-get-thread-info pnm)))
-    (mew-imap-debug "IMAP SENTINEL" event)
-    (set-process-buffer process nil)
-    (set-buffer bnm)
-    (mew-summary-mark-recover mdb)
-    (mew-remove-buffer buf)
-    (if (not done)
-	(let* ((rtrs (mew-imap-get-rtrs pnm))
-	       (lefts (length rtrs))
-	       (msgid (nth 0 (car rtrs)))
-	       leftp)
-	  (if error
-	      (mew-imap-message pnm error)
-	    (mew-imap-message pnm "IMAP connection is lost"))
-	  (cond
-	   ((eq directive 'exec)
-	    (mew-imap-exec-recover bnm))
-	   ((eq directive 'scan)
-	    (mew-summary-visible-buffer bnm))
-	   ((or (eq directive 'delete) (eq directive 'rename))
-	    (setq mew-summary-buffer-process-error t)))
-	  (when (mew-imap-get-dispatched pnm)
+	 (thread-info (mew-imap-get-thread-info pnm))
+	 (disp-info (mew-imap-get-disp-info pnm)))
+    (save-excursion
+      (mew-imap-debug "IMAP SENTINEL" event)
+      (set-process-buffer process nil)
+      (set-buffer bnm)
+      (mew-summary-mark-recover mdb)
+      (mew-remove-buffer buf)
+      (if (not done)
+	  (let* ((rtrs (mew-imap-get-rtrs pnm))
+		 (lefts (length rtrs))
+		 (msgid (nth 0 (car rtrs)))
+		 leftp)
+	    (if error
+		(mew-imap-message pnm error)
+	      (mew-imap-message pnm "IMAP connection is lost"))
 	    (cond
+	     ((eq directive 'exec)
+	      (mew-imap-exec-recover bnm))
 	     ((eq directive 'scan)
-	      (when msgid
-		(setq msgid (number-to-string (1- (string-to-number msgid))))
-		(mew-lisp-save file msgid nil 'unlimit))
-	      (setq leftp t))
-	     ((eq directive 'inc)
-	      ;; uidl is reversed.
-	      (setq msgid (assoc msgid uidl))
-	      (setq uidl (cdr (member msgid uidl)))
-	      (mew-net-uidl-db-set (mew-imap-passtag pnm) uidl)
-	      (setq leftp t)))
-	    (when leftp
-	      (mew-imap-message
-	       pnm
-	       "%d message retrieved. %d messages are left due to an error"
-	       (- rttl lefts) lefts)
-	      (mew-summary-folder-cache-save))))
-      (if thread-info (mew-summary-retrieve-message-for-thread thread-info))
-      (cond
-       ((eq directive 'list)
-	(mew-imap-message pnm "Collecting mailbox list...done"))
-       ((eq directive 'delete)
-	(mew-imap-message pnm "Deleting this folder...done"))
-       ((eq directive 'rename)
-	(mew-imap-message pnm "Renaming this folder...done"))
-       ((eq directive 'biff)
-	(funcall mew-biff-function rttl))
-       ((eq directive 'sync)
-	(mew-imap-message pnm "Synchronizing messages...")
-	(mew-net-folder-sync bnm hlds)
-	(mew-imap-message pnm "Synchronizing messages...done")
-	(mew-summary-folder-cache-save))
-       ((eq directive 'inc)
-	(mew-biff-clear)
-	(mew-net-uidl-db-set (mew-imap-passtag pnm) uidl)
+	      (mew-summary-visible-buffer bnm))
+	     ((or (eq directive 'delete) (eq directive 'rename))
+	      (setq mew-summary-buffer-process-error t)))
+	    (when (mew-imap-get-dispatched pnm)
+	      (cond
+	       ((eq directive 'scan)
+		(when msgid
+		  (setq msgid (number-to-string (1- (string-to-number msgid))))
+		  (mew-lisp-save file msgid nil 'unlimit))
+		(setq leftp t))
+	       ((eq directive 'inc)
+		;; uidl is reversed.
+		(setq msgid (assoc msgid uidl))
+		(setq uidl (cdr (member msgid uidl)))
+		(mew-net-uidl-db-set (mew-imap-passtag pnm) uidl)
+		(setq leftp t)))
+	      (when leftp
+		(mew-imap-message
+		 pnm
+		 "%d message retrieved. %d messages are left due to an error"
+		 (- rttl lefts) lefts)
+		(mew-summary-folder-cache-save))))
+	(if thread-info (mew-summary-retrieve-message-for-thread thread-info))
 	(cond
-	 ((= rttl 0)
-	  (mew-imap-message pnm "No new messages"))
-	 ((= rttl 1)
-	  (mew-imap-message pnm "1 message retrieved")
+	 ((eq directive 'list)
+	  (mew-imap-message pnm "Collecting mailbox list...done"))
+	 ((eq directive 'delete)
+	  (mew-imap-message pnm "Deleting this folder...done"))
+	 ((eq directive 'rename)
+	  (mew-imap-message pnm "Renaming this folder...done"))
+	 ((eq directive 'biff)
+	  (funcall mew-biff-function rttl))
+	 ((eq directive 'sync)
+	  (mew-imap-message pnm "Synchronizing messages...")
+	  (mew-net-folder-sync bnm hlds)
+	  (mew-imap-message pnm "Synchronizing messages...done")
 	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-imap-message pnm "%s messages retrieved" rttl)
-	  (mew-summary-folder-cache-save))))
-       ((eq directive 'get)
-	(cond
-	 ((= rttl 0)
-	  (mew-imap-message pnm "No new messages"))
-	 ((= rttl 1)
-	  (mew-imap-message pnm "1 message retrieved")
-	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-imap-message pnm "%s messages retrieved" rttl)
-	  (mew-summary-folder-cache-save))))
-       ((eq directive 'scan)
-	(mew-biff-clear)
-	(cond
-	 ((= rttl 0)
-	  (mew-imap-message pnm "No new messages"))
-	 ((= rttl 1)
-	  (mew-imap-message pnm "1 message retrieved")
-	  (mew-lisp-save file msgid nil 'unlimit)
-	  (mew-net-invalid-cache-clean))
-	 (t
-	  (mew-imap-message pnm "%s messages retrieved" rttl)
-	  (mew-lisp-save file msgid nil 'unlimit)
-	  (mew-net-invalid-cache-clean))))
-       ((eq directive 'exec)
-	(if kils (mew-mark-exec-unlink bnm kils))
-	(when movs
-	  (let ((mew-inherit-exec-case case)
-		(mew-inherit-offline t))
-	    (mew-mark-exec-refile bnm movs)))
-	(when (or kils movs)
-	  (mew-mark-kill-invisible)
-	  (mew-summary-folder-cache-save))
-	(cond
-	 ((= rttl 1)
-	  (mew-imap-message pnm "1 message refiled"))
-	 ((> rttl 1)
-	  (mew-imap-message pnm "%d messages refiled" rttl))
-	 ((= dttl -1)
-	  (mew-imap-message pnm "All messages deleted"))
-	 ((= dttl 1)
-	  (mew-imap-message pnm "1 message deleted"))
-	 (t
-	  (mew-imap-message pnm "%d messages deleted" dttl))))
-       ((eq directive 'jobs)
-	(if (= jcnt 1)
-	    (mew-imap-message pnm "1 job processed")
-	  (mew-imap-message pnm "%d jobs processed" jcnt)))))
-    ;;
-    (mew-net-status-clear (mew-imap-get-status-buf pnm))
-    (mew-info-clean-up pnm)
-    (set-buffer-modified-p nil)
-    (mew-summary-unlock)
-    (if (and (processp sshpro) (not mew-ssh-keep-connection))
-	(process-send-string sshpro "exit\n"))
-    (if (and (processp sslpro) (not mew-ssl-keep-connection))
- 	 (delete-process sslpro))
-    (unless (eq directive 'biff)
-      (run-hooks 'mew-imap-sentinel-non-biff-hook))
-    (run-hooks 'mew-imap-sentinel-hook)
-    (when (and mew-auto-flush-queue flush)
-      (mew-smtp-flush-queue mew-case))))
+	 ((eq directive 'inc)
+	  (mew-biff-clear)
+	  (mew-net-uidl-db-set (mew-imap-passtag pnm) uidl)
+	  (cond
+	   ((= rttl 0)
+	    (mew-imap-message pnm "No new messages"))
+	   ((= rttl 1)
+	    (mew-imap-message pnm "1 message retrieved")
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-imap-message pnm "%s messages retrieved" rttl)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'get)
+	  (cond
+	   ((= rttl 0)
+	    (mew-imap-message pnm "No new messages"))
+	   ((= rttl 1)
+	    (mew-imap-message pnm "1 message retrieved")
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-imap-message pnm "%s messages retrieved" rttl)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'scan)
+	  (mew-biff-clear)
+	  (cond
+	   ((= rttl 0)
+	    (mew-imap-message pnm "No new messages"))
+	   ((= rttl 1)
+	    (mew-imap-message pnm "1 message retrieved")
+	    (mew-lisp-save file msgid nil 'unlimit)
+	    (mew-net-invalid-cache-clean))
+	   (t
+	    (mew-imap-message pnm "%s messages retrieved" rttl)
+	    (mew-lisp-save file msgid nil 'unlimit)
+	    (mew-net-invalid-cache-clean))))
+	 ((eq directive 'exec)
+	  (if kils (mew-mark-exec-unlink bnm kils))
+	  (when movs
+	    (let ((mew-inherit-exec-case case)
+		  (mew-inherit-offline t))
+	      (mew-mark-exec-refile bnm movs)))
+	  (when (or kils movs)
+	    (mew-mark-kill-invisible)
+	    (mew-summary-folder-cache-save))
+	  (cond
+	   ((= rttl 1)
+	    (mew-imap-message pnm "1 message refiled"))
+	   ((> rttl 1)
+	    (mew-imap-message pnm "%d messages refiled" rttl))
+	   ((= dttl -1)
+	    (mew-imap-message pnm "All messages deleted"))
+	   ((= dttl 1)
+	    (mew-imap-message pnm "1 message deleted"))
+	   (t
+	    (mew-imap-message pnm "%d messages deleted" dttl))))
+	 ((eq directive 'jobs)
+	  (if (= jcnt 1)
+	      (mew-imap-message pnm "1 job processed")
+	    (mew-imap-message pnm "%d jobs processed" jcnt)))))
+      ;;
+      (and (functionp 'unix-sync) (unix-sync))
+      (mew-net-status-clear (mew-imap-get-status-buf pnm))
+      (mew-info-clean-up pnm)
+      (set-buffer-modified-p nil)
+      (mew-summary-unlock)
+      (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	  (process-send-string sshpro "exit\n"))
+      (if (and (processp sslpro) (not mew-ssl-keep-connection))
+	  (delete-process sslpro))
+      (mew-net-disp-info-display disp-info)
+      (unless (eq directive 'biff)
+	(run-hooks 'mew-imap-sentinel-non-biff-hook))
+      (run-hooks 'mew-imap-sentinel-hook)
+      (when (and mew-auto-flush-queue flush)
+	(mew-smtp-flush-queue mew-case)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1533,7 +1555,7 @@
       (save-excursion
 	(set-buffer case:inbox)
 	(when (and (mew-summary-exclusive-p 'no-msg)
-		   (and mew-use-cached-passwd
+		   (and (or mew-use-cached-passwd mew-use-master-passwd)
 			(setq passwd (mew-passwd-get-passwd tag))))
 	  (mew-imap-retrieve case 'biff case:inbox passwd))))))
 
@@ -1564,7 +1586,7 @@
 	 (t2 (nth 1 ent)) ;; may be nil
 	 (alist (nth 2 ent))
 	 (fld (mew-imap-folder case))
-	 (file (mew-expand-folder fld mew-imap-folder-alist-file))
+	 (file (mew-expand-file fld mew-imap-folder-alist-file))
 	 (t1 (mew-file-get-time file)))
     (if (or no-load (and t1 (not (mew-compare-times t1 t2))))
 	alist
@@ -1587,9 +1609,9 @@
 
 (defun mew-imap-folder-load (case)
   (let* ((fld (mew-imap-folder case))
-	 (file (mew-expand-folder fld mew-imap-folder-alist-file))
+	 (file (mew-expand-file fld mew-imap-folder-alist-file))
 	 (mailboxes (mew-lisp-load file))
-	 (file2 (mew-expand-folder fld mew-imap-friend-folder-list-file))
+	 (file2 (mew-expand-file fld mew-imap-friend-folder-list-file))
 	 (friends (mew-lisp-load file2)))
     (mew-imap-folder-set case mailboxes friends 'no-save)))
 
@@ -1726,7 +1748,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2005 Mew developing team.
+;; Copyright (C) 2002-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
