@@ -118,37 +118,52 @@
 	(case (mew-imap2-get-case pnm))
 	(src-list (mew-imap2-get-src-list pnm))
 	(msgs (mew-imap2-get-messages pnm))
-	src dst mailbox msg)
+	src dst mailbox msg src-file)
     (if msgs
 	(setq src (mew-imap2-get-src pnm))
-      (catch 'break
-	(while src-list
-	  (setq src (car src-list))
-	  (setq src-list (cdr src-list))
-	  (mew-imap2-set-src pnm src)
-	  (mew-imap2-set-src-list pnm src-list)
-	  (setq msgs (mew-folder-messages (mew-case-folder case src)))
-	  (when msgs
+      (if (eq directive 'copy)
+	  (progn
+	    (setq msgs src-list)
+	    (mew-imap2-set-src-list pnm nil)
 	    (mew-imap2-set-ttl pnm (length msgs))
 	    (mew-imap2-set-cnt pnm 0)
-	    (cond
-	     ((eq directive 'copy)
-	      (setq dst (mew-imap-local-to-imap case src))
-	      (unless (assoc dst (mew-imap-folder-alist case))
-		(mew-imap-folder-insert case dst))
-	      (setq mailbox (mew-imap-expand-mailbox case dst)))
-	     ((eq directive 'move)
-	      (setq mailbox (mew-imap-fcc-to-mailbox case src))))
-	    (mew-imap2-set-mailbox pnm mailbox)
-	    (throw 'break nil)))))
+	    (setq dst (mew-imap2-get-mailbox pnm))
+	    (setq mailbox (mew-imap-expand-mailbox case dst))
+	    (setq mailbox (mew-imap-utf-7-encode-string mailbox))
+	    (mew-imap2-set-mailbox pnm mailbox))
+	(catch 'break
+	  (while src-list
+	    (setq src (car src-list))
+	    (setq src-list (cdr src-list))
+	    (mew-imap2-set-src pnm src)
+	    (mew-imap2-set-src-list pnm src-list)
+	    (setq msgs (mew-folder-messages (mew-case-folder case src)))
+	    (when msgs
+	      (mew-imap2-set-ttl pnm (length msgs))
+	      (mew-imap2-set-cnt pnm 0)
+	      (cond
+	       ((eq directive 'cp-r)
+		(setq dst (mew-imap-local-to-imap case src))
+		(unless (assoc dst (mew-imap-folder-alist case))
+		  (mew-imap-folder-insert case dst))
+		(setq mailbox (mew-imap-expand-mailbox case dst)))
+	       ((eq directive 'fcc)
+		(setq mailbox (mew-imap-fcc-to-mailbox case src))))
+	      (setq mailbox (mew-imap-utf-7-encode-string mailbox))
+	      (mew-imap2-set-mailbox pnm mailbox)
+	      (throw 'break nil))))))
     (if msgs
 	(progn
 	  (setq msg (car msgs))
 	  (setq msgs (cdr msgs))
+	  (cond
+	   ((eq directive 'copy)
+	    (setq src-file msg))
+	   (t
+	    (setq src-file (mew-expand-msg (mew-case-folder case src) msg))))
 	  (mew-frwlet
 	   mew-cs-text-for-read mew-cs-dummy
-	   (set-buffer (mew-find-file-noselect
-			(mew-expand-folder (mew-case-folder case src) msg))))
+	   (set-buffer (mew-find-file-noselect src-file)))
 	  (mew-set-buffer-multibyte nil)
 	  (mew-info-clean-up pnm mew-imap2-info-list-clean-length)
 	  (mew-imap2-set-messages pnm msgs)
@@ -351,7 +366,7 @@
 	pro tm)
     (condition-case emsg
 	(progn
-	  (setq tm (mew-timer mew-imap-timeout-time 'mew-imap2-timeout))
+	  (setq tm (run-at-time mew-imap-timeout-time nil 'mew-imap2-timeout))
 	  (message "Connecting to the IMAP server...")
 	  (setq pro (open-network-stream pnm nil server sprt))
 	  (process-kill-without-query pro)
@@ -374,7 +389,7 @@
 ;;; Launcher
 ;;;
 
-(defun mew-imap2-copy-message (case directive src-list)
+(defun mew-imap2-copy-message (case directive src-list &optional dst)
   (let* ((server (mew-imap-server case))
          (user (mew-imap-user case))
 	 (port (mew-imap-port case))
@@ -382,7 +397,9 @@
 	 (sshsrv (mew-imap-ssh-server case))
 	 (sslp (mew-imap-ssl case))
 	 (sslport (mew-imap-ssl-port case))
-	 process sshname sshpro sslname sslpro lport)
+	 (proxysrv (mew-imap-proxy-server case))
+	 (proxyport (mew-imap-proxy-port case))
+	 process sshname sshpro sslname sslpro lport tls)
     (cond
      (sshsrv
       (setq sshpro (mew-open-ssh-stream case server port sshsrv))
@@ -392,12 +409,15 @@
 	(when lport
 	  (setq process (mew-imap2-open pnm "localhost" lport)))))
      (sslp
-      (setq sslpro (mew-open-ssl-stream server sslport))
+      (if (string= port sslport) (setq tls mew-tls-imap))
+      (setq sslpro (mew-open-ssl-stream case server sslport tls))
       (when sslpro
 	(setq sslname (process-name sslpro))
 	(setq lport (mew-ssl-pnm-to-lport sslname))
 	(when lport
-	  (setq process (mew-imap2-open pnm "localhost" lport)))))
+	  (setq process (mew-imap2-open pnm mew-ssl-localhost lport)))))
+     (proxysrv
+      (setq process (mew-imap2-open pnm proxysrv proxyport)))
      (t
       (setq process (mew-imap2-open pnm server port))))
     (if (null process)
@@ -405,7 +425,7 @@
 	 ((and sshsrv (null sshpro))
 	  (message "Cannot create to the SSH connection"))
 	 ((and sslp (null sslpro))
-	  (message "Cannot create to the SSL connection"))
+	  (message "Cannot create to the SSL/TLS connection"))
 	 (t
 	  (message "Cannot connect to the IMAP server")))
       (mew-info-clean-up pnm)
@@ -424,6 +444,7 @@
       (mew-imap2-set-status pnm "greeting")
       (mew-imap2-set-src-list pnm src-list)
       (mew-imap2-set-src-list-orig pnm src-list)
+      (mew-imap2-set-mailbox pnm dst)
       ;;
       (set-process-buffer process nil)
       (set-process-sentinel process 'mew-imap2-sentinel)
@@ -448,7 +469,7 @@ time (even several days) if there are many local messages."
       (if (string-match regex fld)
 	  (setq src-list (cons fld src-list))))
     (setq src-list (nreverse src-list))
-    (mew-imap2-copy-message case 'copy src-list)))
+    (mew-imap2-copy-message case 'cp-r src-list)))
 
 (defun mew-imap2-fcc (case)
   (let* ((prefix mew-folder-imap-fcc-prefix)
@@ -456,7 +477,48 @@ time (even several days) if there are many local messages."
 	 (fcc-dirs (mew-dir-list fcc-dir))
 	 (src-list (mew-local-folder-make-alist fcc-dirs prefix 'make-list)))
     ;; src-list
-    (mew-imap2-copy-message case 'move src-list)))
+    (mew-imap2-copy-message case 'fcc src-list)))
+
+(defun mew-summary-imap-copy ()
+  "Copy a message to the server side of IMAP."
+  (interactive)
+  (mew-summary-not-in-draft
+   (let* ((fld (mew-summary-folder-name))
+	  (msg (mew-summary-message-number))
+	  (fld/msg (mew-expand-msg fld msg))
+	  (src-list (list fld/msg))
+	  (case:dst (mew-input-folder mew-case mew-imap-inbox-folder))
+	  (case (mew-case:folder-case case:dst))
+	  (dst (mew-case:folder-folder case:dst)))
+     (when (mew-folder-imapp dst)
+       (mew-imap2-copy-message case 'copy src-list dst)))))
+
+(defun mew-summary-mark-imap-copy (&optional arg)
+  "Copy messages marked with '*' to the server side of IMAP.
+If called with '\\[universal-argument]', only messages marked
+with '*' in the region are handled."
+  (interactive "P")
+  (mew-summary-not-in-draft
+   (let* ((fld (mew-summary-folder-name))
+	  (case:dst (mew-input-folder mew-case mew-imap-inbox-folder))
+	  (case (mew-case:folder-case case:dst))
+	  (dst (mew-case:folder-folder case:dst))
+	  msgs beg end region src-list)
+     (if (mew-mark-active-p) (setq arg t))
+     (cond
+      (arg
+       (setq region (mew-summary-get-region))
+       (setq beg (car region))
+       (setq end (cdr region)))
+      (t
+       (setq beg (point-min))
+       (setq end (point-max))))
+     (setq msgs (mew-summary-mark-collect mew-mark-review beg end))
+     (if (null msgs)
+	 (message "No mark")
+       (when (mew-folder-imapp dst)
+	 (setq src-list (mapcar (lambda (x) (mew-expand-msg fld x)) msgs))
+	 (mew-imap2-copy-message case 'copy src-list dst))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -478,39 +540,40 @@ time (even several days) if there are many local messages."
 	 (str (concat (mew-imap2-get-string pnm) string))
 	 (buf (process-buffer process))
 	 aux stay next func code resp)
-    (mew-imap2-debug (upcase status) string)
-    (if (and buf (get-buffer buf)) (set-buffer buf))
-    (while (string-match "^\\*[^\n]*\n" str)
-      (setq aux (substring str 0 (match-end 0)))
-      (setq str (substring str (match-end 0)))
+    (save-excursion
+      (mew-imap2-debug (upcase status) string)
+      (if (and buf (get-buffer buf)) (set-buffer buf))
+      (while (string-match "^\\*[^\n]*\n" str)
+	(setq aux (substring str 0 (match-end 0)))
+	(setq str (substring str (match-end 0)))
+	(cond
+	 ((string= status "greeting")
+	  (setq next (mew-imap2-fsm-next "greeting" "OK")))
+	 ((string= status "capability")
+	  (mew-imap2-set-aux pnm (concat (mew-imap2-get-aux pnm) aux)))))
+      (mew-imap2-set-string pnm str)
       (cond
-       ((string= status "greeting")
-	(setq next (mew-imap2-fsm-next "greeting" "OK")))
-       ((string= status "capability")
-	(mew-imap2-set-aux pnm (concat (mew-imap2-get-aux pnm) aux)))))
-    (mew-imap2-set-string pnm str)
-    (cond
-     (next
-      nil)
-     ((string-match "^\\+" str)
-      (mew-imap2-set-aux pnm str)
-      (setq next (mew-imap2-fsm-next status "OK")))
-     ((string-match eos str)
-      (mew-imap2-set-tag pnm nil)
-      (setq code (mew-match-string 1 str))
-      (setq resp (mew-match-string 2 str))
-      (setq next (mew-imap2-fsm-next status code))
-      (unless next (setq next (mew-imap2-fsm-next status resp))))
-     (t
-      (setq stay t)))
-    (unless stay
-      (unless next (setq next "logout"))
-      (mew-imap2-set-status pnm next)
-      (setq func (intern-soft (concat "mew-imap2-command-" next)))
-      (if (fboundp func)
-	  (funcall func process pnm)
-	(error "No function called %s" (symbol-name func)))
-      (mew-imap2-set-string pnm nil))))
+       (next
+	nil)
+       ((string-match "^\\+" str)
+	(mew-imap2-set-aux pnm str)
+	(setq next (mew-imap2-fsm-next status "OK")))
+       ((string-match eos str)
+	(mew-imap2-set-tag pnm nil)
+	(setq code (mew-match-string 1 str))
+	(setq resp (mew-match-string 2 str))
+	(setq next (mew-imap2-fsm-next status code))
+	(unless next (setq next (mew-imap2-fsm-next status resp))))
+       (t
+	(setq stay t)))
+      (unless stay
+	(unless next (setq next "logout"))
+	(mew-imap2-set-status pnm next)
+	(setq func (intern-soft (concat "mew-imap2-command-" next)))
+	(if (fboundp func)
+	    (funcall func process pnm)
+	  (error "No function called %s" (symbol-name func)))
+	(mew-imap2-set-string pnm nil)))))
 
 (defun mew-imap2-sentinel (process event)
   (let* ((pnm (process-name process))
@@ -522,27 +585,28 @@ time (even several days) if there are many local messages."
 	 (sslpro (mew-imap2-get-ssl-process pnm))
 	 (srcflds (mew-imap2-get-src-list-orig pnm))
 	 srcfld)
-    (mew-imap2-debug "IMAP SENTINEL" event)
-    (cond
-     (error
-      (message "%s" error))
-     (done
-      (when (eq directive 'move)
-	(while srcflds
-	  (setq srcfld (car srcflds))
-	  (setq srcflds (cdr srcflds))
-	  (mew-imap2-copy-fcc-cache case srcfld))
-	(mew-delete-directory-recursively 
-	 (mew-folder-imap-fcc-dir case)))
-      (message "Copying in background...done"))
-     (t 
-      (message "IMAP connection is lost")))
-    (mew-info-clean-up pnm)
-    (if (and (processp sshpro) (not mew-ssh-keep-connection))
-	(process-send-string sshpro "exit\n"))
-    (if (and (processp sslpro) (not mew-ssl-keep-connection))
-	(delete-process sslpro))
-    (run-hooks 'mew-imap2-sentinel-hook)))
+    (save-excursion
+      (mew-imap2-debug "IMAP SENTINEL" event)
+      (cond
+       (error
+	(message "%s" error))
+       (done
+	(when (eq directive 'fcc)
+	  (while srcflds
+	    (setq srcfld (car srcflds))
+	    (setq srcflds (cdr srcflds))
+	    (mew-imap2-copy-fcc-cache case srcfld))
+	  (mew-delete-directory-recursively 
+	   (mew-folder-imap-fcc-dir case)))
+	(message "Copying in background...done"))
+       (t 
+	(message "IMAP connection is lost")))
+      (mew-info-clean-up pnm)
+      (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	  (process-send-string sshpro "exit\n"))
+      (if (and (processp sslpro) (not mew-ssl-keep-connection))
+	  (delete-process sslpro))
+      (run-hooks 'mew-imap2-sentinel-hook))))
 
 (defun mew-imap2-copy-fcc-cache (case srcfld)
   (let* ((case:srcfld (mew-case-folder case srcfld))
@@ -554,14 +618,14 @@ time (even several days) if there are many local messages."
     (setq dstfld (concat mew-folder-imap (substring srcfld (match-end 0))))
     (setq case:dstfld (mew-case-folder case dstfld))
     (mew-local-folder-check case:dstfld)
-    (setq i (string-to-int (mew-folder-new-message case:dstfld 'num 'cache)))
+    (setq i (string-to-number (mew-folder-new-message case:dstfld 'num 'cache)))
     ;; to make code simple, we just copy the file.
     (while srcs
       (setq src (car srcs))
       (setq srcs (cdr srcs))
       (setq dst (mew-i2s i 'cache))
-      (copy-file (mew-expand-folder case:srcfld src)
-		 (mew-expand-folder case:dstfld dst))
+      (copy-file (mew-expand-msg case:srcfld src)
+		 (mew-expand-new-msg case:dstfld dst))
       (setq i (1+ i)))
     (mew-touch-folder case:dstfld)))
 
@@ -569,7 +633,7 @@ time (even several days) if there are many local messages."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2003 Mew developing team.
+;; Copyright (C) 2003-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
