@@ -47,10 +47,11 @@
     'mew-summary-analyze-again-alternative)
   (define-key mew-summary-mode-map ","    'mew-summary-display-asis)
   (define-key mew-summary-mode-map ";"    'mew-summary-trace-path)
-  (define-key mew-summary-mode-map "<"    'mew-summary-jump-top)
-  (define-key mew-summary-mode-map ">"    'mew-summary-jump-bottom)
-  (define-key mew-summary-mode-map "\177" 'mew-summary-prev-page)
-  (define-key mew-summary-mode-map [delete] 'mew-summary-prev-page)
+  (define-key mew-summary-mode-map "\e<"  'mew-summary-jump-top)
+  (define-key mew-summary-mode-map "\e>"  'mew-summary-jump-bottom)
+  (define-key mew-summary-mode-map "<"    'mew-summary-scroll-right)
+  (define-key mew-summary-mode-map ">"    'mew-summary-scroll-left)
+  (define-key mew-summary-mode-map "\d"   'mew-summary-prev-page)
   (define-key mew-summary-mode-map "\r"   'mew-summary-scroll-up)
   (define-key mew-summary-mode-map "-"    'mew-summary-scroll-down)
   (define-key mew-summary-mode-map "\e\r" 'mew-summary-scroll-down)
@@ -104,6 +105,7 @@
   (define-key mew-summary-mode-map "\C-c\t"   'mew-summary-find-file)
   (define-key mew-summary-mode-map "\C-c\C-f" 'mew-pgp-fetch-key)
   (define-key mew-summary-mode-map "\C-c\C-v" 'mew-pgp-select)
+  (define-key mew-summary-mode-map "\C-c\C-m" 'mew-passwd-change)
   (define-key mew-summary-mode-map "\C-c\C-s" 'mew-summary-isearch-forward)
   (define-key mew-summary-mode-map "\C-c\C-r" 'mew-summary-isearch-backward)
   (define-key mew-summary-mode-map "\C-c\C-o" 
@@ -142,15 +144,16 @@
   (define-key mew-summary-mode-map "mli"  'mew-summary-mark-imap-copy)
   (define-key mew-summary-mode-map "V"    'mew-summary-virtual-old)
   (define-key mew-summary-mode-map "/"    'mew-summary-virtual)
-  (define-key mew-summary-mode-map "?"    'mew-summary-pick-mark)
-  (define-key mew-summary-mode-map "'"    'mew-summary-grep-mark)
-  (define-key mew-summary-mode-map "\""   'mew-summary-cmd-mark)
-  (define-key mew-summary-mode-map "mt"   'mew-summary-mark-make-thread)
+  (define-key mew-summary-mode-map "?"    'mew-summary-pick)
+  (define-key mew-summary-mode-map "'"    'mew-summary-grep-old)
+  (define-key mew-summary-mode-map "m/"   'mew-summary-mark-virtual)
+  (define-key mew-summary-mode-map "mt"   'mew-summary-mark-thread)
   (define-key mew-summary-mode-map "t"    (make-sparse-keymap))
   (define-key mew-summary-mode-map "tt"   'mew-summary-make-thread)
   (define-key mew-summary-mode-map "tn"   'mew-thread-down)
   (define-key mew-summary-mode-map "tp"   'mew-thread-up)
   (define-key mew-summary-mode-map "t "   'mew-thread-toggle)
+  (define-key mew-summary-mode-map "ta"   'mew-thread-toggle-all)
   (define-key mew-summary-mode-map "t("   'mew-thread-all-graft)
   (define-key mew-summary-mode-map "t)"   'mew-thread-all-prune)
   (define-key mew-summary-mode-map "t*"   'mew-thread-mark-review)
@@ -178,21 +181,13 @@
   (define-key mew-summary-mode-map "lh"   'mew-summary-learn-ham)
   (define-key mew-summary-mode-map "ls"   'mew-summary-learn-spam)
   (define-key mew-summary-mode-map "="    'mew-summary-info)
+  (define-key mew-summary-mode-map "_"    'mew-summary-line)
   (define-key mew-summary-mode-map "k"	  (make-sparse-keymap))
-  (define-key mew-summary-mode-map "k?"   'mew-nmz-search-mark)
-  (define-key mew-summary-mode-map "k/"   'mew-nmz-virtual)
-  (define-key mew-summary-mode-map "kV"   'mew-nmz-virtual)
-  (define-key mew-summary-mode-map "kg"   'mew-nmz-gcnmz)
-  (define-key mew-summary-mode-map "km"   'mew-nmz-mknmz)
-  (define-key mew-summary-mode-map "kM"   'mew-nmz-mknmz-all-folders)
-  (define-key mew-summary-mode-map "ks"   'mew-nmz-mknmz-save-mewmknmz)
-  (define-key mew-summary-mode-map "kK"   'mew-nmz-mknmz-kill-process)
-  (define-key mew-summary-mode-map "ku"   'mew-nmz-mark-unindexed)
-  (define-key mew-summary-mode-map "k^"   'mew-nmz-search-parent)
-  (define-key mew-summary-mode-map "kp"   'mew-nmz-search-parent)
-  (define-key mew-summary-mode-map "kn"   'mew-nmz-search-child)
-  (define-key mew-summary-mode-map "kN"   'mew-nmz-namazu)
-  (define-key mew-summary-mode-map "kj"   'mew-nmz-original-message)
+  (define-key mew-summary-mode-map "kc"   'mew-summary-search-change-method)
+  (define-key mew-summary-mode-map "k?"   'mew-summary-search)
+  (define-key mew-summary-mode-map "k/"   'mew-summary-search-virtual)
+  (define-key mew-summary-mode-map "km"   'mew-summary-make-index-folder)
+  (define-key mew-summary-mode-map "kM"   'mew-summary-make-index-all)
   ;;
   ;; not provided in Virtual mode
   ;;
@@ -205,7 +200,6 @@
   (define-key mew-summary-mode-map "X"    'mew-summary-exec-one)
   (define-key mew-summary-mode-map "lx"   'mew-summary-exec-offline)
   (define-key mew-summary-mode-map "lX"   'mew-summary-exec-offline-one)
-  (define-key mew-summary-mode-map "m/"   'mew-summary-mark-virtual)
   (define-key mew-summary-mode-map "mxd"   'mew-summary-exec-delete)
   (define-key mew-summary-mode-map "mx\ed" 'mew-summary-exec-unlink)
   (define-key mew-summary-mode-map "mxo"   'mew-summary-exec-refile)
@@ -262,10 +256,8 @@
      ["Multi forward" mew-summary-multi-forward (mew-summary-case1)]
      )
     ("Select"
-     ["Pick then mark"     mew-summary-pick-mark   (mew-pickable-p)]
-     ["Grep then mark"     mew-summary-grep-mark   (mew-pickable-p)]
-     ["Command then mark"  mew-summary-cmd-mark    (mew-pickable-p)]
-     ["Virtual mode"       mew-summary-virtual     (mew-summary-or-virtual-p)]
+     ["Pick then mark"     mew-summary-pick        (mew-pickable-p)]
+     ["Virtual mode"       mew-summary-virtual     (mew-pickable-p)]
      )
     ("Misc"
      ["Clean +trash"         mew-summary-clean-trash      t]
@@ -296,8 +288,7 @@
 (unless mew-message-mode-map
   (setq mew-message-mode-map (make-sparse-keymap))
   (define-key mew-message-mode-map " "    'mew-message-next-page)
-  (define-key mew-message-mode-map "\177" 'mew-message-prev-page)
-  (define-key mew-message-mode-map [delete] 'mew-message-prev-page)
+  (define-key mew-message-mode-map "\d"   'mew-message-prev-page)
   (define-key mew-message-mode-map "n"    'mew-message-next-msg)
   (define-key mew-message-mode-map "p"    'mew-message-prev-msg)
   (define-key mew-message-mode-map "h"    'mew-message-goto-summary)
@@ -305,9 +296,14 @@
   (define-key mew-message-mode-map "A"    'mew-message-reply-with-citation)
   (define-key mew-message-mode-map "f"    'mew-message-forward)
   (define-key mew-message-mode-map "r"    'mew-message-resend)
+  (define-key mew-message-mode-map "<"    'mew-message-scroll-right)
+  (define-key mew-message-mode-map ">"    'mew-message-scroll-left)
+  (define-key mew-message-mode-map "_"    'mew-message-line)
   (define-key mew-message-mode-map "k"	  (make-sparse-keymap))
   (define-key mew-message-mode-map "kp"   'mew-nmz-search-msgid-at-point)
-  (define-key mew-message-mode-map "kr"   'mew-nmz-search-msgid-region))
+  (define-key mew-message-mode-map "kr"   'mew-nmz-search-msgid-region)
+  (define-key mew-message-mode-map "g"    'mew-browse-url-at-point)
+  (define-key mew-message-mode-map "\C-i" 'mew-message-goto-next-url))
 
 (defvar mew-message-mode-menu-spec
   '("Mew"
@@ -510,6 +506,7 @@
   (define-key mew-draft-attach-map "S"      'mew-attach-pgp-sign)
   (define-key mew-draft-attach-map "E"      'mew-attach-pgp-enc)
   (define-key mew-draft-attach-map "\es"    'mew-attach-smime-sign)
+  (define-key mew-draft-attach-map "\ee"    'mew-attach-smime-enc)
   (define-key mew-draft-attach-map "p"      'mew-attach-pgp-public-key)
   (define-key mew-draft-attach-map "U"      'mew-attach-undo)
   (define-key mew-draft-attach-map "\C-c\t" 'mew-draft-insert-signature)
@@ -538,11 +535,9 @@
 	  (make-sparse-keymap)))
   (define-key mew-input-map "\r"     'mew-input-exit-minibuffer)
   (define-key mew-input-map "\n"     'mew-input-exit-minibuffer)
-  (define-key mew-input-map [return] 'mew-input-exit-minibuffer)
   (define-key mew-input-map ","      'mew-input-comma)
   (define-key mew-input-map " "      'mew-input-complete)
   (define-key mew-input-map "\t"     'mew-input-complete)
-  (define-key mew-input-map [tab]    'mew-input-complete)
   (define-key mew-input-map "?"      'mew-input-complete)
   (define-key mew-input-map "\C-c\t" 'mew-circular-complete-switch)
   (define-key mew-input-map [menu-bar minibuf]
@@ -561,12 +556,10 @@
       (define-key mew-input-folder-map (char-to-string c)
 	'mew-input-folder-self-insert)
       (setq c (1+ c))))
-  (define-key mew-input-folder-map [delete]    'mew-input-folder-self-insert)
-  (define-key mew-input-folder-map [backspace] 'mew-input-folder-self-insert)
+  (define-key mew-input-folder-map "\d"     'mew-input-folder-self-insert)
   (define-key mew-input-folder-map "\C-g"   'mew-input-folder-abort-minibuffer)
   (define-key mew-input-folder-map "\r"     'mew-input-folder-exit-minibuffer)
   (define-key mew-input-folder-map "\n"     'mew-input-folder-exit-minibuffer)
-  (define-key mew-input-folder-map [return] 'mew-input-folder-exit-minibuffer)
   (define-key mew-input-folder-map "+"      'mew-input-folder-prefix)
   (define-key mew-input-folder-map "%"      'mew-input-folder-prefix)
   (define-key mew-input-folder-map "$"      'mew-input-folder-prefix)
@@ -660,7 +653,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2005 Mew developing team.
+;; Copyright (C) 2000-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
