@@ -216,7 +216,7 @@ Binary search is used for speed reasons."
 ;;;
 
 (defvar mew-local-info-list
-  '("directive" "bnm" "mdb" "flush" "rcnt" "rttl" "first" "case"))
+  '("directive" "buf" "bnm" "mdb" "flush" "rcnt" "rttl" "first" "case"))
 
 (mew-info-defun "mew-local-" mew-local-info-list)
 
@@ -240,8 +240,11 @@ Binary search is used for speed reasons."
 ;;; Scan
 ;;;
 
+(defun mew-scan-mewl-folder (folder)
+  (mew-replace-character folder mew-sp ?\007))
+
 (defun mew-scan-mewl-src (folder &optional range)
-  (setq folder (mew-expand-folder2 folder))
+  (setq folder (mew-scan-mewl-folder (mew-expand-folder2 folder)))
   (if range
       (list "-s" (format "%s %s" folder range))
     (list "-s" folder)))
@@ -260,9 +263,11 @@ Binary search is used for speed reasons."
      (mew-sinfo-set-summary-form (mew-get-summary-form bnm))
      (mew-sinfo-set-unread-mark nil)
      (mew-sinfo-set-scan-id nil)
+     (mew-sinfo-set-scan-md5 nil)
      (mew-info-clean-up pnm)
      (mew-local-set-directive pnm directive)
      (mew-local-set-bnm pnm bnm)
+     (mew-local-set-buf pnm buf)
      (mew-local-set-case pnm mew-case) ;; xxx specfy to mew-local-retrieve?
      (cond
       ((eq directive 'inc)
@@ -284,7 +289,8 @@ Binary search is used for speed reasons."
        (setq opts (nth 0 args))
        (mew-vinfo-set-func  (nth 1 args))
        (mew-vinfo-set-lra   (nth 2 args))
-       (unless (nth 3 args) (mew-erase-buffer))))
+       (unless (nth 3 args) (mew-erase-buffer))
+       (mew-local-set-rttl pnm (nth 4 args))))
      (mew-sinfo-set-start-point (point)) ;; after erase-buffer
      (mew-local-set-rcnt pnm 1)
      (save-excursion
@@ -294,6 +300,7 @@ Binary search is used for speed reasons."
 			      "-l" (number-to-string mew-scan-max-field-length)
 			      "-c" (number-to-string mew-scan-max-body-length)
 			      "-w"
+			      "-x" mew-suffix
 			      "-f" (mapconcat 'identity
 					      (nthcdr 2 mew-scan-fields)
 					      ","))
@@ -347,35 +354,38 @@ Binary search is used for speed reasons."
 (defun mew-local-sentinel (process event)
   (let* ((pnm (process-name process))
 	 (bnm (mew-local-get-bnm pnm))
+	 (buf (mew-local-get-buf pnm))
 	 (virtualp (mew-folder-virtualp bnm))
 	 (mdb (mew-local-get-mdb pnm))
 	 (flush (mew-local-get-flush pnm)))
-    (mew-local-debug "SENTINEL" event)
-    (mew-filter
-     (mew-set-buffer-multibyte t)
-     (set-buffer bnm)
-     (mew-summary-mark-recover mdb)
-     (if (and virtualp (mew-vinfo-get-func))
-	 (funcall (mew-vinfo-get-func)))
-     (mew-vinfo-set-func nil)
-     (mew-info-clean-up pnm)
-     (if virtualp
-	 (mew-summary-set-count-line)
-       (mew-summary-folder-cache-save))
-     (set-buffer-modified-p nil)
-     (mew-summary-unlock)
-     (if (and (mew-folder-imapp bnm)
-	      (not (mew-folder-imap-queuep)))
-	 (if (mew-local-get-rttl pnm)
-	     (message "Type '\\[mew-summary-ls]' to override invalid messages")
-	   ;; This folder has been just created.
-	   ;; Ideally scan should not be called. But there is no way
-	   ;; to avoid the call.
-	   (message ""))
-       (message "Scanning %s...done" bnm))
-     (run-hooks 'mew-scan-sentinel-hook)
-     (when (and mew-auto-flush-queue flush)
-       (mew-smtp-flush-queue mew-case)))))
+    (save-excursion
+      (mew-local-debug "SENTINEL" event)
+      (mew-filter
+       (mew-set-buffer-multibyte t)
+       (set-buffer bnm)
+       (mew-summary-mark-recover mdb)
+       (if (and virtualp (mew-vinfo-get-func))
+	   (funcall (mew-vinfo-get-func)))
+       (mew-vinfo-set-func nil)
+       (mew-info-clean-up pnm)
+       (if virtualp
+	   (mew-summary-set-count-line)
+	 (mew-summary-folder-cache-save))
+       (set-buffer-modified-p nil)
+       (mew-summary-unlock)
+       (if (and (mew-folder-imapp bnm)
+		(not (mew-folder-imap-queuep)))
+	   (if (mew-local-get-rttl pnm)
+	       (message "Type '\\[mew-summary-ls]' to override invalid messages")
+	     ;; This folder has been just created.
+	     ;; Ideally scan should not be called. But there is no way
+	     ;; to avoid the call.
+	     (message ""))
+	 (message "Scanning %s...done" bnm))
+       (run-hooks 'mew-scan-sentinel-hook)
+       (when (and mew-auto-flush-queue flush)
+	 (mew-smtp-flush-queue mew-case))
+       (mew-remove-buffer buf)))))
 
 ;;; Code:
 
@@ -385,7 +395,7 @@ Binary search is used for speed reasons."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2001-2005 Mew developing team.
+;; Copyright (C) 2001-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
