@@ -495,9 +495,6 @@ it is deleted automatically."
 	(if (and (mew-folder-popp folder) ;; not $inbox
 		 (not (string= mew-pop-inbox-folder folder)))
 	    (throw 'continue nil))
-	(when (string-match " " folder)
-	  (mew-warn "SPC is not allowed!")
-	  (throw 'continue nil))
 	(setq case:folder (mew-canonicalize-case-folder case:folder))
 	(setq ret (cons case:folder ret))))
     (setq ret (delete nil ret))
@@ -573,9 +570,6 @@ it is deleted automatically."
 	(if (and (= (length folder) 1)
 		 (member folder mew-folder-prefixes)) ;; "prefix" only
 	    (throw 'continue nil))
-	(when (string-match " " folder)
-	  (mew-warn "SPC is not allowed!")
-	  (throw 'continue nil))
 	(setq folder (mew-canonicalize-case-folder folder))
 	(setq ret (cons folder ret))))
     (nreverse ret)))
@@ -650,30 +644,45 @@ it is deleted automatically."
 
 (defvar mew-input-pick-pattern-hist nil)
 
-(defun mew-input-pick-pattern ()
+(defun mew-input-pick-pattern (prompt)
   (mew-input-clear)
   (let ((mew-input-complete-function 'mew-complete-pick-pattern)
 	(mew-circular-complete-function 'mew-circular-complete-pick-pattern)
 	(keymap (copy-keymap mew-input-map)))
     (define-key keymap " " nil)
     (mew-pick-macro-expand-string
-     (read-from-minibuffer "Pick pattern: "
+     (read-from-minibuffer (concat prompt " pattern: ")
 			   (car mew-pick-pattern-list)
 			   keymap
 			   nil
 			   'mew-input-pick-pattern-hist))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Grep pattern
-;;;
+(defvar mew-input-pick-command-hist nil)
 
-(defvar mew-input-grep-pattern-hist nil)
-
-(defun mew-input-grep-pattern ()
-  (read-string "Grep pattern: "
-               (car-safe mew-input-grep-pattern-hist)
-               'mew-input-grep-pattern-hist))
+(defun mew-input-pick-command (prog opts)
+  (mew-input-clear)
+  (let ((init (mapconcat 'identity (cons prog opts) " "))
+	(prompt "Cmd opts pattern: ")
+	input lst cmd opts len pat)
+    (unless (string-match " $" init)
+      (setq init (concat init " ")))
+    (setq input (read-string prompt init 'mew-input-pick-command-hist))
+    (setq mew-input-pick-command-hist
+	  (mew-uniq-list mew-input-pick-command-hist))
+    (setq lst (mew-split-quoted input mew-sp))
+    (setq cmd (car lst))
+    (setq opts (cdr lst))
+    (setq len (length opts))
+    (cond
+     ((= len 0)
+      )
+     ((= len 1)
+      (setq pat (car opts))
+      (setq opts nil))
+     (t
+      (setq pat (nth (1- len) opts))
+      (setcdr (nthcdr (- len 2) opts) nil)))
+    (list cmd opts pat)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -861,7 +870,11 @@ it is deleted automatically."
        (setq def (concat dir ,file)))
      (setq ret (mew-input-file-name ,prompt def))
      (if ,preservep
-	 (setq ,previous (file-name-directory (mew-convert-to-home-dir ret))))
+	 (setq ,previous (file-name-directory
+			  (mew-convert-to-home-dir
+			   (if (file-directory-p ret)
+			       (file-name-as-directory ret)
+			     ret)))))
      (if (and ,file (file-directory-p ret))
 	 (setq ret (expand-file-name (file-name-nondirectory ,file) ret)))
      ret))
@@ -995,79 +1008,6 @@ it is deleted automatically."
      (t (message "Mark %c is not supported" char)
 	nil))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Password
-;;;
-
-(defvar mew-passwd-alist nil)
-(defvar mew-passwd-timer-id nil)
-
-(defsubst mew-passwd-get-passwd (key)
-  (nth 1 (assoc key mew-passwd-alist)))
-(defsubst mew-passwd-get-counter (key)
-  (nth 2 (assoc key mew-passwd-alist)))
-
-(defun mew-passwd-set-passwd (key val)
-  (if (assoc key mew-passwd-alist)
-      (setcar (nthcdr 1 (assoc key mew-passwd-alist)) val)
-    (setq mew-passwd-alist (cons (list key val 0) mew-passwd-alist))))
-(defun mew-passwd-set-counter (key val)
-  (if (assoc key mew-passwd-alist)
-      (setcar (nthcdr 2 (assoc key mew-passwd-alist)) val)))
-
-(defun mew-passwd-get-keys ()
-  (mapcar 'car mew-passwd-alist))
-
-(defsubst mew-passwd-reset ()
-  (setq mew-passwd-alist nil))
-
-(defun mew-passwd-setup ()
-  (if mew-passwd-timer-id (cancel-timer mew-passwd-timer-id))
-  (setq mew-passwd-timer-id
-	(mew-timer (* mew-passwd-timer-unit 60) 'mew-passwd-timer)))
-
-(defun mew-passwd-clean-up ()
-  (mew-passwd-reset)
-  (if mew-passwd-timer-id (cancel-timer mew-passwd-timer-id))
-  (setq mew-passwd-timer-id nil))
-
-(defun mew-passwd-timer ()
-  (let ((keys (mew-passwd-get-keys)) key)
-    (while keys
-      (setq key (car keys))
-      (setq keys (cdr keys))
-      (if (< (mew-passwd-get-counter key) mew-passwd-lifetime)
-	  (mew-passwd-set-counter key (1+ (mew-passwd-get-counter key)))
-	;; time out
-	(mew-passwd-set-passwd key nil)
-	(mew-passwd-set-counter key 0)))))
-
-(defun mew-input-passwd (prompt key)
-  (if (and key mew-use-cached-passwd)
-      (if (mew-passwd-get-passwd key)
-	  (progn
-	    (mew-timing)
-	    (if mew-passwd-reset-timer (mew-passwd-set-counter key 0))
-	    (mew-passwd-get-passwd key))
-	(let ((pass (mew-read-passwd prompt)))
-	  (mew-passwd-set-passwd key pass)
-	  (mew-passwd-set-counter key 0)
-	  pass))
-    (mew-read-passwd prompt)))
-
-(defun mew-read-passwd (prompt)
-  (let ((inhibit-input-event-recording t)
-	;; A process filter sets inhibit-quit to t to prevent quitting.
-	;; Set inhibit-quit to nil so that C-g can be used
-	(inhibit-quit nil))
-    (condition-case nil
-	(read-passwd prompt)
-      ;; If read-passwd causes an error, let's return "" so that
-      ;; the password process will safely fail.
-      (quit "")
-      (error ""))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Language name
@@ -1114,11 +1054,11 @@ it is deleted automatically."
 
 (defun mew-input-command (&optional default)
   (let ((program (read-string "Command args: " default)))
-    (setq program (mew-split-quoted program mew-sp ?\" ?\"))
+    (setq program (mew-split-quoted program mew-sp))
     (setq program (delete "" program))
     (while (not (mew-which-exec (car program)))
       (setq program (read-string "Command args: " default))
-      (setq program (mew-split-quoted program mew-sp ?\" ?\"))
+      (setq program (mew-split-quoted program mew-sp))
       (setq program (delete "" program)))
     program))
 
@@ -1126,7 +1066,7 @@ it is deleted automatically."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2005 Mew developing team.
+;; Copyright (C) 1997-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
