@@ -35,9 +35,9 @@
 ;;; Net folders
 ;;;
 
-(defvar mew-pop-folder  "+.pop/%s@%s%%%s")
-(defvar mew-imap-folder "+.imap/%s@%s%%%s")
-(defvar mew-nntp-folder "+.nntp/%s@%s%%%s")
+(defvar mew-pop-folder  "+#pop/%s@%s#%s")
+(defvar mew-imap-folder "+#imap/%s@%s#%s")
+(defvar mew-nntp-folder "+#nntp/%s@%s#%s")
 
 (defun mew-pop-folder (&optional case)
   (format mew-pop-folder
@@ -70,7 +70,7 @@
   (force-mode-line-update))
 
 (defun mew-net-status-clear (buf)
-  (when mew-use-net-status
+  (when (and mew-use-net-status buf (get-buffer buf))
     (save-excursion
       (set-buffer buf)
       (setq mew-summary-buffer-process-status nil)
@@ -235,17 +235,18 @@
 
 (defun mew-net-get-new-message (pnm bnm msg get set)
   (if msg
-      (cons msg (mew-expand-folder bnm msg)) ;; 'get
+      (cons msg (mew-expand-msg bnm msg)) ;; 'get
     (let ((msg (funcall get pnm))
 	  file nxt)
       (cond
        (msg
 	(setq nxt (number-to-string (1+ (string-to-number msg))))
 	(funcall set pnm nxt)
-	(setq file (mew-expand-folder bnm msg)))
+	(setq file (mew-expand-new-msg bnm msg)))
        (t
 	(setq file (mew-folder-new-message bnm))
-	(setq msg (file-name-nondirectory file))
+	;; 1.eml => 1
+	(setq msg (file-name-sans-extension (file-name-nondirectory file)))
 	(setq nxt (number-to-string (1+ (string-to-number msg))))
 	(funcall set pnm nxt)))
       (cons msg file))))
@@ -259,7 +260,8 @@
   (if (not (file-exists-p (concat file mew-queue-work-suffix)))
       file
     (let* ((dir (file-name-directory file))
-	   (num (file-name-nondirectory file))
+	   ;; 1.eml => 1
+	   (num (file-name-sans-extension (file-name-nondirectory file)))
 	   (n (1+ (string-to-number num))))
       (while (or (file-exists-p (format "%s%d" dir n))
 		 (file-exists-p (format "%s%d%s" dir n mew-queue-work-suffix)))
@@ -296,7 +298,7 @@
     (rename-file iwrk info 'override)))
 
 (defun mew-queue-insert-file (pnm n qfld msg) ;; 1
-  (let* ((file (mew-expand-folder qfld msg))
+  (let* ((file (mew-expand-new-msg qfld msg))
 	 (work (concat file mew-queue-work-suffix)) ;; 1.wrk
 	 (info (concat file mew-queue-info-suffix)) ;; 1.mqi
 	 (iwrk (concat info mew-queue-work-suffix)) ;; 1.mqi.wrk
@@ -442,17 +444,35 @@
       (mew-rendezvous mew-summary-buffer-process))))
 
 (defun mew-summary-retrieve (&optional no-flush)
-  "Retrieve messages to +inbox asynchronously according to
-'mew-mailbox-type'. 
-
+  "In local folders, retrieve messages to +inbox asynchronously 
+according to 'mew-mailbox-type' and 'mew-case'. 
 If 'mew-auto-flush-queue' is non-nil, +queue is flushed.
-If called with '\\[universal-argument]', +queue is not flushed."
+If called with '\\[universal-argument]', +queue is not flushed.
+
+In remote folders, visit an inbox folder and scan with 'update."
   (interactive "P")
-  (let* ((case mew-case)
-	 (proto (mew-proto case))
-	 (inbox (mew-proto-inbox-folder proto case))
-	 (case:inbox (mew-case-folder case inbox))
-	 mailbox)
+  (let (proto case inbox case:inbox mailbox case:folder)
+    (cond
+     ((mew-virtual-p)
+      (cond
+       ((setq case:folder (or (mew-vinfo-get-parent-folder) ;; Thread
+			      (car (mew-vinfo-get-flds)))) ;; Virtual by "/"
+	(setq proto (mew-case:folder-folder case:folder))
+	(setq case (or (mew-case:folder-case case:folder)
+                       mew-case)))
+       (t ;; Virtual by "k/"
+	(setq proto (mew-proto mew-case))
+	(setq case mew-case))))
+     (t
+      (setq proto (mew-sinfo-get-folder)) ;; folder is proto
+      (setq case (or (mew-sinfo-get-case) ;; remote
+		     mew-case)))) ;; local
+    (setq inbox (mew-proto-inbox-folder proto case))
+    (cond
+     ((mew-folder-remotep proto)
+      (setq case:inbox (mew-case-folder case inbox)))
+     (t ;; local
+      (setq case:inbox inbox)))
     ;; for mew-summary-exchange-point.
     (mew-sinfo-set-ret-pos (point))
     (mew-summary-visit-folder case:inbox)
@@ -527,8 +547,11 @@ If called with '\\[universal-argument]', +queue is not flushed."
 	      (setq opts (list "-e" mbox-command "-m" mbox-command-arg))
 	    (setq dir (mew-expand-folder inbox))
 	    (setq mode (mew-get-file-modes dir))
-	    (setq opts (list "-e" mbox-command
-			     "-m" (format "%s -p %d" mbox-command-arg mode))))
+	    (if mew-use-suffix
+		(setq opts (list "-e" mbox-command
+				 "-m" (format "%s -o -p %d" mbox-command-arg mode)))
+	      (setq opts (list "-e" mbox-command
+			       "-m" (format "%s -p %d" mbox-command-arg mode)))))
 	(setq opts (list "-e" mbox-command)))
       (mew-local-retrieve 'inc inbox flush opts))))
 
@@ -539,30 +562,78 @@ If called with '\\[universal-argument]', +queue is not flushed."
 
 (defun mew-summary-get-inherit-case (fld msg)
   (let ((buf (mew-cache-hit fld msg))
-	uidl delbuf)
-    (unless buf
-      (save-excursion
-	(setq buf (generate-new-buffer mew-buffer-prefix))
-	(setq delbuf t)
-	(set-buffer buf)
-	(mew-erase-buffer)
-	(mew-insert-message fld msg
-			    mew-cs-text-for-read mew-header-reasonable-size)))
-    (save-excursion
-      (set-buffer buf)
-      (setq uidl (mew-header-get-value mew-x-mew-uidl:))
-      (prog1
-	  (when (and uidl (string-match "^[^ ]+ [0-9]+ \\(.+\\)$" uidl))
-	    (mew-match-string 1 uidl))
-	(if delbuf (mew-remove-buffer buf))))))
+	uid)
+    (if buf
+	(save-excursion
+	  (set-buffer buf)
+	  (setq uid (mew-header-get-value mew-x-mew-uidl:)))
+      (with-temp-buffer
+	(mew-insert-message fld msg mew-cs-text-for-read mew-header-reasonable-size)
+	(setq uid (mew-header-get-value mew-x-mew-uidl:))))
+    (mew-scan-uid-case uid)))
+
+(defun mew-net-thread-info (msgs)
+  (let ((virtual (mew-summary-folder-name 'ext))
+	(summary (mew-summary-folder-name)))
+    (list virtual summary msgs)))
+
+(defun mew-net-thread-info-get-virtual (thread-info)
+  (nth 0 thread-info))
+
+(defun mew-net-thread-info-get-summary (thread-info)
+  (nth 1 thread-info))
+
+(defun mew-net-thread-info-get-msg (thread-info)
+  (nth 2 thread-info))
+
+(defun mew-net-disp-info ()
+  (let* ((fid (mew-frame-id))
+	 (buf (current-buffer))
+	 (fld (mew-current-get-fld fid))
+	 (msg (mew-current-get-msg fid))
+	 (disp (mew-sinfo-get-disp-msg)))
+    (if disp (list fid buf fld msg))))
+
+(defun mew-net-disp-info-fid (disp-info)
+  (nth 0 disp-info))
+
+(defun mew-net-disp-info-buf (disp-info)
+  (nth 1 disp-info))
+
+(defun mew-net-disp-info-fld (disp-info)
+  (nth 2 disp-info))
+
+(defun mew-net-disp-info-msg (disp-info)
+  (nth 3 disp-info))
+
+(defun mew-net-disp-info-display (disp-info)
+  (when disp-info
+    (let ((fid (mew-net-disp-info-fid disp-info))
+	  (buf (mew-net-disp-info-buf disp-info))
+	  (fld (mew-net-disp-info-fld disp-info))
+	  (msg (mew-net-disp-info-msg disp-info))
+	  (nfid (mew-frame-id))
+	  m)
+      (when (and (equal fid nfid)
+		 (equal fld (mew-current-get-fld nfid)))
+	(save-excursion
+	  (set-buffer buf)
+	  (setq m (point-marker))
+	  (set-marker-insertion-type m t)
+	  (when (and (mew-sinfo-get-disp-msg)
+		     (equal msg (mew-current-get-msg nfid))
+		     (mew-summary-search-msg msg))
+	    ;; The message was truncated probaby because it's too large.
+	    (mew-summary-analyze-again))
+	  (goto-char m))))))
 
 (defun mew-summary-retrieve-message (&optional rev-del)
   "Retrieve the rest of a truncated('T') message.
 
-In a LOCAL folder: a method to retrieve the message is determined by
-'mew-input-case' and 'mew-mailbox-type'. If
-'mew-pop-delete'/'mew-imap-delete' is non-nil, delete the message from
-the mailbox. When executed with '\\[universal-argument]',
+In a LOCAL folder: a method to retrieve the message is determined
+by 'mew-case' and 'mew-mailbox-type'. 
+If 'mew-pop-delete'/'mew-imap-delete' is non-nil, delete the message
+from the mailbox. When executed with '\\[universal-argument]',
 'mew-pop-delete'/'mew-imap-delete' is considered reversed.
 
 In a REMOTE folder: case and protocol are determined by the folder.
@@ -574,10 +645,9 @@ The message in the server side is always retained."
 	    (uid (mew-sumsyn-message-uid))
 	    (siz (mew-sumsyn-message-size))
 	    (bnm (mew-summary-folder-name))
-	    (cbuf (mew-summary-folder-name 'ext)) 
-	    folder case del rtr rtrs mailbox thread-info)
-       (when (mew-thread-p)
-	 (setq thread-info (list cbuf bnm msg)))
+	    (disp-info (mew-net-disp-info))
+	    (thread-info (if (mew-thread-p) (mew-net-thread-info (list msg))))
+	    folder case del rtr rtrs mailbox)
        (save-excursion
 	 (set-buffer bnm)
 	 (when (mew-summary-exclusive-p)
@@ -590,7 +660,7 @@ The message in the server side is always retained."
 	      ((eq mailbox 'pop)
 	       (setq del (eq (mew-pop-delete case) t))) ;; delete may be number
 	      ((eq mailbox 'imap)
-	       (setq del (eq (mew-imap-delete case) t))))	;; delete may be number
+	       (setq del (eq (mew-imap-delete case) t)))) ;; delete may be number
 	     (if rev-del (setq del (not del))))
 	   (when (and uid (not (string= uid "")))
 	     (setq rtr (list uid siz del (list bnm msg)))
@@ -603,57 +673,58 @@ The message in the server side is always retained."
 	    ((mew-folder-localp folder)
 	     (cond
 	      ((eq mailbox 'pop)
-	       (mew-pop-retrieve case 'get bnm rtrs thread-info))
+	       (mew-pop-retrieve case 'get bnm rtrs thread-info disp-info))
 	      ((eq mailbox 'imap)
-	       (mew-imap-retrieve case 'get bnm rtrs thread-info))))
+	       (mew-imap-retrieve case 'get bnm rtrs thread-info disp-info))))
 	    ((mew-folder-popp folder)
-	     (mew-pop-retrieve case 'get bnm rtrs thread-info))
+	     (mew-pop-retrieve case 'get bnm rtrs thread-info disp-info))
 	    ((mew-folder-imapp folder)
-	     (mew-imap-retrieve case 'get bnm rtrs thread-info))
+	     (mew-imap-retrieve case 'get bnm rtrs thread-info disp-info))
 	    ((mew-folder-nntpp folder)
-	     (mew-nntp-retrieve case 'get bnm rtrs thread-info)))))))))
+	     (mew-nntp-retrieve case 'get bnm rtrs thread-info disp-info)))))))))
 
 (defun mew-summary-retrieve-message-for-thread (thread-info)
-  (let ((to-buf (nth 0 thread-info))
-	(from-buf (nth 1 thread-info))
-	(msg (nth 2 thread-info))
-	str start beg end idt-str level length)
+  (let ((virtual (mew-net-thread-info-get-virtual thread-info))
+	(summary (mew-net-thread-info-get-summary thread-info))
+	(msgs    (mew-net-thread-info-get-msg     thread-info))
+	msg str start beg end idt-str level length)
     (save-excursion
-      (set-buffer from-buf)
-      (save-excursion
-	(goto-char (point-min))
-	(re-search-forward (mew-regex-sumsyn-msg msg) nil t)
-	(beginning-of-line)
-	(setq start (point))
-	(forward-line)
-	(setq str (buffer-substring start (point))))
-      (set-buffer to-buf)
-      (save-excursion
-	(goto-char (point-min))
-	(re-search-forward (mew-regex-sumsyn-msg msg) nil t)
-	(beginning-of-line)
-	(setq start (point))
-	(setq beg (mew-thread-next-property start))
-	(setq level (mew-thread-get-property beg))
-	(setq length (* mew-thread-indent-length level))
-	(setq end (+ beg length))
-	(setq idt-str (buffer-substring beg end))
-	(forward-line)
-	(mew-elet
-	 (delete-region start (point))
-	 (insert str)
-	 (goto-char beg)
-	 (if (= level 0) (setq end (1+ end)))
-	 (insert idt-str)
-	 (mew-thread-put-property beg end level)))
+      (while msgs
+	(setq msg (car msgs))
+	(setq msgs (cdr msgs))
+	(set-buffer summary)
+	;; Summary
+	(save-excursion
+	  (when (mew-summary-search-msg msg)
+	    (setq start (point))
+	    (forward-line)
+	    (setq str (buffer-substring start (point)))))
+	(set-buffer virtual)
+	;; Virtual
+	(save-excursion
+	  (when (mew-summary-search-msg msg)
+	    (setq start (point))
+	    (setq beg (mew-thread-next-property start))
+	    (setq level (mew-thread-get-property beg))
+	    (setq length (* mew-thread-indent-length level))
+	    (setq end (+ beg length))
+	    (setq idt-str (buffer-substring beg end))
+	    (forward-line)
+	    (mew-elet
+	     (delete-region start (point))
+	     (insert str)
+	     (goto-char beg)
+	     (if (= level 0) (setq end (1+ end)))
+	     (insert idt-str)
+	     (mew-thread-put-property beg end level)))))
       (mew-summary-unlock))))
 
 (defun mew-summary-mark-retrieve-message (&optional rev-del)
   "Retrieve the rest of truncated('T') messages marked with '*'.
 
 In a LOCAL folder: a method to retrieve the messages is determined by
-'mew-input-case' and 'mew-mailbox-type'.  If
-'mew-pop-delete'/'mew-imap-delete' is non-nil, delete the messages
+'mew-case' and 'mew-mailbox-type'.  
+If 'mew-pop-delete'/'mew-imap-delete' is non-nil, delete the messages
 from the mailbox.  When executed with '\\[universal-argument]',
 'mew-pop-delete'/'mew-imap-delete' is considered reversed.
 
@@ -704,17 +775,17 @@ The messages in the server side is always retained."
        (cond
 	((mew-folder-localp folder)
 	 (while case-rtrs
-	 (setq case (nth 0 (car case-rtrs)))
-	 (setq mailbox (nth 1 (car case-rtrs)))
-	 (setq rtrs (nth 2 (car case-rtrs)))
-	 (setq case-rtrs (cdr case-rtrs))
-	 (cond
-	  ((eq mailbox 'pop)
-	   (mew-pop-retrieve case 'get bnm rtrs))
-	  ((eq mailbox 'imap)
-	   (mew-imap-retrieve case 'get bnm rtrs)))
-	 (when case-rtrs
-	   (mew-rendezvous mew-summary-buffer-process))))
+	   (setq case (nth 0 (car case-rtrs)))
+	   (setq mailbox (nth 1 (car case-rtrs)))
+	   (setq rtrs (nth 2 (car case-rtrs)))
+	   (setq case-rtrs (cdr case-rtrs))
+	   (cond
+	    ((eq mailbox 'pop)
+	     (mew-pop-retrieve case 'get bnm rtrs))
+	    ((eq mailbox 'imap)
+	     (mew-imap-retrieve case 'get bnm rtrs)))
+	   (when case-rtrs
+	     (mew-rendezvous mew-summary-buffer-process))))
 	((mew-folder-popp folder)
 	 (mew-pop-retrieve case 'get bnm rtrs))
 	((mew-folder-imapp folder)
@@ -784,7 +855,7 @@ The messages in the server side is always retained."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2005 Mew developing team.
+;; Copyright (C) 2002-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
