@@ -2,7 +2,7 @@
 ;;
 ;; mew-nmz.el --- Namazu interfaces for Mew
 ;;
-;;   Author: Hideyuki SHIRAI <shirai@mew.org>
+;;   Author: Hideyuki SHIRAI
 ;;   Created: Dec 24, 2004
 
 (require 'mew)
@@ -58,18 +58,16 @@
   :type 'file)
 
 (defcustom mew-nmz-mknmz-skip-folders-regexp
-  `(,(regexp-quote mew-draft-folder)
-    ,(regexp-quote mew-trash-folder)
-    ,(regexp-quote mew-queue-folder)
-    ,(regexp-quote mew-attach-folder)
-    ,(regexp-quote mew-imap-queue-folder)
-    ,(regexp-quote mew-imap-trash-folder)
-    ,(regexp-quote mew-postq-folder))
+  (mapcar 'regexp-quote
+	  (delq nil
+		`(,mew-draft-folder ,mew-trash-folder ,mew-queue-folder ,mew-attach-folder
+		  ,mew-imap-queue-folder ,mew-imap-trash-folder ,mew-postq-folder
+		  ,@mew-trash-folder-list ,@mew-imap-trash-folder-list)))
   "*Folders regexp to skip the index creating."
   :group 'mew-nmz
   :type '(repeat regexp))
 
-(defcustom mew-nmz-mknmz-use-mode-line t
+(defcustom mew-nmz-mknmz-use-mode-line nil
   "*Display indicator of namazu in mode line."
   :group 'mew-nmz
   :type 'boolean)
@@ -137,6 +135,11 @@ This value MUST be less then equal `INDEX_MAX' of libnamazu.h."
   :group 'mew-nmz
   :type 'character)
 
+(defcustom mew-ask-virtual-folder-name nil
+  "*If *non-nil*, ask a virtual folder name."
+  :group 'mew-summary
+  :type 'boolean)
+
 (defcustom mew-nmz-cache-file-prefix ".mew-nmz-"
   "*Prefix of the cache filename.  Expand with mew-conf-path when use it."
   :group 'mew-nmz
@@ -221,7 +224,7 @@ This value MUST be less then equal `INDEX_MAX' of libnamazu.h."
   (concat "^\\(.*\\)" (regexp-quote mew-path-separator) "\\([0-9]+\\)"))
 
 (defconst mew-nmz-use-drive-letter
-  (memq system-type '(OS/2 emx windows-nt)))
+  (memq system-type '(OS/2 emx windows-nt cygwin)))
 
 (defun mew-nmz-original-message (&optional arg)
   "Jump to physical folder and message from virtual folder.
@@ -290,7 +293,7 @@ If executed with '\\[universal-argument]', move to next virtual folder include s
 
 ;;
 ;; "Make Index" functions.
-(defun mew-nmz-mknmz (&optional fld all)
+(defun mew-nmz-mknmz (&optional fld all disp)
   "Make namazu index for mew-nmz.
 If executed with '\\[universal-argument]', make indices without the check of timestamp files.
 If executed with '\\[universal-argument] 0', remove indices before make index."
@@ -300,7 +303,7 @@ If executed with '\\[universal-argument] 0', remove indices before make index."
 			    (or (mew-sinfo-get-folder) mew-inbox-folder)))
 	 nil))
   (save-excursion
-    (let ((msgenb (interactive-p))
+    (let ((msgenb (or (interactive-p) disp))
 	  (force (and current-prefix-arg
 		      (not (eq current-prefix-arg 0))))
 	  (remove (eq current-prefix-arg 0))
@@ -354,7 +357,7 @@ If executed with '\\[universal-argument] 0', remove indices before make index."
 	(set-buffer (get-buffer-create bufname))
 	(erase-buffer)
 	;; folder set
-	(and msgenb (message "Mew mknmz (%s)..." fld))
+	(and msgenb (message "Namazu indexing for %s..." fld))
 	(mew-nmz-timestamp-new fld)
 	(mew-piolet
 	 mew-cs-autoconv mew-cs-text-for-write
@@ -405,7 +408,7 @@ If executed with '\\[universal-argument] 0', remove indices before make index."
 	      (error nil)))
 	(setq success t)
 	(mew-nmz-timestamp-rename fld)
-	(setq msg (format "Mew mknmz (%s)...done" fld))
+	(setq msg (format "Namazu indexing %s...done" fld))
 	(when mew-nmz-setup-p
 	  (setq fld (mew-nmz-case-folder-normalize fld))
 	  (unless (mew-nmz-folder-to-nmzdir fld)
@@ -466,6 +469,7 @@ If executed with '\\[universal-argument] 0', remove indices before make index."
       (message "gcnmz cannot run on %s" fld))))
 
 (defun mew-nmz-gcnmz-sentinel (process event)
+  ;; xxx save-excursion
   (when (buffer-name (process-buffer process))
     (set-buffer (process-buffer process))
     (let ((fld mew-nmz-mknmz-process-folder))
@@ -922,6 +926,15 @@ then put the '*' mark onto them. "
 	(setq ret init))
       (setq ret (mapcar 'mew-chop (mew-split ret ?,))))))
 
+(defun mew-summary-get-vfolder (flds)
+  (let ((vfolder (mew-folder-to-virtual (mapconcat 'identity flds ","))))
+    (when mew-ask-virtual-folder-name
+      (setq vfolder (mew-input-string
+                     "Virtual folder name %s(%s): " "" vfolder)))
+    (unless (mew-folder-virtualp vfolder)
+      (setq vfolder (mew-folder-to-virtual vfolder)))
+    vfolder))
+
 (defun mew-nmz-virtual ()
   "Another virtual mode with namazu."
   (interactive)
@@ -1668,8 +1681,8 @@ If executed with '\\[universal-argument]', search result indexes."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2005 Hideyuki SHIRAI <shirai@mew.org>
-;; Copyright (C) 1994-2005 Mew developing team.
+;; Copyright (C) 1999-2005 Hideyuki SHIRAI
+;; Copyright (C) 1994-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
