@@ -30,7 +30,7 @@
     "size" "get-body" "no-msg" "case" "msgdb" "done" "dispatched" "error"
     "max"
     "newsgroup" "msgid" "truncated"
-    "thread-info" "status-buf"))
+    "thread-info" "disp-info" "status-buf"))
 
 (mew-info-defun "mew-nntp-" mew-nntp-info-list)
 
@@ -107,7 +107,7 @@
 	(refs (mew-nntp-get-refs pnm)) ;; (uid siz del (+fld msg))
 	(bnm (mew-nntp-get-bnm pnm))
 	(range (mew-nntp-get-range pnm))
-	max)
+	max first last)
     (if (and mew-nntp-skip-uidl (eq directive 'get))
 	(mew-nntp-command-dispatch pro pnm directive refs nil)
       (mew-net-status (mew-nntp-get-status-buf pnm)
@@ -116,21 +116,26 @@
        ((eq directive 'scan)
 	(if (eq range nil) ;; update
 	    (setq max (mew-lisp-load
-		       (mew-expand-folder bnm mew-nntp-msgid-file))))))
-      (if (re-search-forward "^[0-9]+ +[0-9]+ +\\([0-9]+\\) +[0-9]+ +[-.a-zA-Z0-9]+" nil t)
+		       (mew-expand-file bnm mew-nntp-msgid-file))))))
+      ;; 221 total first last newsgroup (xxx)
+      (if (re-search-forward "^[0-9]+ +[0-9]+ +\\([0-9]+\\) +\\([0-9]+\\) +[-.a-zA-Z0-9]+" nil t)
 	  (progn
+	    (setq first (string-to-number (mew-match-string 1)))
+	    (setq last (string-to-number (mew-match-string 2)))
 	    (cond
 	     ((stringp max)
-	      (setq max (string-to-number max)))
+	      (setq max (1+ (string-to-number max))))
 	     (max ;; backward compatibility
 	      ;; reversed
 	      (setq max (car max))
-	      (setq max (string-to-number max)))
+	      (setq max (1+ (string-to-number max))))
+	     ((and (eq directive 'scan) (numberp range))
+	      (setq max (1+ (- last range)))
+	      (if (< max first) (setq max first)))
 	     (t
-	      (setq max (string-to-number (mew-match-string 1)))
-	      (setq max (1- max))))
+	      (setq max first)))
 	    (mew-nntp-set-max pnm max)
-	    (mew-nntp-process-send-string pro "XOVER %d-" (1+ max)))
+	    (mew-nntp-process-send-string pro "XOVER %d-" max))
 	(mew-nntp-set-status pnm "quit")
 	(mew-nntp-command-quit pro pnm)))))
 
@@ -158,10 +163,10 @@
        ((eq directive 'sync)
 	(if uid (setq hlds (cons uid hlds))))))
     (mew-nntp-set-msgid pnm (nth 0 (car rtrs))) ;; 'scan
-    ;; last:n
-    (when (and (eq directive 'scan) (numberp range))
-      (if (> (length rtrs) range)
-	  (setcdr (nthcdr (1- range) rtrs) nil)))
+    ;; last:n xxx
+;;    (when (and (eq directive 'scan) (numberp range))
+;;      (if (> (length rtrs) range)
+;;	  (setcdr (nthcdr (1- range) rtrs) nil)))
     (setq rtrs (nreverse rtrs))
     (setq hlds (nreverse hlds))
     (mew-nntp-command-dispatch pro pnm directive rtrs hlds)))
@@ -248,7 +253,7 @@
     (cond
      ((eq directive 'scan)
       (setq msg uid) 
-      (setq file (mew-expand-folder fld msg)))
+      (setq file (mew-expand-new-msg fld msg)))
      (t
       (setq msg-file (mew-net-get-new-message
 		      pnm fld msg 'mew-nntp-get-msgdb 'mew-nntp-set-msgdb))
@@ -259,21 +264,24 @@
       (mew-header-insert-xmu uid siz nil))
     (catch 'write-error
       (condition-case nil
-	  (mew-frwlet 
-	   mew-cs-dummy mew-cs-text-for-write
-	   (write-region (point-min) (point-max) file nil 'no-msg))
+	  (let ((write-region-inhibit-fsync mew-use-async-write))
+	    (mew-frwlet 
+	     mew-cs-dummy mew-cs-text-for-write
+	     (write-region (point-min) (point-max) file nil 'no-msg)))
 	(error
 	 (mew-nntp-set-status pnm "quit")
 	 (mew-nntp-command-quit pro pnm)
 	 (throw 'write-error nil)))
-      (mew-set-file-modes file)
-      (mew-set-buffer-multibyte t)
-      (setq vec (mew-scan-header))
-      (mew-scan-set-folder vec fld)
-      (mew-scan-set-message vec msg)
-      (mew-scan-body vec)
-      (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line fld vec width lmsg)
+      (when (file-exists-p file)
+	(mew-set-file-modes file)
+	(mew-set-file-type file)
+	(mew-set-buffer-multibyte t)
+	(setq vec (mew-scan-header))
+	(mew-scan-set-folder vec fld)
+	(mew-scan-set-message vec msg)
+	(mew-scan-body vec)
+	(mew-set-buffer-multibyte nil)
+	(mew-scan-insert-line fld vec width lmsg))
       (mew-nntp-command-next-article pro pnm))))
 
 (defun mew-nntp-command-next-article (pro pnm)
@@ -387,7 +395,7 @@
 	pro tm)
     (condition-case emsg
 	(progn
-	  (setq tm (mew-timer mew-nntp-timeout-time 'mew-nntp-timeout))
+	  (setq tm (run-at-time mew-nntp-timeout-time nil 'mew-nntp-timeout))
 	  (or no-msg (message "Connecting to the NNTP server..."))
 	  (setq pro (open-network-stream pnm nil server sprt))
 	  (process-kill-without-query pro)
@@ -422,7 +430,7 @@
 	 (buf (get-buffer-create (mew-nntp-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
 	 process sshname sshpro sslname sslpro lport tls
-	 thread-info to-buf)
+	 thread-info disp-info virtual)
     (if (mew-nntp-get-process pnm)
 	(message "Another NNTP process is running. Try later")
       (cond
@@ -440,7 +448,7 @@
 	  (setq sslname (process-name sslpro))
 	  (setq lport (mew-ssl-pnm-to-lport sslname))
 	  (when lport
-	    (setq process (mew-nntp-open pnm "localhost" lport no-msg)))))
+	    (setq process (mew-nntp-open pnm mew-ssl-localhost lport no-msg)))))
        (t
 	(setq process (mew-nntp-open pnm server port no-msg))))
       (when process
@@ -448,6 +456,7 @@
 	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
 	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
+	(mew-sinfo-set-scan-md5 nil)
 	(mew-info-clean-up pnm)
 	(mew-nntp-set-no-msg pnm no-msg) ;; must come here
 	(mew-nntp-message pnm "Communicating with the NNTP server...")
@@ -474,11 +483,13 @@
 	  (mew-nntp-set-refs pnm (nth 0 args))
 	  (setq thread-info (nth 1 args))
 	  (mew-nntp-set-thread-info pnm thread-info)
-	  (setq to-buf (nth 0 thread-info))
-	  (when to-buf
-	    (mew-nntp-set-status-buf pnm to-buf)
+	  (setq disp-info (nth 1 args))
+	  (mew-nntp-set-disp-info pnm disp-info)
+	  (setq virtual (mew-net-thread-info-get-virtual thread-info))
+	  (when virtual
+	    (mew-nntp-set-status-buf pnm virtual)
 	    (save-excursion
-	      (set-buffer to-buf)
+	      (set-buffer virtual)
 	      (mew-summary-lock process "NNTPing" (or sshpro sslpro)))))
 	 ((eq directive 'scan)
 	  (mew-nntp-set-range pnm (nth 0 args))
@@ -566,74 +577,80 @@
 	 (msgid (mew-nntp-get-msgid pnm))
 	 (done (mew-nntp-get-done pnm))
 	 (error (mew-nntp-get-error pnm))
-	 (file (mew-expand-folder bnm mew-nntp-msgid-file))
+	 (file (mew-expand-file bnm mew-nntp-msgid-file))
 	 (buf (process-buffer process))
-	 (thread-info (mew-nntp-get-thread-info pnm)))
-    (mew-nntp-debug "NNTP SENTINEL" event)
-    (set-process-buffer process nil)
-    (set-buffer bnm)
-    (mew-summary-mark-recover mdb)
-    (mew-remove-buffer buf)
-    (if (not done)
-	(let* ((rtrs (mew-nntp-get-rtrs pnm))
-	       (lefts (length rtrs))
-	       (msgid (nth 0 (car rtrs)))
-	       recovered)
-	  (mew-nntp-message pnm "NNTP connection is lost")
-	  (when (mew-nntp-get-dispatched pnm)
-	    (cond
-	     ((eq directive 'scan)
-	      (setq msgid (number-to-string (1- (string-to-number msgid))))
-	      (mew-lisp-save file msgid nil 'unlimit)
-	      (setq recovered t)))
-	    (when recovered
-	      (mew-nntp-message
-	       pnm
-	       "%d message retrieved. %d messages are left due to an error"
-	       (- rttl lefts) lefts)
-	      (mew-summary-folder-cache-save))))
-      (if thread-info (mew-summary-retrieve-message-for-thread thread-info))
-      (cond
-       (error 
-	;; retain the error message
-	)
-       ((eq directive 'list)
-	(mew-nntp-message pnm "Collecting newsgroup list...done"))
-       ((eq directive 'sync)
-	(mew-nntp-message pnm "Synchronizing messages...")
-	(mew-net-folder-sync bnm hlds)
-	(mew-nntp-message pnm "Synchronizing messages...done")
-	(mew-summary-folder-cache-save))
-       ((eq directive 'get)
+	 (thread-info (mew-nntp-get-thread-info pnm))
+	 (disp-info (mew-nntp-get-disp-info pnm)))
+    (save-excursion
+      (mew-nntp-debug "NNTP SENTINEL" event)
+      (set-process-buffer process nil)
+      (set-buffer bnm)
+      (mew-summary-mark-recover mdb)
+      (mew-remove-buffer buf)
+      (if (not done)
+	  (let* ((rtrs (mew-nntp-get-rtrs pnm))
+		 (lefts (length rtrs))
+		 (msgid (nth 0 (car rtrs)))
+		 recovered)
+	    (mew-nntp-message pnm "NNTP connection is lost")
+	    (when (mew-nntp-get-dispatched pnm)
+	      (cond
+	       ((eq directive 'scan)
+		(setq msgid (number-to-string (1- (string-to-number msgid))))
+		(mew-lisp-save file msgid nil 'unlimit)
+		(setq recovered t)))
+	      (when recovered
+		(mew-nntp-message
+		 pnm
+		 "%d message retrieved. %d messages are left due to an error"
+		 (- rttl lefts) lefts)
+		(mew-summary-folder-cache-save))))
+	(if thread-info (mew-summary-retrieve-message-for-thread thread-info))
 	(cond
-	 ((= rttl 0)
-	  (mew-nntp-message pnm "No new messages"))
-	 ((= rttl 1)
-	  (mew-nntp-message pnm "1 message retrieved"))
-	 (t
-	  (mew-nntp-message pnm "%d messages retrieved" rttl))))
-       ((eq directive 'scan)
-	(cond
-	 ((= rttl 0)
-	  (mew-nntp-message pnm "No messages scanned"))
-	 ((= rttl 1)
-	  (mew-nntp-message pnm "1 message scanned")
-	  (mew-lisp-save file msgid nil 'unlimit)
+	 (error 
+	  ;; retain the error message
+	  )
+	 ((eq directive 'list)
+	  (mew-nntp-message pnm "Collecting newsgroup list...done"))
+	 ((eq directive 'sync)
+	  (mew-nntp-message pnm "Synchronizing messages...")
+	  (mew-net-folder-sync bnm hlds)
+	  (mew-nntp-message pnm "Synchronizing messages...done")
 	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-nntp-message pnm "%d messages scanned" rttl)
-	  (mew-lisp-save file msgid nil 'unlimit)
-	  (mew-summary-folder-cache-save))))))
-    ;;
-    (mew-net-status-clear (mew-nntp-get-status-buf pnm))
-    (mew-info-clean-up pnm)
-    (set-buffer-modified-p nil)
-    (mew-summary-unlock)
-    (if (and (processp sshpro) (not mew-ssh-keep-connection))
-	(process-send-string sshpro "exit\n"))
-    (if (and (processp sslpro) (not mew-ssl-keep-connection))
-	(delete-process sslpro))
-    (run-hooks 'mew-nntp-sentinel-hook)))
+	 ((eq directive 'get)
+	  (cond
+	   ((= rttl 0)
+	    (mew-nntp-message pnm "No new messages"))
+	   ((= rttl 1)
+	    (mew-nntp-message pnm "1 message retrieved")
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-nntp-message pnm "%d messages retrieved" rttl)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'scan)
+	  (cond
+	   ((= rttl 0)
+	    (mew-nntp-message pnm "No messages scanned"))
+	   ((= rttl 1)
+	    (mew-nntp-message pnm "1 message scanned")
+	    (mew-lisp-save file msgid nil 'unlimit)
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-nntp-message pnm "%d messages scanned" rttl)
+	    (mew-lisp-save file msgid nil 'unlimit)
+	    (mew-summary-folder-cache-save))))))
+      ;;
+      (and (functionp 'unix-sync) (unix-sync))
+      (mew-net-status-clear (mew-nntp-get-status-buf pnm))
+      (mew-info-clean-up pnm)
+      (set-buffer-modified-p nil)
+      (mew-summary-unlock)
+      (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	  (process-send-string sshpro "exit\n"))
+      (if (and (processp sslpro) (not mew-ssl-keep-connection))
+	  (delete-process sslpro))
+      (mew-net-disp-info-display disp-info)
+      (run-hooks 'mew-nntp-sentinel-hook))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -658,9 +675,9 @@
 
 (defun mew-nntp-folder-load (case &optional two)
   (let* ((fld (mew-nntp-folder case))
-	 (file (mew-expand-folder fld mew-nntp-folder-alist-file))
+	 (file (mew-expand-file fld mew-nntp-folder-alist-file))
 	 (groups (mew-lisp-load file))
-	 (file2 (mew-expand-folder fld mew-nntp-folder-alist2-file))
+	 (file2 (mew-expand-file fld mew-nntp-folder-alist2-file))
 	 (groups2 (mew-lisp-load file2)))
     (mew-nntp-folder-alist-set case groups)
     (mew-nntp-folder-alist2-set case groups2)
@@ -698,7 +715,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2005 Mew developing team.
+;; Copyright (C) 1999-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
