@@ -29,7 +29,7 @@
     "size" "truncated" "get-body"
     "flush" "no-msg" "msgdb" "done" "dispatched" "error"
     "body-lines" "delete" "case"
-    "thread-info" "status-buf"))
+    "thread-info" "disp-info" "status-buf"))
 
 (mew-info-defun "mew-pop-" mew-pop-info-list)
 
@@ -77,7 +77,7 @@
 		  "Auth'ing"
 		  nil
 		  (mew-pop-secure-p pnm))
-  (if (re-search-forward "<[^<>]+@[^<>]+>" nil t)
+  (if (re-search-forward "<[=!-;?-~]+@[=!-;?-~]+>" nil t)
       (mew-pop-set-key pnm (mew-match-string 0)))
   (mew-pop-process-send-string pro "CAPA"))
 
@@ -241,7 +241,7 @@
      ((eq directive 'scan)
       (if (eq range nil) ;; update
 	  (setq old-uidl (mew-lisp-load
-			  (mew-expand-folder bnm mew-pop-msgid-file))))))
+			  (mew-expand-file bnm mew-pop-msgid-file))))))
     (while (re-search-forward "^\\([0-9]+\\) +\\([!-~]*\\)" nil t)
       (setq left (1+ left))
       (setq num (mew-match-string 1))
@@ -415,7 +415,7 @@
 	 (fld-msg (nth 4 rtr))
 	 (truncated (mew-pop-get-truncated pnm))
 	 (case (mew-pop-get-case pnm))
-	 fld msg vec file msg-file lmsg mark)
+	 fld msg vec file msg-file lmsg mark folder)
     (cond
      ((null fld-msg)
       (setq fld (mew-pop-get-bnm pnm)))
@@ -424,6 +424,7 @@
      ((listp fld-msg)
       (mew-set '(fld msg) fld-msg)
       (setq lmsg msg)))
+    (setq folder (mew-case:folder-folder fld))
     ;; deleting +OK
     (goto-char (point-min))
     (forward-line)
@@ -442,33 +443,36 @@
 						mew-case-default))))
      ((eq directive 'scan)
       (mew-header-insert-xmu uid siz nil))
-     ((and (eq directive 'get) (mew-folder-popp (mew-case:folder-folder fld)))
+     ((and (eq directive 'get) (mew-folder-popp folder))
       (mew-header-insert-xmu uid siz nil)))
     (catch 'write-error
       (condition-case nil
-	  (mew-frwlet 
-	   mew-cs-dummy mew-cs-text-for-write
-	   (write-region (point-min) (point-max) file nil 'no-msg))
+	  (let ((write-region-inhibit-fsync mew-use-async-write))
+	    (mew-frwlet 
+	     mew-cs-dummy mew-cs-text-for-write
+	     (write-region (point-min) (point-max) file nil 'no-msg)))
 	(error
 	 (mew-pop-set-status pnm "quit")
 	 (mew-pop-command-quit pro pnm)
 	 (throw 'write-error nil)))
-      (mew-set-file-modes file)
       (mew-pop-set-rcnt pnm (1+ (mew-pop-get-rcnt pnm)))
-      ;;
-      (mew-set-buffer-multibyte t)
-      (setq vec (mew-scan-header))
-      (mew-scan-set-folder vec fld)
-      (mew-scan-set-message vec msg)
-      (if (or (eq directive 'inc) (eq directive 'scan))
-	  (setq mark (mew-scan-inbox-action vec case)))
-      (if (and mark
-	       (eq directive 'scan)
-	       (not (memq mark (list mew-mark-delete mew-mark-unlink))))
-	  (setq mark nil))
-      (mew-scan-body vec)
-      (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line fld vec width lmsg mark)
+      (when (file-exists-p file)
+	(mew-set-file-modes file)
+	(mew-set-file-type file)
+	;;
+	(mew-set-buffer-multibyte t)
+	(setq vec (mew-scan-header))
+	(mew-scan-set-folder vec fld)
+	(mew-scan-set-message vec msg)
+	(if (or (eq directive 'inc) (eq directive 'scan))
+	    (setq mark (mew-scan-inbox-action vec case)))
+	(if (and mark
+		 (eq directive 'scan)
+		 (stringp mark)) ;; in the case of refiling
+	    (setq mark nil))
+	(mew-scan-body vec)
+	(mew-set-buffer-multibyte nil)
+	(mew-scan-insert-line fld vec width lmsg mark))
       (mew-pop-set-rtrs pnm (cdr rtrs))
       (if (and del (not truncated))
 	  (progn
@@ -609,7 +613,7 @@
 	pro tm)
     (condition-case emsg
 	(progn
-	  (setq tm (mew-timer mew-pop-timeout-time 'mew-pop-timeout))
+	  (setq tm (run-at-time mew-pop-timeout-time nil 'mew-pop-timeout))
 	  (or no-msg (message "Connecting to the POP server..."))
 	  (setq pro (open-network-stream pnm nil server sprt))
 	  (process-kill-without-query pro)
@@ -646,7 +650,7 @@
 	 (buf (get-buffer-create (mew-pop-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
 	 process sshname sshpro sslname sslpro lport tls
-	 thread-info to-buf)
+	 thread-info disp-info virtual)
     (if (mew-pop-get-process pnm)
 	(message "Another POP process is running. Try later")
       (cond
@@ -664,7 +668,7 @@
 	  (setq sslname (process-name sslpro))
 	  (setq lport (mew-ssl-pnm-to-lport sslname))
 	  (when lport
-	    (setq process (mew-pop-open pnm "localhost" lport no-msg)))))
+	    (setq process (mew-pop-open pnm mew-ssl-localhost lport no-msg)))))
        (proxysrv
 	(setq process (mew-pop-open pnm proxysrv proxyport no-msg)))
        (t
@@ -676,6 +680,7 @@
 	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
 	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
+	(mew-sinfo-set-scan-md5 nil)
 	(mew-info-clean-up pnm)
 	(mew-pop-set-no-msg pnm no-msg) ;; must come here
 	(mew-pop-message pnm "Communicating with the POP server...")
@@ -711,11 +716,13 @@
 	  (mew-pop-set-refs pnm (nth 0 args))
 	  (setq thread-info (nth 1 args))
 	  (mew-pop-set-thread-info pnm thread-info)
-	  (setq to-buf (nth 0 thread-info))
-	  (when to-buf
-	    (mew-pop-set-status-buf pnm to-buf)
+	  (setq disp-info (nth 2 args))
+	  (mew-pop-set-disp-info pnm disp-info)
+	  (setq virtual (mew-net-thread-info-get-virtual thread-info))
+	  (when virtual
+	    (mew-pop-set-status-buf pnm virtual)
 	    (save-excursion
-	      (set-buffer to-buf)
+	      (set-buffer virtual)
 	      (mew-summary-lock process "POPing" (or sshpro sslpro)))))
 	 ((eq directive 'scan)
 	  (mew-pop-set-range pnm (nth 0 args))
@@ -808,111 +815,115 @@
 	 (uidl (mew-pop-get-uidl pnm))
 	 (done (mew-pop-get-done pnm))
  	 (error (mew-pop-get-error pnm))
-	 (file (mew-expand-folder bnm mew-pop-msgid-file))
+	 (file (mew-expand-file bnm mew-pop-msgid-file))
 	 (buf (process-buffer process))
-	 (thread-info (mew-pop-get-thread-info pnm)))
-    (mew-pop-debug "POP SENTINEL" event)
-    (set-process-buffer process nil)
-    (set-buffer bnm)
-    (mew-summary-mark-recover mdb)
-    (mew-remove-buffer buf)
-    (if (not done)
-	(let* ((rtrs (mew-pop-get-rtrs pnm))
-	       (lefts (length rtrs))
-	       (uid (nth 1 (car rtrs)))
-	       recovered)
-	  (mew-pop-message pnm "POP connection is lost")
-	  (when (mew-pop-get-dispatched pnm)
-	    (cond
-	     ((eq directive 'scan)
-	      ;; uidl is reversed.
-	      (setq uidl (cdr (member uid uidl)))
-	      (mew-lisp-save file uidl nil 'unlimit)
-	      (setq recovered t))
-	     ((eq directive 'inc)
-	      ;; uidl is reversed.
-	      (setq uid (assoc uid uidl))
-	      (setq uidl (cdr (member uid uidl)))
-	      (mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
-	      (setq recovered t)))
-	    (when recovered
-	      (mew-pop-message
-	       pnm
-	       "%d message retrieved. %d messages are left due to an error"
-	       (- rttl lefts) lefts)
-	      (mew-summary-folder-cache-save))))
-      (if thread-info (mew-summary-retrieve-message-for-thread thread-info))
-      (cond
-       (error 
- 	;; retain the error message
- 	)
-       ((eq directive 'biff)
-	(funcall mew-biff-function rttl))
-       ((eq directive 'sync)
-	(mew-pop-message pnm "Synchronizing messages...")
-	(mew-net-folder-sync bnm hlds)
-	(mew-pop-message pnm "Synchronizing messages...done"))
-       ((eq directive 'inc)
-	(mew-biff-clear)
-	(mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
+	 (thread-info (mew-pop-get-thread-info pnm))
+	 (disp-info (mew-pop-get-disp-info pnm)))
+    (save-excursion
+      (mew-pop-debug "POP SENTINEL" event)
+      (set-process-buffer process nil)
+      (set-buffer bnm)
+      (mew-summary-mark-recover mdb)
+      (mew-remove-buffer buf)
+      (if (not done)
+	  (let* ((rtrs (mew-pop-get-rtrs pnm))
+		 (lefts (length rtrs))
+		 (uid (nth 1 (car rtrs)))
+		 recovered)
+	    (mew-pop-message pnm "POP connection is lost")
+	    (when (mew-pop-get-dispatched pnm)
+	      (cond
+	       ((eq directive 'scan)
+		;; uidl is reversed.
+		(setq uidl (cdr (member uid uidl)))
+		(mew-lisp-save file uidl nil 'unlimit)
+		(setq recovered t))
+	       ((eq directive 'inc)
+		;; uidl is reversed.
+		(setq uid (assoc uid uidl))
+		(setq uidl (cdr (member uid uidl)))
+		(mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
+		(setq recovered t)))
+	      (when recovered
+		(mew-pop-message
+		 pnm
+		 "%d message retrieved. %d messages are left due to an error"
+		 (- rttl lefts) lefts)
+		(mew-summary-folder-cache-save))))
+	(if thread-info (mew-summary-retrieve-message-for-thread thread-info))
 	(cond
-	 ((= rttl 0)
-	  (mew-pop-message2 pnm "No new messages" left))
-	 ((= rttl 1)
-	  (mew-pop-message2 pnm "1 message retrieved" left)
-	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
-	  (mew-summary-folder-cache-save))))
-       ((eq directive 'get)
-	(cond
-	 ((= rttl 0)
-	  (mew-pop-message2 pnm "No new messages" left))
-	 ((= rttl 1)
-	  (mew-pop-message2 pnm "1 message retrieved" left)
-	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
-	  (mew-summary-folder-cache-save))))
-       ((eq directive 'scan)
-	(mew-biff-clear)
-	(cond
-	 ((= rttl 0)
-	  (mew-pop-message pnm "No messages scanned"))
-	 ((= rttl 1)
-	  (mew-pop-message pnm "1 message scanned")
-	  (mew-lisp-save file uidl nil 'unlimit)
-	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-pop-message pnm "%d messages scanned" rttl)
-	  (mew-lisp-save file uidl nil 'unlimit)
-	  (mew-summary-folder-cache-save))))
-       ((eq directive 'exec)
-	(when kils
-	  (mew-mark-exec-unlink bnm kils)
-	  (mew-mark-kill-invisible)
-	  (mew-summary-folder-cache-save))
-	(cond
-	 ((= dttl 0)
-	  (mew-pop-message pnm "No messages deleted"))
-	 ((= dttl 1)
-	  (mew-pop-message pnm "1 message deleted"))
-	 (t
-	  (mew-pop-message pnm "%d messages deleted" dttl))))))
-    ;;
-    (mew-net-status-clear (mew-pop-get-status-buf pnm))
-    (mew-info-clean-up pnm)
-    (set-buffer-modified-p nil)
-    (mew-summary-unlock)
-    (if (and (processp sshpro) (not mew-ssh-keep-connection))
-	(process-send-string sshpro "exit\n"))
-    (if (and (processp sslpro) (not mew-ssl-keep-connection))
-	(delete-process sslpro))
-    (unless (eq directive 'biff)
-      (run-hooks 'mew-pop-sentinel-non-biff-hook))
-    (run-hooks 'mew-pop-sentinel-hook)
-    (when (and mew-auto-flush-queue flush)
-      (mew-smtp-flush-queue mew-case))))
+	 (error 
+	  ;; retain the error message
+	  )
+	 ((eq directive 'biff)
+	  (funcall mew-biff-function rttl))
+	 ((eq directive 'sync)
+	  (mew-pop-message pnm "Synchronizing messages...")
+	  (mew-net-folder-sync bnm hlds)
+	  (mew-pop-message pnm "Synchronizing messages...done"))
+	 ((eq directive 'inc)
+	  (mew-biff-clear)
+	  (mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
+	  (cond
+	   ((= rttl 0)
+	    (mew-pop-message2 pnm "No new messages" left))
+	   ((= rttl 1)
+	    (mew-pop-message2 pnm "1 message retrieved" left)
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'get)
+	  (cond
+	   ((= rttl 0)
+	    (mew-pop-message2 pnm "No new messages" left))
+	   ((= rttl 1)
+	    (mew-pop-message2 pnm "1 message retrieved" left)
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'scan)
+	  (mew-biff-clear)
+	  (cond
+	   ((= rttl 0)
+	    (mew-pop-message pnm "No messages scanned"))
+	   ((= rttl 1)
+	    (mew-pop-message pnm "1 message scanned")
+	    (mew-lisp-save file uidl nil 'unlimit)
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-pop-message pnm "%d messages scanned" rttl)
+	    (mew-lisp-save file uidl nil 'unlimit)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'exec)
+	  (when kils
+	    (mew-mark-exec-unlink bnm kils)
+	    (mew-mark-kill-invisible)
+	    (mew-summary-folder-cache-save))
+	  (cond
+	   ((= dttl 0)
+	    (mew-pop-message pnm "No messages deleted"))
+	   ((= dttl 1)
+	    (mew-pop-message pnm "1 message deleted"))
+	   (t
+	    (mew-pop-message pnm "%d messages deleted" dttl))))))
+      ;;
+      (and (functionp 'unix-sync) (unix-sync))
+      (mew-net-status-clear (mew-pop-get-status-buf pnm))
+      (mew-info-clean-up pnm)
+      (set-buffer-modified-p nil)
+      (mew-summary-unlock)
+      (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	  (process-send-string sshpro "exit\n"))
+      (if (and (processp sslpro) (not mew-ssl-keep-connection))
+	  (delete-process sslpro))
+      (mew-net-disp-info-display disp-info)
+      (unless (eq directive 'biff)
+	(run-hooks 'mew-pop-sentinel-non-biff-hook))
+      (run-hooks 'mew-pop-sentinel-hook)
+      (when (and mew-auto-flush-queue flush)
+	(mew-smtp-flush-queue mew-case)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -929,7 +940,7 @@
       (save-excursion
 	(set-buffer case:inbox)
 	(when (and (mew-summary-exclusive-p 'no-msg)
-		   (and mew-use-cached-passwd
+		   (and (or mew-use-cached-passwd mew-use-master-passwd)
 			(setq passwd (mew-passwd-get-passwd tag))))
 	  (mew-pop-retrieve case 'biff case:inbox passwd))))))
 
@@ -949,7 +960,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2005 Mew developing team.
+;; Copyright (C) 1999-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
