@@ -51,8 +51,9 @@
 effect to this function."
   (let ((mark-delete mew-mark-delete)
 	(mark-review mew-mark-review)
+	(mark-spam   mew-mark-delete)
 	(unread (and mew-use-unread-mark (mew-sinfo-get-unread-mark)))
-	duplicated review id)
+	duplicated spam review id body md5)
     (when mew-summary-form-mark-delete
       (if (mew-characterp mew-summary-form-mark-delete)
 	  (setq mark-delete mew-summary-form-mark-delete))
@@ -61,6 +62,17 @@ effect to this function."
 	    (unless (mew-scan-message-invalidp)
 	      (setq duplicated t))
 	  (mew-sinfo-set-scan-id (cons id (mew-sinfo-get-scan-id))))))
+    (when (and mew-summary-form-mark-spam
+	       (mew-sinfo-get-inboxp))
+      (if (mew-characterp mew-summary-form-mark-spam)
+	  (setq mark-spam mew-summary-form-mark-spam))
+      (setq body (MEW-BODY))
+      (unless (string= body "")
+	(setq md5 (mew-md5 body))
+	(if (member md5 (mew-sinfo-get-scan-md5))
+	    (unless (mew-scan-message-invalidp)
+	      (setq spam t))
+	  (mew-sinfo-set-scan-md5 (cons md5 (mew-sinfo-get-scan-md5))))))
     (when mew-summary-form-mark-review
       (if (mew-characterp mew-summary-form-mark-review)
 	  (setq mark-review mew-summary-form-mark-review))
@@ -75,6 +87,7 @@ effect to this function."
 	    (setq to (cdr to))))))
     (cond
      (duplicated (char-to-string mark-delete))
+     (spam       (char-to-string mark-spam))
      (review     (char-to-string mark-review))
      (unread     (char-to-string mew-mark-unread))
      (t          (char-to-string mew-mark-read)))))
@@ -84,13 +97,20 @@ effect to this function."
   (let ((ct (MEW-CT))
 	(case-fold-search t))
     (cond
-     ((mew-scan-message-invalidp)                  "#")
-     ((mew-scan-message-truncatedp)                "T")
-     ((string-match "Multipart/Signed"    ct)      "S")
-     ((string-match "Multipart/Encrypted" ct)      "E")
-     ((string-match "Application/X-Pkcs7-Mime" ct) "E")
-     ((mew-ct-multipartp ct)                       "M")
-     ((string-match "Message/Partial"     ct)      "P")
+     ((mew-scan-message-invalidp)         "#")
+     ((mew-scan-message-truncatedp)       "T")
+     ((string-match mew-ct-mls  ct)       "S")
+     ((string-match mew-ct-mle  ct)       "E")
+     ((string-match mew-ct-sms  ct)       "E")
+     ((string-match mew-ct-xsms ct)       "E")
+     ((or (string-match mew-ct-smm ct) (string-match mew-ct-xsmm ct))
+      ;; checking smime-type
+      (cond
+       ((string-match mew-ct-smm-sig ct)  "S")
+       ((string-match mew-ct-smm-enc ct)  "E")
+       (t "E"))) ;; smime-type is optional, sigh
+     ((mew-ct-multipartp ct)              "M")
+     ((string-match "Message/Partial" ct) "P")
      (t " "))))
 
 (defun mew-summary-form-time ()
@@ -99,7 +119,7 @@ effect to this function."
     (if (or (string= s "")
 	    (not (string-match mew-time-rfc-regex s)))
 	(setq s (mew-time-ctz-to-rfc
-		 (mew-file-get-time (mew-expand-folder (MEW-FLD) (MEW-NUM))))))
+		 (mew-file-get-time (mew-expand-msg (MEW-FLD) (MEW-NUM))))))
     (if (string-match mew-time-rfc-regex s)
 	(format "%02d:%02d"
 		(or (mew-time-rfc-hour) 0)
@@ -112,7 +132,7 @@ effect to this function."
     (when (or (string= s "")
 	      (not (string-match mew-time-rfc-regex s)))
       (setq s (mew-time-ctz-to-rfc
-	       (mew-file-get-time (mew-expand-folder (MEW-FLD) (MEW-NUM))))))
+	       (mew-file-get-time (mew-expand-msg (MEW-FLD) (MEW-NUM))))))
     (if (string-match mew-time-rfc-regex s)
 	(format "%02d/%02d"
 		(mew-time-mon-str-to-int (mew-time-rfc-mon))
@@ -125,7 +145,7 @@ effect to this function."
     (when (or (string= s "")
 	      (not (string-match mew-time-rfc-regex s)))
       (setq s (mew-time-ctz-to-rfc
-	       (mew-file-get-time (mew-expand-folder (MEW-FLD) (MEW-NUM))))))
+	       (mew-file-get-time (mew-expand-msg (MEW-FLD) (MEW-NUM))))))
     (if (not (string-match mew-time-rfc-regex s))
 	"0000"
       (setq year (mew-time-rfc-year))
@@ -144,7 +164,7 @@ with -4. See also 'mew-summary-form-size-0k' and 'mew-summary-form-size-huge'."
 	(i 0) size unit)
     (if (and SIZE (string-match "^[0-9]+$" SIZE))
 	(setq size (string-to-number SIZE))
-      (setq size (mew-file-get-size (mew-expand-folder (MEW-FLD) (MEW-NUM)))))
+      (setq size (mew-file-get-size (mew-expand-msg (MEW-FLD) (MEW-NUM)))))
     (while (and (< i len-1) (>= size 1000))
       (setq size (/ size 1000))
       (setq i (1+ i)))
@@ -334,8 +354,10 @@ of thread indent position is returned."
 			 (car irt-list)
 			 ""))))
     (when (mew-virtual-p)
-      (setq fld (cdr (assoc (mew-scan-get-folder mew-vec)
-			    (mew-vinfo-get-lra)))))
+      (setq fld (or (cdr (assoc (mew-scan-get-folder mew-vec)
+				(mew-vinfo-get-lra)))
+		    ;; Spotlight
+		    (MEW-FLD)))) ;; xxx
     (setq msg (mew-scan-get-message mew-vec))
     (setq uid (or (mew-scan-uid-uid (MEW-UID)) ""))
     (setq siz (or (mew-scan-uid-size (MEW-UID)) ""))
@@ -355,11 +377,8 @@ of thread indent position is returned."
 	 (if (null lmsg)
 	     (goto-char (point-max))
 	   ;; a message marked with 'T'.
-	   (goto-char (point-min))
-	   (if (not (re-search-forward (mew-regex-sumsyn-msg lmsg) nil t))
-	       (goto-char (point-max)) ;; xxx
+	   (when (mew-summary-search-msg lmsg)
 	     (setq mark (mew-summary-get-mark))
-	     (beginning-of-line)
 	     (setq beg (point))
 	     (forward-line)
 	     ;; To avoid inserting a line AFTER the cursor underline,
@@ -490,7 +509,7 @@ of thread indent position is returned."
     "^[ \t]*[-a-zA-Z0-9]+: "
     "^[ \t]*[>:|#;/_}]"
     "^[ \t]*\\w+\\(['._-]+\\w+\\)*>"
-    "^[ \t]*[[</(.]+ *snip"
+    "^[ \t]*[[</(.-]+ *\\(snip\\|\\.\\.\\)"
     "^   "
     "^--"
     "^- --"
@@ -503,7 +522,7 @@ of thread indent position is returned."
 (defun mew-scan-body (mew-vec &optional draftp)
   (forward-line)
   (let* ((i 0) (I mew-scan-max-body-length)
-	 (j 0) (J 3)
+	 (j 0) (J mew-scan-body-length)
 	 (ctr (MEW-CT))
 	 (cte (MEW-CTE))
 	 (body "")
@@ -592,6 +611,9 @@ of thread indent position is returned."
 (defsubst mew-scan-uid-size (uid)
   (nth 1 (mew-split uid mew-sp)))
 
+(defsubst mew-scan-uid-case (uid)
+  (nth 2 (mew-split uid mew-sp)))
+
 (defun mew-header-insert-xmu (uid siz truncated &optional case)
   (when (and (stringp uid) (stringp siz))
     (setq siz (number-to-string (string-to-number siz))) ;; removing 0
@@ -719,7 +741,7 @@ non-nil, only headers of messages are cached. If executed with
 
 (defsubst mew-summary-folder-cache-newp ()
   (let* ((folder (mew-summary-folder-name 'ext))
-	 (cache (mew-expand-folder folder mew-summary-cache-file))
+	 (cache (mew-expand-file folder mew-summary-cache-file))
 	 (t1 (mew-file-get-time cache))
 	 (t2 (mew-sinfo-get-cache-time)))
     (mew-compare-times t1 t2)))
@@ -746,7 +768,7 @@ non-nil, only headers of messages are cached. If executed with
 
 (defun mew-summary-folder-cache-load ()
   (let* ((folder (mew-summary-folder-name 'ext))
-	 (cache (mew-expand-folder folder mew-summary-cache-file))
+	 (cache (mew-expand-file folder mew-summary-cache-file))
 	 refile refs)
     (when (and (file-readable-p cache)
 	       (mew-summary-folder-cache-newp))
@@ -769,7 +791,7 @@ non-nil, only headers of messages are cached. If executed with
 
 (defun mew-summary-folder-cache-save ()
   (let* ((folder (mew-summary-folder-name 'ext))
-	 (cache (mew-expand-folder folder mew-summary-cache-file)))
+	 (cache (mew-expand-file folder mew-summary-cache-file)))
     (when (file-writable-p cache)
       (mew-touch-folder folder)
       (save-restriction
@@ -807,15 +829,21 @@ non-nil, only headers of messages are cached. If executed with
 	(set-buffer folder)
 	(mew-erase-buffer)
 	(set-buffer-modified-p nil)))
-  (let ((cfile (mew-expand-folder folder mew-summary-cache-file)))
+  (let ((cfile (mew-expand-file folder mew-summary-cache-file)))
     (if (file-exists-p cfile)
 	(write-region "" nil cfile nil 'no-msg))))
 
 (provide 'mew-scan)
 
+(defvar mew-compiling nil)
+(eval-when-compile
+  (when mew-compiling
+    (require 'mew-varsx)
+    (mew-scan-setup)))
+
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
