@@ -41,12 +41,15 @@ A file name of a certificate should be 'cert-hash.0'.
 (defconst mew-tls-nntp "nntp")
 (defconst mew-tls-imap "imap") ;; xxx stunnel does not support this.
 
+;; stunnel does not support IPv6, sigh
+(defconst mew-ssl-localhost "127.0.0.1")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; SSL/TLS info
 ;;;
 
-(defvar mew-ssl-info-list '("status" "try" "file"))
+(defvar mew-ssl-info-list '("status" "try" "file" "string"))
 
 (mew-info-defun "mew-ssl-" mew-ssl-info-list)
 
@@ -77,7 +80,7 @@ A file name of a certificate should be 'cert-hash.0'.
 	(setq args
 	      `("-c" "-f"
 		"-a" ,(expand-file-name (mew-ssl-cert-directory case))
-		"-d" ,(format "localhost:%d" localport)
+		"-d" ,(format "%s:%d" mew-ssl-localhost localport)
 		"-v" ,(number-to-string (mew-ssl-verify-level case))
 		"-D" "debug"
 		"-P" "none"
@@ -94,9 +97,9 @@ A file name of a certificate should be 'cert-hash.0'.
 	(insert "debug=debug\n")
 	(insert "CApath=" (expand-file-name (mew-ssl-cert-directory case)) "\n")
 	(insert (format "[%d]\n" localport))
-	(insert (format "accept=localhost:%d\n" localport))
+	(insert (format "accept=%s:%d\n" mew-ssl-localhost localport))
 	(insert (format "connect=%s:%s\n" server remoteport))
-	(if tls (insert (format "protocol=%s\n" tls)))
+	(if tls (insert (format "protocol=%s\nsslVersion=TLSv1\n" tls)))
 	(mew-frwlet
 	 mew-cs-dummy mew-cs-text-for-write
 	 ;; NEVER use call-process-region for privacy reasons
@@ -123,7 +126,7 @@ A local port number can be obtained the process name after ':'. "
 	   (i 0) (N mew-ssl-process-exec-cnt)
 	   (pros (process-list))
 	   (regex (mew-ssl-info-name-regex server remoteport))
-	   buf name pnm pro dummy bound opts)
+	   name pnm pro dummy bound opts)
       (catch 'find
 	(while pros
 	  (when (string-match regex (process-name (car pros)))
@@ -136,12 +139,11 @@ A local port number can be obtained the process name after ':'. "
 	  pro
 	(message "Creating an SSL/TLS connection...")
 	(setq pro nil)
-	(setq buf (generate-new-buffer (concat mew-buffer-prefix "ssl")))
 	(catch 'loop
 	  (while (< i N)
 	    (setq name (mew-ssl-info-name server remoteport localport))
 	    (setq opts (mew-ssl-options case server remoteport localport tls))
-	    (setq pro (apply 'start-process name buf mew-prog-ssl opts))
+	    (setq pro (apply 'start-process name nil mew-prog-ssl opts))
 	    ;; An error would occur. So, let's exit in the case.
 	    (cond
 	     ((not (processp pro))
@@ -176,7 +178,7 @@ A local port number can be obtained the process name after ':'. "
 	  ;; So, we need to check the SSL/TLS tunnel with a dummy
 	  ;; tunneled connection here.
 	  (set-process-filter pro 'mew-ssl-filter2)
-	  (setq dummy (open-network-stream " *Mew dummy*" nil "localhost" localport))
+	  (setq dummy (open-network-stream " *Mew dummy*" nil mew-ssl-localhost localport))
 	  (mew-rendezvous (null (mew-ssl-get-status pnm)))
 	  (if (processp dummy) (delete-process dummy))
 	  (if (eq (mew-ssl-get-status pnm) t)
@@ -202,37 +204,46 @@ A local port number can be obtained the process name after ':'. "
       (insert (format "\n<%s>\n%s\n" label string)))))
 
 (defun mew-ssl-filter1 (process string)
-  (mew-ssl-debug "SSL/TLS: " string)
-  (let ((pnm (process-name process)))
-    (cond
-     ((string-match "bound to" string)
-      (mew-ssl-set-status pnm t))
-     ((string-match "gethostbyname: Valid name, no data record of requested type" string)
-      (mew-ssl-set-status pnm 'gethostbyname-failure))
-     ((string-match "gethostbyname: Host not found" string)
-      (mew-ssl-set-status pnm 'gethostbyname-failure))
-     ((string-match "Local: bind: Address already in use" string)
-      (mew-ssl-set-status pnm 'bind-failure)))))
+  (let* ((pnm (process-name process))
+	 (prev-str (mew-ssl-get-string pnm)))
+    (save-excursion
+      (mew-ssl-debug "SSL/TLS: " string)
+      (mew-ssl-set-string pnm string)
+      (setq string (concat prev-str string))
+      (cond
+       ((string-match "bound to" string)
+	(mew-ssl-set-status pnm t))
+       ((string-match "gethostbyname: Valid name, no data record of requested type" string)
+	(mew-ssl-set-status pnm 'gethostbyname-failure))
+       ((string-match "gethostbyname: Host not found" string)
+	(mew-ssl-set-status pnm 'gethostbyname-failure))
+       ((string-match "Local: bind: Address already in use" string)
+	(mew-ssl-set-status pnm 'bind-failure))))))
 
 (defun mew-ssl-filter2 (process string)
-  (mew-ssl-debug "SSL/TLS: " string)
-  (let ((pnm (process-name process)))
-    (cond
-     ((string-match "Negotiated ciphers\\|opened with SSL" string)
-      (mew-ssl-set-status pnm t))
-     ((string-match "Failed to initialize" string)
-      (mew-ssl-set-status pnm t)) ;; xxx
-     ((string-match "verify failed" string)
-      (mew-ssl-set-status pnm 'verify-failure)))))
+  (let* ((pnm (process-name process))
+	 (prev-str (mew-ssl-get-string pnm)))
+    (save-excursion
+      (mew-ssl-debug "SSL/TLS: " string)
+      (mew-ssl-set-string pnm string)
+      (setq string (concat prev-str string))
+      (cond
+       ((string-match "Negotiated ciphers\\|opened with SSL" string)
+	(mew-ssl-set-status pnm t))
+       ((string-match "Failed to initialize" string)
+	(mew-ssl-set-status pnm t)) ;; xxx
+       ((string-match "verify failed" string)
+	(mew-ssl-set-status pnm 'verify-failure))))))
 
 (defun mew-ssl-filter3 (process string)
-  (mew-ssl-debug "SSL/TLS: " string))
+  (save-excursion
+    (mew-ssl-debug "SSL/TLS: " string)))
 
 (defun mew-ssl-sentinel (process event)
   (let* ((pnm (process-name process))
 	 (file (mew-ssl-get-file pnm)))
-    (mew-delete-file file)
-    (mew-remove-buffer (process-buffer process))))
+    (save-excursion
+      (mew-delete-file file))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -251,7 +262,7 @@ A local port number can be obtained the process name after ':'. "
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2005 Mew developing team.
+;; Copyright (C) 2002-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
