@@ -49,16 +49,16 @@
       (mew-summary-goto-message)
       (cond
        ((eq (mew-sinfo-get-direction) 'up)
-	(when (re-search-backward mew-regex-sumsyn-short nil t)
-	  (setq fld (mew-sumsyn-folder-name))
-	  (setq next (mew-sumsyn-message-number))))
+	(when (re-search-backward mew-regex-msg-show nil t)
+	  (setq fld (mew-summary-folder-name))
+	  (setq next (mew-summary-message-number))))
        ((eq (mew-sinfo-get-direction) 'down)
 	(if (mew-decode-syntax-end)
 	    (goto-char (mew-decode-syntax-end))
 	  (forward-line))
-	(when (re-search-forward mew-regex-sumsyn-short nil t)
-	  (setq fld (mew-sumsyn-folder-name))
-	  (setq next (mew-sumsyn-message-number))))))
+	(when (re-search-forward mew-regex-msg-show nil t)
+	  (setq fld (mew-summary-folder-name))
+	  (setq next (mew-summary-message-number))))))
     ;; should get the cursor back for display
     (save-excursion
       (if (and fld next
@@ -125,13 +125,15 @@ If nodisplay is non-nil, displays the cached message."
   (mew-message-clear-end-of)
   (mew-overlay-delete-buffer) ;; veil, xxx also delete extents?
   ;; kill mark for cite
-  (and (mark-marker) (set-marker (mark-marker) nil)))
+  (and (mark-marker) (set-marker (mark-marker) nil))
+  (mew-normal-line))
 
 (defvar mew-message-last-buffer nil)
 
 (defsubst mew-summary-display-postscript (&optional no-hook)
   ;; message buffer
   (unless no-hook (run-hooks 'mew-message-hook))
+  (mew-minfo-set-reob nil)
   (setq mew-message-last-buffer (current-buffer))
   (mew-message-set-end-of)
   (set-buffer-modified-p nil))
@@ -141,6 +143,12 @@ If nodisplay is non-nil, displays the cached message."
   (mew-summary-recenter)
   (mew-thread-move-cursor)
   (mew-highlight-cursor-line)
+  (when (and mew-use-unread-mark
+	     (mew-summary-message-number)
+	     (equal (mew-summary-get-mark) mew-mark-unread))
+    (unless (and mew-inherit-after-marking
+		 (not mew-delete-unread-mark-by-mark))
+      (mew-mark-put-mark mew-mark-read 'no-msg)))
   (set-buffer-modified-p nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -149,32 +157,33 @@ If nodisplay is non-nil, displays the cached message."
 ;;;
 
 (defsubst mew-summary-message-toobig (fld msg)
-  (let ((file (mew-expand-folder fld msg)))
+  (let ((file (mew-expand-msg fld msg)))
     (and (file-readable-p file)
 	 (> (mew-file-get-size file) mew-file-max-size))))
 
 (defun mew-summary-display (&optional redisplay)
-  " (1) If called interactively, this command lets you read through
-messages.  That is, display a message, scroll it, and
+  " (1) If called interactively, this command lets you read
+through messages.  That is, display a message, scroll it, and
 move-then-display another message or part.
 
-See 'mew-summary-show-direction' to set the direction that the cursor
-moves. You can select a value out of 'up, 'down, 'next(current
-direction) or 'stop.  Default is 'down. 'mew-summary-show-direction'
-is valid in this case only because the cursor stays in the other
-cases.
+See 'mew-summary-show-direction' to set the direction that the
+cursor moves. You can select a value out of 'up, 'down,
+'next(current direction) or 'stop.  Default is
+'down. 'mew-summary-show-direction' is valid in this case only
+because the cursor stays in the other cases.
 
  (2) If called interactively with '\\[universal-argument]' (i.e.
-REDISPLAY is 't'), this command displays the current message or part
-again. This is a convenient way to get back to the beginning of the
-current message or part.
+REDISPLAY is non-nil), this command displays the current message
+or part again. This is a convenient way to get back to the
+beginning of the current message or part.
 
- (3) If called internally, this function displays the current message
-or part. If it is already displayed, nothing changes.
+ (3) If called internally, this function displays the current
+message or part. If it is already displayed, nothing changes.
 
- (4) If called internally and REDISPLAY is 't', this function displays
-the current message or part. Even if it is already displayed, this
-function displays it again getting gack to the beginning."
+ (4) If called internally and REDISPLAY is 'non-nil', this
+function displays the current message or part. Even if it is
+already displayed, this function displays it again getting back
+to the beginning."
   (interactive "P")
   (when (or redisplay (mew-sinfo-get-disp-msg) (interactive-p))
     (mew-summary-msg-or-part
@@ -182,9 +191,10 @@ function displays it again getting gack to the beginning."
 	    (vfld (mew-summary-folder-name 'ext))
 	    (msg (mew-summary-message-number))
 	    (part (mew-syntax-nums))
-	    (ofld (mew-current-get-fld (mew-frame-id)))
-	    (omsg (mew-current-get-msg (mew-frame-id)))
-	    (opart (mew-current-get-part (mew-frame-id)))
+	    (fid (mew-frame-id))
+	    (ofld (mew-current-get-fld fid))
+	    (omsg (mew-current-get-msg fid))
+	    (opart (mew-current-get-part fid))
 	    (cache (mew-cache-hit fld (or msg omsg)))
 	    (win (selected-window))
 	    (read-through (interactive-p))
@@ -194,7 +204,7 @@ function displays it again getting gack to the beginning."
 	   (progn
 	     (mew-summary-toggle-disp-msg 'on)
 	     (mew-window-configure 'message)
-	     ;; messge buffer
+	     ;; message buffer
 	     (mew-current-set fld (or msg omsg) part)
 	     (mew-minfo-set-summary vfld)
 	     (cond
@@ -342,6 +352,7 @@ This function does not create a cache."
   "This command analyzes the message again with 
 'mew-use-alternative' and 'mew-use-text-body' reversed."
   (interactive "P")
+  (mew-summary-goto-message)
   (let ((mew-use-alternative (not mew-use-alternative))
 	(mew-use-text-body   (not mew-use-text-body)))
     (mew-summary-analyze-again arg)))
@@ -420,7 +431,11 @@ displays it."
 ;;;
 
 (defun mew-summary-display-asis ()
-  "Insert this message or part into Message mode in the raw format."
+  "Insert this message or part into Message mode in the raw format.
+This command treats PGP/MIME transparent. That is an encrypted/signed
+part is displayed as a decrypted/verified part. Also, if this command
+is used for a message whose entire body is encrypted/signed, 
+a decrypted/verified body is displayed."
   (interactive)
   (mew-summary-msg-or-part
    (let* ((fld (mew-summary-folder-name))
@@ -488,14 +503,14 @@ If called with '\\[universal-argument]', it stays writable."
   (mew-summary-msg-or-part
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number2))
-	  (file (mew-expand-folder fld msg)))
+	  (file (mew-expand-msg fld msg)))
      (when (mew-sinfo-get-disp-msg)
        (mew-summary-toggle-disp-msg))
      (mew-frwlet
       (if (mew-folder-draftp fld) mew-cs-m17n mew-cs-autoconv) mew-cs-dummy
       (if arg
 	  (switch-to-buffer (mew-find-file-noselect2 file))
-	(view-buffer (mew-find-file-noselect2 file)))))))
+	(view-buffer (mew-find-file-noselect2 file) 'kill-buffer))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -507,74 +522,65 @@ If called with '\\[universal-argument]', it stays writable."
   (interactive)
   (mew-summary-msg-or-part
    (let* ((win (selected-window))
-	  fld msg cache lst ent svr-date snd rcv svr date tmp)
-     (save-excursion
-       (mew-summary-goto-message)
-       (when (mew-sumsyn-match mew-regex-sumsyn-short)
-	 (setq fld (mew-sumsyn-folder-name))
-	 (setq msg (mew-sumsyn-message-number))) ;; msg is never nil
-       (setq cache (mew-cache-hit fld msg))
-       (cond
-	(cache
-	 (set-buffer cache)
-	 (setq lst (mew-header-get-value mew-received: 'as-list)))
-	(t
-	 (with-temp-buffer
-	   (mew-insert-message fld msg mew-cs-autoconv
-			       mew-header-reasonable-size)
-	   (setq lst (mew-header-get-value mew-received: 'as-list))))))
-     (mew-summary-toggle-disp-msg 'on)
-     (mew-window-configure 'message)
-     ;; messge buffer
+	  (fld (mew-summary-folder-name))
+	  (msg (mew-summary-message-number2))
+	  lst ent svr-date (snd "?") (rcv "?") last svr date tmp)
+     (mew-summary-set-message-buffer fld msg)
+     ;; message buffer or cache buffer
+     (setq lst (nreverse (mew-header-get-value mew-received: 'as-list)))
+     (set-buffer (mew-buffer-message))
+     ;; message buffer
      (mew-summary-display-preamble)
-     (setq lst (nreverse lst))
-     (while lst
-       (setq ent (car lst))
-       (setq lst (cdr lst))
-       (setq svr-date (mew-split ent ?\;))
-       (setq svr (nth 0 svr-date))
-       (setq date (nth 1 svr-date))
-       (unless date
-	 ;; for broken MTAs
-	 (if (string-match "Mon\\|Tue\\|Wed\\|Thu\\|Fri\\|Sat\\|Sun" ent)
-	     (setq date (substring ent (match-beginning 0)))))
-       (if date (setq date (mew-time-rfc-to-sortkey date 'tzadj)))
-       (unless date (setq date "19700101000000"))
-       (setq tmp (cons (list date svr) tmp))
-       (cond
-	((string-match "(\\(qmail [0-9]+\\)" svr)
-	 (setq rcv (match-string 1 svr)))
-	((string-match "by[ \t]*\\([^() \t\n]+\\)" svr)
-	 (setq rcv (match-string 1 svr))
-	 (if (string-match "from[ \t]*\\([^() \t\n]+\\)" svr)
-	     (setq snd (match-string 1 svr)))
-	 (when (and (string= snd "unknown")
-		    (string-match "from unknown (HELO \\([^() \t\n]+\\))" svr))
-	   (setq snd (concat (match-string 1 svr) "(unknown)"))))
-	((string-match "[ \t]*\\([^() \t\n]+\\)" svr)
-	 (setq rcv (match-string 1 svr))
-	 (if (string-match "from[ \t]*\\([^() \t\n]+\\)" svr)
-	     (setq snd (match-string 1 svr)))))
-       (mew-elet
-	(insert date)
-	(beginning-of-line)
-	(forward-char 4)
-	(insert "/")
-	(forward-char 2)
-	(insert "/")
-	(forward-char 2)
-	(insert " ")
-	(forward-char 2)
-	(insert ":")
-	(forward-char 2)
-	(insert ":")
-	(forward-char 2)
-	(insert " (" (mew-time-tmzn-int) ") ")
-	(if snd (insert snd))
-	(insert " => ")
-	(if rcv (insert rcv))
-	(insert "\n")
-	(setq snd nil rcv nil)))
+     (save-excursion
+       (while lst
+	 (setq ent (car lst))
+	 (setq lst (cdr lst))
+	 (setq svr-date (mew-split ent ?\;))
+	 (mew-set '(svr date) svr-date)
+	 (unless date
+	   ;; for broken MTAs
+	   (if (string-match "Mon\\|Tue\\|Wed\\|Thu\\|Fri\\|Sat\\|Sun" ent)
+	       (setq date (substring ent (match-beginning 0)))))
+	 (if date (setq date (mew-time-rfc-to-sortkey date 'tzadj)))
+	 (unless date (setq date "19700101000000"))
+	 (setq tmp (cons (list date svr) tmp))
+	 (cond
+	  ((string-match "(\\(qmail [0-9]+\\)" svr)
+	   (setq rcv (match-string 1 svr)))
+	  ((string-match "by[ \t]+\\([^() \t\n]+\\)" svr)
+	   (setq rcv (match-string 1 svr))
+	   (if (string-match "from[ \t]+\\([^() \t\n]+\\).*\\([[][.0-9]+[]]\\)" svr)
+	       (setq snd (format "%s %s" (match-string 1 svr) (match-string 2 svr)))
+	     (if (string-match "from[ \t]+\\([^() \t\n]+\\)" svr)
+		 (setq snd (match-string 1 svr))))
+	   (when (and (string= snd "unknown")
+		      (string-match "from unknown (HELO \\([^() \t\n]+\\))" svr))
+	     (setq snd (concat (match-string 1 svr) "(unknown)"))))
+	  ((string-match "[ \t]*\\([^() \t\n]+\\)" svr)
+	   (setq rcv (match-string 1 svr))
+	   (if (string-match "from[ \t]*\\([^() \t\n]+\\)" svr)
+	       (setq snd (match-string 1 svr)))))
+	 (mew-elet
+	  (insert date)
+	  (beginning-of-line)
+	  (forward-char 4)
+	  (insert "/")
+	  (forward-char 2)
+	  (insert "/")
+	  (forward-char 2)
+	  (insert " ")
+	  (forward-char 2)
+	  (insert ":")
+	  (forward-char 2)
+	  (insert ":")
+	  (forward-char 2)
+	  (insert " (" (mew-time-tmzn-int) ") ")
+	  (if (and snd (not (string= snd last))) (insert snd))
+	  (insert " => ")
+	  (if rcv (insert rcv))
+	  (insert "\n")
+	  (setq last rcv)
+	  (setq snd "?" rcv "?"))))
      (mew-summary-display-postscript 'no-hook)
      (select-window win))))
 
@@ -613,9 +619,7 @@ ad-hoc solution for putting the review mark on a thread.")
     t)
    (t
     (mew-decode-syntax-delete)
-    (forward-line -1)
-    (while (and (eq (get-text-property (point) 'invisible) t)
-		(= (forward-line -1) 0)))
+    (previous-line 1) ;; for invisible
     (unless (pos-visible-in-window-p)
       ;; This ensures that some messages are displayed in Summary.
       (recenter))
@@ -644,7 +648,7 @@ ad-hoc solution for putting the review mark on a thread.")
 ;;;
 
 (defun mew-summary-display-next ()
-  (if (mew-summary-next) (mew-summary-display nil)))
+  (if (mew-summary-next) (mew-summary-display)))
 
 (defun mew-summary-display-up (&optional arg)
   "Move to above then display. Targets includes parts, messages 
@@ -655,7 +659,7 @@ parts are skipped."
   (when arg
     (mew-summary-goto-message)
     (mew-decode-syntax-delete))
-  (if (mew-summary-up) (mew-summary-display nil)))
+  (if (mew-summary-up) (mew-summary-display)))
 
 (defun mew-summary-display-down (&optional arg)
   "Move to below then display. Targets includes parts, messages 
@@ -665,7 +669,7 @@ parts are skipped."
   (when arg
     (mew-summary-goto-message)
     (mew-decode-syntax-delete))
-  (if (mew-summary-down) (mew-summary-display nil)))
+  (if (mew-summary-down) (mew-summary-display)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -687,11 +691,12 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
   "Make this message scroll up with one line."
   (interactive)
   (mew-summary-msg-or-part
-   (let ((win (selected-window))
-	 (msg (mew-summary-message-number))
-	 (part (mew-syntax-nums))
-	 (omsg (mew-current-get-msg (mew-frame-id)))
-	 (opart (mew-current-get-part (mew-frame-id))))
+   (let* ((win (selected-window))
+	  (msg (mew-summary-message-number))
+	  (part (mew-syntax-nums))
+	  (fid (mew-frame-id))
+	  (omsg (mew-current-get-msg fid))
+	  (opart (mew-current-get-part fid)))
      ;; xxx how about folder check?
      (if (or (and msg (string= msg omsg) (null part) (null opart))
 	     (and part (equal part opart))) ;; MUST be equal
@@ -708,11 +713,12 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
   "Make this message scroll down with one line."
   (interactive)
   (mew-summary-msg-or-part
-   (let ((win (selected-window))
-	 (msg (mew-summary-message-number))
-	 (part (mew-syntax-nums))
-	 (omsg (mew-current-get-msg (mew-frame-id)))
-	 (opart (mew-current-get-part (mew-frame-id))))
+   (let* ((win (selected-window))
+	  (msg (mew-summary-message-number))
+	  (part (mew-syntax-nums))
+	  (fid (mew-frame-id))
+	  (omsg (mew-current-get-msg fid))
+	  (opart (mew-current-get-part fid)))
      ;; xxx how about folder check?
      (if (or (and msg (string= msg omsg) (null part) (null opart))
 	     (and part (equal part opart))) ;; MUST be equal
@@ -736,7 +742,12 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
   (or arg (setq arg 1))
   (if (< arg 0)
       (mew-summary-previous-line (- arg))
-    (let ((col (current-column)))
+    (let (col) 
+      (if (eq last-command this-command) 
+	  (setq col (mew-sinfo-get-column)))
+      (unless col
+	(setq col (current-column))
+	(mew-sinfo-set-column col))
       (while (> arg 0)
 	(forward-line)
 	(while (or (looking-at mew-regex-thread-separator)
@@ -751,11 +762,15 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
   (or arg (setq arg 1))
   (if (< arg 0)
       (mew-summary-next-line (- arg))
-    (let ((col (current-column)))
+    (let (col) 
+      (if (eq last-command this-command) 
+	  (setq col (mew-sinfo-get-column)))
+      (unless col
+	(setq col (current-column))
+	(mew-sinfo-set-column col))
       (while (> arg 0)
-	(forward-line -1)
-	(while (or (looking-at mew-regex-thread-separator)
-		   (eq (get-text-property (point) 'invisible) t))
+	(previous-line 1) ;; for invisible
+	(while (looking-at mew-regex-thread-separator)
 	  (forward-line -1))
 	(setq arg (1- arg)))
       (move-to-column col))))
@@ -776,7 +791,7 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2003 Mew developing team.
+;; Copyright (C) 1996-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
