@@ -25,6 +25,7 @@
 	   ,@body
 	   ;; XEmacs does not draw attachments unless sit for 0...
 	   (mew-redraw)
+	   (if quit-flag (setq quit-flag nil))
 	   ;; XEmacs does not draw toolbar, so...
 	   (if mew-xemacs-p (mew-xemacs-redraw-toolbar)))
        (mew-draft-save-buffer)) ;; to make sure not to use this draft again
@@ -96,19 +97,22 @@ the current message is copied to To: in a draft."
 (defun mew-summary-send (&optional to cc subject newsgroups)
   "Write a message. A new draft is prepared in Draft mode."
   (interactive)
-  (let ((draft (mew-folder-new-message mew-draft-folder)))
+  (let ((draft (mew-folder-new-message mew-draft-folder))
+	asked)
     (if (mew-folder-nntpp (mew-summary-physical-folder))
 	(if (null newsgroups) (setq newsgroups ""))
       (when (and mew-ask-to (null to))
-        (setq to (mew-input-address (concat mew-to: " "))))
+        (setq to (mew-input-address (concat mew-to: " ")))
+	(setq asked t))
       (when (and mew-ask-cc (null cc))
-        (setq cc (mew-input-address (concat mew-cc: " ")))))
+        (setq cc (mew-input-address (concat mew-cc: " ")))
+	(setq asked t)))
     (mew-current-set-window-config)
     (mew-window-configure 'draft)
     (mew-summary-prepare-draft
      (mew-draft-find-and-switch draft)
      (mew-delete-directory-recursively (mew-attachdir draft))
-     (mew-draft-header subject nil to cc newsgroups)
+     (mew-draft-header subject nil to cc newsgroups nil nil nil asked)
      (mew-draft-mode)
      (run-hooks 'mew-draft-mode-newdraft-hook))))
 
@@ -392,11 +396,12 @@ Draft mode and this message is automatically attached."
     (let* ((owin (selected-window))
 	   (fld (mew-summary-folder-name))
 	   (msg (mew-summary-message-number2))
-	   (file (mew-expand-folder fld msg))
+	   (file (mew-expand-msg fld msg))
 	   (draft (mew-folder-new-message mew-draft-folder))
 	   (draftdir (file-name-nondirectory draft))
 	   (to (and mew-ask-to (mew-input-address (concat mew-to: " "))))
 	   (cc (and mew-ask-cc (mew-input-address (concat mew-cc: " "))))
+	   (asked (or mew-ask-to mew-ask-cc))
 	   subject fwsubject cwin)
       (mew-summary-prepare-draft
        (mew-summary-prepare-three-windows)
@@ -410,7 +415,7 @@ Draft mode and this message is automatically attached."
 	   (setq fwsubject (mew-subject-simplify (concat mew-forward-string subject))))
        (select-window cwin) ;; draft
        ;;
-       (mew-draft-header fwsubject 'nl to cc)
+       (mew-draft-header fwsubject 'nl to cc nil nil nil nil asked)
        (mew-draft-mode)
        (run-hooks 'mew-draft-mode-newdraft-hook)
        (mew-draft-multi-copy draft (list file))
@@ -434,12 +439,13 @@ attached."
     (let* ((draft (mew-folder-new-message mew-draft-folder))
 	   (draftdir (file-name-nondirectory draft))
 	   (to (and mew-ask-to (mew-input-address (concat mew-to: " "))))
-	   (cc (and mew-ask-cc (mew-input-address (concat mew-cc: " ")))))
+	   (cc (and mew-ask-cc (mew-input-address (concat mew-cc: " "))))
+	   (asked (or mew-ask-to mew-ask-cc)))
       (mew-summary-prepare-draft
        (mew-summary-prepare-three-windows)
        (mew-draft-find-and-switch draft t)
        (mew-delete-directory-recursively (mew-attachdir draft))
-       (mew-draft-header nil 'nl to cc)
+       (mew-draft-header nil 'nl to cc nil nil nil nil asked)
        (mew-draft-mode)
        (run-hooks 'mew-draft-mode-newdraft-hook)
        (mew-draft-multi-copy draft FILES)
@@ -452,7 +458,7 @@ attached."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
