@@ -36,8 +36,12 @@
       (funcall func (get-buffer-create case:folder))
       (mew-set-buffer-cs mew-cs-m17n)
       (mew-buffers-setup case:folder)
-      (mew-sinfo-set-case (mew-case:folder-case case:folder))
-      (mew-sinfo-set-folder (mew-case:folder-folder case:folder))
+      (let* ((case (mew-case:folder-case case:folder))
+	     (folder (mew-case:folder-folder case:folder))
+	     (inboxp (string= folder (mew-proto-inbox-folder folder (or case mew-case)))))
+	(mew-sinfo-set-case case)
+	(mew-sinfo-set-folder folder)
+	(mew-sinfo-set-inboxp inboxp))
       (if (mew-folder-virtualp case:folder)
 	  (progn
 	    (if p-folder (mew-vinfo-set-parent-folder p-folder))
@@ -75,11 +79,13 @@ Summary mode."
 	(setq sep (mew-imap-separator case))
 	(unless (or (assoc folder folder-alist)
 		    (assoc (concat folder sep) folder-alist))
-	  (message "%s does not exist" case:folder)
+	  (message "\"%s\" does not exist. %s" case:folder
+		   (mew-substitute-for-summary "Type '3\\[mew-status-update]' to collect IMAP folders"))
 	  (setq case:folder nil)))
        ((mew-folder-nntpp folder)
 	(unless (assoc folder (mew-nntp-folder-alist case))
-	  (message "%s does not exist" case:folder)
+	  (message "\"%s\" does not exist. %s" case:folder
+		   (mew-substitute-for-summary "Type '2\\[mew-status-update]' to collect newsgroups"))
 	  (setq case:folder nil))))
       (when case:folder
 	(when mew-use-other-frame-for-summary
@@ -146,13 +152,10 @@ If 'mew-summary-goto-line-then-display' is non-nil,
 the message is then displayed."
   (interactive "P")
   (if arg
-      (let ((msg (read-string "Message No.: " "")) regex)
+      (let ((msg (read-string "Message No.: " "")))
 	(while (not (string-match mew-regex-message-files3 msg))
 	  (setq msg (read-string "Message No.: " "")))
-	(setq regex (mew-regex-sumsyn-msg msg))
-	(when (or (re-search-forward regex nil t)
-		  (re-search-backward regex nil t))
-	  (beginning-of-line)
+	(when (mew-summary-search-msg msg)
 	  (if mew-summary-goto-line-then-display
 	      (mew-summary-display))))
     (let ((line (string-to-number (read-string "Line No.: " ""))))
@@ -183,15 +186,18 @@ the message is then displayed."
 ;;;
 
 (defun mew-max-draft-buffer ()
-  (let* ((regex (mew-folder-regex (file-name-as-directory mew-draft-folder)))
-	 (bufs (mew-buffer-list regex))
-	 buf draft)
-    (while bufs
-      (setq buf (car bufs))
-      (setq bufs (cdr bufs))
-      (if (or (null draft) (string< draft buf))
-	  (setq draft buf)))
-    draft))
+  (let* ((draft-dir (file-name-as-directory mew-draft-folder))
+	 (regex (mew-folder-regex draft-dir))
+         (bufs (mew-buffer-list regex))
+	 (msgregex (format "\\([0-9]+\\)\\(%s\\)?$" mew-suffix))
+         nums num n)
+    (setq nums (mapcar (lambda (x) (string-match msgregex x) (string-to-number (mew-match-string 1 x))) bufs))
+    (while nums
+      (setq num (car nums))
+      (setq nums (cdr nums))
+      (if (or (null n) (< n num))
+	  (setq n num)))
+    (and n (format "%s%d%s" draft-dir n (if mew-use-suffix mew-suffix "")))))
 
 (defun mew-summary-jump-to-draft-buffer ()
   "Jump to one of drafts if exists."
@@ -226,7 +232,10 @@ the top message is then displayed."
   (interactive)
   (if mew-use-push-mark (mew-push-mark))
   (goto-char (point-max))
-  (unless (bobp) (forward-line -1))
+  (if (and (mew-summary-exclusive-p 'no-msg)
+	   (not (bobp))
+	   (mew-sinfo-get-disp-msg))
+      (forward-line -1))
   (if mew-summary-jump-bottom-then-display
       (mew-summary-display)))
 
@@ -322,7 +331,7 @@ If called with '\\[universal-argument]', you can specify a printer name."
       (save-restriction
 	(widen)
 	(setq have-header (mew-msghdr-p))
-	(setq str (mew-buffer-substring (point-min) (point-max)))))
+	(setq str (buffer-substring (point-min) (point-max)))))
     (when (y-or-n-p "Print this message? ")
       (with-temp-buffer
 	(insert str)
@@ -364,7 +373,7 @@ argument."
   (mew-summary-msg-or-part
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number))
-	  (file (mew-expand-folder fld msg))
+	  (file (mew-expand-msg fld msg))
 	  (cmd-args (mew-input-command))
 	  (cmd (car cmd-args)) 
 	  (args (nconc (cdr cmd-args) (list file))))
@@ -430,8 +439,8 @@ Content-Type is automatically asked. And if it has both
 internal and external visualization mechanisms, you are asked
 which you want to use.
 
-If executed with'\\[universal-argument]', you can specify a command
-to be executed.
+If executed with '\\[universal-argument]', you can specify a
+command to be executed.
 
 See 'mew-mime-content-type' to know how actions can be defined."
   (interactive "P")
@@ -607,7 +616,7 @@ If executed with '\\[universal-argument]', you can set the sending case."
 	   (proto (mew-proto-to-flush fld))
 	   qfld case)
       (if (or arg mew-ask-flush-case)
-	  (setq case (mew-input-case mew-case"Queue"))
+	  (setq case (mew-input-case mew-case "Queue"))
 	(setq case mew-case))
       (if (or (mew-folder-queuep fld) (mew-folder-postqp fld))
 	  (progn
@@ -767,7 +776,7 @@ message."
        (message "Joining...")
        (with-temp-buffer
 	 (mew-set-buffer-multibyte t)
-	 (insert "CD: " cfld "\n")
+	 (insert "CD: " (mew-expand-folder2 cfld) "\n")
 	 (while targets
 	   (setq fld-msg (car targets))
 	   (setq targets (cdr targets))
@@ -778,6 +787,7 @@ message."
 	 (mew-erase-buffer)
 	 (call-process mew-prog-mewl nil t nil
 		       "-b" mew-mail-path "-l" "0"
+		       "-x" mew-suffix
 		       "-i" tfile "-d" "content-type")
 	 (mew-delete-file tfile)
 	 (goto-char (point-min))
@@ -826,7 +836,7 @@ message."
 	  (goto-char (point-max))
 	  (save-restriction
 	    (narrow-to-region (point) (point))
-	    (mew-insert-file-contents (mew-expand-folder cfld (aref vec 0)))
+	    (mew-insert-file-contents (mew-expand-msg cfld (aref vec 0)))
 	    ;; Removing unnecessary fields from the encapsulating
 	    ;; (outer) header.
 	    (goto-char (point-min))
@@ -835,7 +845,9 @@ message."
 	    (goto-char (point-min))
 	    (re-search-forward mew-eoh nil t)
 	    (setq beg (point))
-	    (while (looking-at mew-eoh) (forward-line))
+	    (while (looking-at mew-eoh)
+	      (forward-line)
+	      (if (eobp) (error "invalid message (number 1)")))
 	    (delete-region beg (point))
 	    ;; Removing unnecessary fields from the encapsulated
 	    ;; (inner) header.
@@ -859,12 +871,14 @@ message."
 	    (goto-char (point-max))
 	    (save-restriction
 	      (narrow-to-region (point) (point))
-	      (mew-insert-file-contents (mew-expand-folder cfld (aref vec i)))
+	      (mew-insert-file-contents (mew-expand-msg cfld (aref vec i)))
 	      (goto-char (point-min))
 	      (re-search-forward mew-eoh nil t)
 	      (forward-line)
+	      (if (eobp) (error "invalid message (number %d)" (1+ i)))
 	      (delete-region (point-min) (point)))
 	    (setq i (1+ i)))
+	  (mew-header-delete-lines `(,mew-x-mew-uidl:))
 	  (write-region (point-min) (point-max)
 			(mew-folder-new-message folder)
 			nil 'no-msg)))
@@ -897,7 +911,7 @@ message."
 	(message "Nothing was processed")
       (setq case (mew-case:folder-case case:trash))
       (setq trash (mew-case:folder-folder case:trash))
-      (setq trashdir (mew-expand-folder trash))
+      (setq trashdir (mew-expand-folder case:trash))
       (setq msgs (mew-dir-messages trashdir))
       (if (and (mew-folder-localp trash) (null msgs))
 	  (message "No messages to be removed in %s" trash)
@@ -905,6 +919,10 @@ message."
 	    (message "No messages removed in %s" trash)
 	  (if (string= folder trash) (mew-summary-reset))
 	  (message "Removing all messages in %s..." case:trash)
+	  (when (get-buffer case:trash)
+	    (save-excursion
+	      (set-buffer case:trash)
+	      (mew-summary-kill-subprocess)))
 	  (mew-summary-unlink-msgs case:trash msgs)
 	  (mew-summary-folder-cache-clean case:trash)
 	  (message "Removing all messages in %s...done" case:trash)
@@ -929,11 +947,11 @@ message."
      (save-excursion
        (mew-summary-goto-message)
        (setq msg (mew-summary-message-number)))
-     (setq file (mew-expand-folder fld msg))
+     (setq file (mew-expand-msg fld msg))
      (setq dstfld (mew-input-local-folder mew-inbox-folder))
      (when (and dstfld (mew-local-folder-check dstfld 'ask))
        (setq dstmsg (mew-folder-new-message dstfld 'numonly))
-       (setq dstfile (mew-expand-folder dstfld dstmsg))
+       (setq dstfile (mew-expand-msg dstfld dstmsg))
        (when (get-buffer dstfld)
 	 (set-buffer dstfld)
 	 (setq exclusivep (mew-summary-exclusive-p)))
@@ -951,7 +969,7 @@ message."
   "Copy messages marked with '*' to a local folder.  If called
 with '\\[universal-argument]', only messages marked with '*' in
 the region are handled."
-  (interactive "P")
+ (interactive "P")
   (mew-summary-not-in-draft
    (let ((mew-use-highlight-x-face nil)
 	 (fld (mew-summary-folder-name))
@@ -969,7 +987,8 @@ the region are handled."
        (setq end (point-max))))
      (setq dstfld (mew-input-local-folder mew-inbox-folder))
      (when (and dstfld (mew-local-folder-check dstfld 'ask))
-       (setq dstmsg (mew-folder-new-message dstfld 'numonly))
+       (let ((mew-use-suffix nil))
+	 (setq dstmsg (mew-folder-new-message dstfld 'numonly)))
        (setq msgs (mew-summary-mark-collect mew-mark-review beg end))
        (when (get-buffer dstfld)
 	 (save-excursion
@@ -985,8 +1004,8 @@ the region are handled."
 	 (setq total (length msgs))
 	 (setq i 1)
 	 (while msgs
-	   (setq file (mew-expand-folder fld (car msgs)))
-	   (setq dstfile (mew-expand-folder dstfld dstmsg))
+	   (setq file (mew-expand-msg fld (car msgs)))
+	   (setq dstfile (mew-expand-new-msg dstfld dstmsg))
 	   (message "Copying (%d/%d)..." i total)
 	   (mew-summary-local-copy-one file dstfile)
 	   (setq dstmsg (number-to-string (1+ (string-to-number dstmsg))))
@@ -1232,7 +1251,7 @@ See also 'mew-spam-prog' and 'mew-spam-prog-args."
   (mew-summary-msg
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number))
-	  (file (mew-expand-folder fld msg))
+	  (file (mew-expand-msg fld msg))
 	  (case (mew-sinfo-get-case))
 	  (prog (mew-spam-prog case))
 	  (args (mew-spam-prog-args case))
@@ -1256,7 +1275,7 @@ See also 'mew-ham-prog' and 'mew-ham-prog-args."
   (mew-summary-msg
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number))
-	  (file (mew-expand-folder fld msg))
+	  (file (mew-expand-msg fld msg))
 	  (case (mew-sinfo-get-case))
 	  (prog (mew-ham-prog case))
 	  (args (mew-ham-prog-args case))
@@ -1279,17 +1298,66 @@ See also 'mew-ham-prog' and 'mew-ham-prog-args."
 ;;;
 
 (defun mew-summary-toggle-invisible ()
+  "Toggle the value of search-invisible.
+Invisible information in Summary mode cannot be searched by default.
+Executing this command enables searching such information."
   (interactive)
   (if (eq search-invisible t)
       (setq search-invisible 'open)
     (setq search-invisible t))
   (message "'search-invisible' is set to '%s'" search-invisible))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Lines
+;;;
+
+(defun mew-summary-line ()
+  "Toggle normal lines, wrapped lines and long lines."
+  (interactive)
+  (let ((cwin (get-buffer-window (current-buffer)))
+	(mwin (get-buffer-window (mew-buffer-message))))
+    (if (not mwin)
+	(message "No message is displayed")
+      (select-window mwin)
+      (unwind-protect
+	  (mew-message-line)
+	(select-window cwin)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Horizontal Scrolling
+;;;
+
+(defun mew-summary-scroll-left ()
+  "Scroll the Message window to the right."
+  (interactive)
+  (let ((cwin (get-buffer-window (current-buffer)))
+	(mwin (get-buffer-window (mew-buffer-message))))
+    (if (not mwin)
+	(message "No message is displayed")
+      (select-window mwin)
+      (unwind-protect
+	  (mew-message-scroll-left)
+	(select-window cwin)))))
+
+(defun mew-summary-scroll-right ()
+  "Scroll the Message window to the left."
+  (interactive)
+  (let ((cwin (get-buffer-window (current-buffer)))
+	(mwin (get-buffer-window (mew-buffer-message))))
+    (if (not mwin)
+	(message "No message is displayed")
+      (select-window mwin)
+      (unwind-protect
+	  (mew-message-scroll-right)
+	(select-window cwin)))))
+
 (provide 'mew-summary4)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
