@@ -189,7 +189,7 @@
     (let ((body (mew-syntax-get-part syntax)))
       (if (mew-syntax-multipart-p body)
 	  (mew-syntax-get-entry (mew-syntax-get-part syntax) nums)
-	;; nums sould be "1"
+	;; nums should be "1"
 	body)))
    ((mew-syntax-multipart-p syntax)
     (if (null nums)
@@ -330,7 +330,7 @@
 
 (defsubst mew-syntax-number-to-nums (strnum)
   (if strnum
-      (mapcar 'string-to-int (mew-split strnum ?.))
+      (mapcar 'string-to-number (mew-split strnum ?.))
     nil))
 
 (defsubst mew-syntax-nums ()
@@ -358,9 +358,15 @@
 ;;; Summary mode syntax
 ;;;
 
+(defvar mew-use-cut-file-name t)
+
 (defun mew-summary-info (arg)
   "Display the folder name of this message and the message number.
-If this is a cache message, both the unique id and the size are also shown."
+If this is a cache message, both the unique id and the size are also shown.
+If called with '\\[universal-argument]', its Message-ID: and its parent's
+Message-ID: are displayed instead.
+If 'mew-use-cut-file-name' is non-nil, the file path is
+pushed to the cut buffer of your window system."
   (interactive "P")
   (mew-summary-msg
    (when (mew-sumsyn-match mew-regex-sumsyn-long)
@@ -368,11 +374,17 @@ If this is a cache message, both the unique id and the size are also shown."
 	 (message "my-id=%s par-id=%s"
 		  (mew-sumsyn-my-id)
 		  (mew-sumsyn-parent-id))
-       (message "%s/%s size=%s uid=%s"
-		(mew-sumsyn-folder-name)
-		(mew-sumsyn-message-number)
-		(mew-sumsyn-message-size)
-		(mew-sumsyn-message-uid))))))
+       (let* ((fld (mew-sumsyn-folder-name))
+	      (msg (mew-sumsyn-message-number))
+	      (size (mew-sumsyn-message-size))
+	      (uid (mew-sumsyn-message-uid))
+	      (file (concat (file-name-as-directory fld) msg))
+	      (path (mew-expand-folder file)))
+	 (setq path (mew-msg-get-filename path))
+	 (setq file (file-name-nondirectory path))
+	 (message "%s size=%s uid=%s" file size uid)
+	 (when mew-use-cut-file-name
+	   (kill-new path)))))))
 
 (defsubst mew-sumsyn-match (regex)
   (save-excursion
@@ -387,8 +399,8 @@ If this is a cache message, both the unique id and the size are also shown."
 	(bn (buffer-name)))
     (cond
      ((not (string= fn "")) fn)
-     ((mew-thread-p)
-      (mew-thread-to-folder bn))
+     ((mew-thread-p) ;; xxx
+      (mew-physical-folder bn))
      (t bn))))
 
 (defsubst mew-sumsyn-message-number () (mew-match-string 2))
@@ -403,15 +415,13 @@ If this is a cache message, both the unique id and the size are also shown."
   (cond
    ((or ext (mew-summary-p))
     (buffer-name))
-   ((mew-thread-p)
-    (mew-thread-to-folder (buffer-name)))
    (t ;; Virtual mode
     (save-excursion
       (mew-summary-goto-message)
       (when (mew-sumsyn-match mew-regex-sumsyn-short)
 	(mew-sumsyn-folder-name))))))
 
-(defsubst mew-summary-message-number ()
+(defun mew-summary-message-number ()
   (let ((event last-command-event)
 	ret str)
     (if (and mew-xemacs-p
@@ -516,10 +526,11 @@ If this is a cache message, both the unique id and the size are also shown."
    (list (mew-encode-syntax-text))))
 
 (defun mew-encode-syntax-initial-multi (dir n)
-  (let ((i 1) (ret))
+  (let ((i 1) file ret)
     (while (<= i n)
+      (setq file (mew-msg-new-filename (number-to-string i)))
       (setq ret (vconcat ret (list (mew-encode-syntax-single
-				    (int-to-string i) nil nil nil nil 'cdp))))
+				    file nil nil nil nil 'cdp))))
       (setq i (1+ i)))
     (vconcat (mew-encode-syntax-multi dir mew-type-mlm)
 	     (list (mew-encode-syntax-single mew-draft-coverpage 
@@ -536,10 +547,16 @@ If this is a cache message, both the unique id and the size are also shown."
 
 ;; MUST generate a new vector
 
-(defun mew-decode-syntax-rfc822 (&optional msg-head)
+(defun mew-decode-syntax-rfc822 (&optional msg-head multi)
   ;; msg-head may include CD:
   (unless msg-head (setq msg-head (mew-decode-syntax-rfc822-head t)))
-  (vconcat msg-head (vector (mew-decode-syntax-text))))
+  (if (not multi)
+      (vconcat msg-head (vector (mew-decode-syntax-text)))
+    (let ((body (mew-decode-syntax-text))
+	  (head (mew-decode-syntax-multi-head)))
+      (mew-syntax-set-begin head (mew-syntax-get-begin body))
+      (mew-syntax-set-end head (mew-syntax-get-end body))
+      (vconcat msg-head (vector (mew-syntax-cat head body))))))
 
 (defun mew-decode-syntax-rfc822-head (&optional reg-hend)
   (vector 'message (point-min)
@@ -676,8 +693,8 @@ If this is a cache message, both the unique id and the size are also shown."
 				(mew-ctdb-icon (mew-ctdb-by-ct ct)) func))
     (while (< cnt len)
       (if part
-	  (setq strpart (concat part "." (int-to-string num)))
-	(setq strpart (int-to-string num)))
+	  (setq strpart (concat part "." (number-to-string num)))
+	(setq strpart (number-to-string num)))
       (setq subsyntax (aref syntax cnt))
       (cond
        ((mew-syntax-multipart-p subsyntax)
@@ -691,8 +708,8 @@ If this is a cache message, both the unique id and the size are also shown."
       (setq num (1+ num)))
     (unless dec 
       (if part
-	  (setq part (concat part "." (int-to-string num)))
-	(setq part (int-to-string num)))
+	  (setq part (concat part "." (number-to-string num)))
+	(setq part (number-to-string num)))
       (mew-syntax-format mew-encode-syntax-dot part dec)
       (mew-syntax-set-icon-spec part "Attach Here" cd mew-icon-blank func))))
 
@@ -756,12 +773,12 @@ If this is a cache message, both the unique id and the size are also shown."
 			  (mew-join "," decrypters-list)))
 	 (cd-or-dec cd)
 	 (privacy (mew-syntax-get-privacy syntax))
-	 (space " ") (LT (- (min mew-window-magic (window-width)) 2))
+	 (space " ") (LT (- (max mew-window-magic (window-width)) 2))
 	 (ln (length number))
 	 (lm 4) (lt 27)
 	 (ld (* (/ (- LT lm lt) 5) 3))
 	 (lf (- LT lm ln lt ld 8))
-	 (marks (make-string lm 32))
+	 (marks (make-string lm mew-sp))
 	 (i 0) (N (length privacy))
 	 ctm ctp)
 
@@ -790,19 +807,21 @@ If this is a cache message, both the unique id and the size are also shown."
 	   ((string= cte mew-b64) (aset marks 0 ?B))
 	   ((string= cte mew-qp)  (aset marks 0 ?Q))
 	   ((string= cte mew-xg)  (aset marks 0 ?G))
-	   (t                     (aset marks 0 ?X))))
+	   (t                     (aset marks 0 ??))))
       
       (if dec (setq privacy (reverse privacy)))
       (while (< i N)
-	(setq ctm (nth 0 (nth i privacy)))
-	(setq ctp (nth 1 (nth i privacy)))
+	(mew-set '(ctm ctp) (nth i privacy))
 	(let ((case-fold-search t))
 	  (cond
-	   ((string-match "pgp"  ctp) (aset marks (* i 2) ?P))
-	   ((string-match "moss" ctp) (aset marks (* i 2) ?M)))
+	   ((string-match "pgp"   ctp) (aset marks (* i 2) ?P))
+	   ((string-match "pkcs"  ctm) (aset marks (* i 2) ?S))
+	   ((string-match "moss"  ctp) (aset marks (* i 2) ?M)))
 	  (cond
 	   ((mew-case-equal mew-ct-mle ctm) (aset marks (1+ (* i 2)) ?E))
-	   ((mew-case-equal mew-ct-mls ctm) (aset marks (1+ (* i 2)) ?S)))
+	   ((mew-case-equal mew-ct-mls ctm) (aset marks (1+ (* i 2)) ?S))
+	   ((mew-case-equal "enveloped-data" ctp) (aset marks (1+ (* i 2)) ?E))
+	   ((mew-case-equal "signed-data" ctp) (aset marks (1+ (* i 2)) ?S)))
 	  (setq i (1+ i)))))
 
     (if (and (not dec) decrypters) (setq cd-or-dec decrypters))
@@ -829,7 +848,7 @@ If this is a cache message, both the unique id and the size are also shown."
 	   (mew-xinfo-get-multi-form)))))
 
 (defsubst mew-decode-syntax-buffer ()
-  (set-buffer (marker-buffer mew-marker-decode-syntax-begin)))
+  (marker-buffer mew-marker-decode-syntax-begin))
 
 (defun mew-decode-syntax-delete ()
   (when (mew-decode-syntax-p)
@@ -912,7 +931,7 @@ If this is a cache message, both the unique id and the size are also shown."
 (defun mew-decode-syntax-insert-warning ()
   (when (or (mew-xinfo-get-warning) (mew-xinfo-get-info))
     (let ((beg (point))
-	  (pref (make-string (1+ (length mew-x-mew:)) ?\ )))
+	  (pref (make-string (1+ (length mew-x-mew:)) mew-sp)))
       (when (mew-xinfo-get-warning)
 	(insert mew-x-mew: " "
 		(mapconcat 'identity (mew-xinfo-get-warning) pref)))
@@ -934,6 +953,7 @@ If this is a cache message, both the unique id and the size are also shown."
 (defsubst mew-in-decode-syntax-p ()
   (and (marker-position mew-marker-decode-syntax-begin)
        (marker-position mew-marker-decode-syntax-end)
+       (eq (current-buffer) (marker-buffer mew-marker-decode-syntax-begin))
        (>= (point) (marker-position mew-marker-decode-syntax-begin))
        (<  (point) (marker-position mew-marker-decode-syntax-end))))
 
@@ -951,7 +971,7 @@ If this is a cache message, both the unique id and the size are also shown."
   (and (marker-position mew-marker-decode-syntax-begin)
        (marker-position mew-marker-decode-syntax-end)))
 
-(defsubst mew-msghder-p ()
+(defsubst mew-msghdr-p ()
   (save-restriction
     (widen)
     (next-single-property-change (point-min) 'read-only)))
@@ -972,7 +992,7 @@ If this is a cache message, both the unique id and the size are also shown."
   (marker-position mew-marker-decode-syntax-end))
 
 (defsubst mew-header-end ()
-  (mew-msghder-p))
+  (mew-msghdr-p))
 
 (defsubst mew-attach-begin ()
   (if (mew-attach-p)
@@ -996,13 +1016,14 @@ If this is a cache message, both the unique id and the size are also shown."
   (let (pos beg end)
     (save-excursion
       (set-buffer (mew-decode-syntax-buffer))
-      (setq pos (point))
-      (setq end (mew-decode-syntax-end))
-      (goto-char end)
-      (forward-line -1)
-      (beginning-of-line)
-      (setq beg (point))
-      (and (<= beg pos) (< pos end)))))
+      (save-excursion
+	(setq pos (point))
+	(setq end (mew-decode-syntax-end))
+	(goto-char end)
+	(forward-line -1)
+	(beginning-of-line)
+	(setq beg (point))
+	(and (<= beg pos) (< pos end))))))
 
 ;;
 
@@ -1088,7 +1109,7 @@ If this is a cache message, both the unique id and the size are also shown."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2003 Mew developing team.
+;; Copyright (C) 1996-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
