@@ -154,7 +154,7 @@ All members must have the same length."
 ;;; Making thread
 ;;;
 
-(defun mew-summary-mark-make-thread ()
+(defun mew-summary-mark-thread ()
   "Make threads for messages marked with '*'."
   (interactive)
   (mew-summary-thread-region (point-min) (point-max) 'mark))
@@ -230,7 +230,7 @@ corresponding Summary mode and move to the current message."
     (if args
 	(setq iter (lambda () (re-search-forward regex nil t)))
       (setq iter (lambda () (mew-summary-search-regexp-visible regex))))
-    (mew-summary-thread-region (point-min) (point-max) 'mark nil iter)))
+    (mew-summary-thread-region (point-min) (point-max) nil nil iter)))
 
 (defun mew-summary-thread-region (beg end &optional mark disp-msg iter)
   "Make threads for messages in a region.  If you want to know how
@@ -287,9 +287,10 @@ threads are created, see 'mew-use-complete-thread'."
 	       (mew-thread-set-line me line)
 	       (if (string= my-id "")
 		   (setq top (cons me top))
-		 (mew-thread-set-myid me my-id)
-		 (set (intern my-id db) me)
-		 (if (string= prnt-id "")
+		 ;; some broken messages refer themselves
+		 ;; don't register me here so that his parent
+		 ;; will not be found.
+		 (if (or (string= prnt-id "") (string= my-id prnt-id))
 		     (setq top (cons me top))
 		   (mew-thread-set-prntid me prnt-id)
 		   (setq prnt (symbol-value (intern-soft prnt-id db)))
@@ -298,7 +299,9 @@ threads are created, see 'mew-use-complete-thread'."
 		     (setq prnt-cld (mew-thread-get-child prnt))
 		     (if prnt-cld
 			 (nconc prnt-cld (list me))
-		       (mew-thread-set-child prnt (list me))))))))))
+		       (mew-thread-set-child prnt (list me)))))
+		 (mew-thread-set-myid me my-id)
+		 (set (intern my-id db) me))))))
        ;;
        (if (null me)
  	   (message "No target messages")
@@ -805,9 +808,9 @@ a top node, move onto the top of the previous thread."
 (defvar mew-mark-thread-root ?+)
 
 (defun mew-thread-toggle ()
-  "Hide children and leave the parent only. If its children are hidden,
-\"+\" is displayed on the parent. Also, if this command is executed on a
-mail whose children are hidden, they appear."
+  "If children of a message are displayed, they will hide and
+\"+\" is displayed on the parent. 
+If the children are hidden, they will appear."
   (interactive)
   (mew-thread-only
    (mew-summary-goto-message)
@@ -818,6 +821,24 @@ mail whose children are hidden, they appear."
    (mew-thread-move-cursor)
    (set-buffer-modified-p nil)))
 
+(defun mew-thread-toggle-all ()
+  "Toggle appearance of children for all threads."
+  (interactive)
+  (mew-thread-only
+   (let (here)
+     (save-excursion
+       (goto-char (point-min))
+       (mew-decode-syntax-delete)
+       (while (setq here (mew-thread-next-property2 (point) (point-max) 0))
+	 (goto-char here)
+       (beginning-of-line)
+       (if (looking-at (concat "^." (regexp-quote (char-to-string mew-mark-thread-root))))
+	   (mew-thread-graft)
+	 (mew-thread-prune))
+       (forward-line))
+       (mew-thread-move-cursor)
+       (set-buffer-modified-p nil)))))
+
 (defun mew-thread-all-prune ()
   "Hide all children."
   (interactive)
@@ -930,7 +951,7 @@ The thread/message is specified with the mark(\\[set-mark-command])."
     ;; parent    
     (let ((id (mew-summary-my-id))
 	  fld msg)
-      (goto-char (marker-position (mark-marker)));; user's mark
+      (goto-char (mark-marker));; user's mark
       ;; children
       (when (mew-sumsyn-match mew-regex-sumsyn-long)
 	(setq fld (mew-sumsyn-folder-name))
@@ -944,10 +965,9 @@ The thread/message is specified with the mark(\\[set-mark-command])."
 
 (defun mew-thread-glue-it ()
   (let ((column (mew-vinfo-get-column))
-	(m (make-marker))
 	(width 0) (wd 0) (adjust 0)
 	(prefix "")
-	fld msg id beg end tree indent idt pbeg pindent has-child)
+	fld msg id beg end tree indent idt pbeg pindent has-child m)
     (save-excursion
       ;; parent
       (mew-summary-goto-message)
@@ -976,7 +996,7 @@ The thread/message is specified with the mark(\\[set-mark-command])."
       (setq pindent (1+ pindent))
       (forward-line)
       ;; the next line of parent
-      (set-marker m (point))
+      (setq m (point-marker))
       (unless (looking-at mew-regex-thread-separator)
 	(move-to-column column)
 	(if (and (mew-thread-get-property (point))
@@ -984,7 +1004,7 @@ The thread/message is specified with the mark(\\[set-mark-command])."
 	    (setq has-child t)))
       (move-to-column column)
       ;; children
-      (goto-char (marker-position (mark-marker)));; user's mark
+      (goto-char (mark-marker));; user's mark
       (when (mew-sumsyn-match mew-regex-sumsyn-long)
 	(setq fld (mew-sumsyn-folder-name))
 	(setq msg (mew-sumsyn-message-number)))
@@ -1027,7 +1047,7 @@ The thread/message is specified with the mark(\\[set-mark-command])."
 	 (setq tree (buffer-substring beg end))
 	 (delete-region beg end)
 	 ;; the next line of parent
-	 (goto-char (marker-position m))
+	 (goto-char m)
 	 (insert tree)
 	 (set-buffer-modified-p nil))
 	(mew-summary-change-parent-id fld msg id)
@@ -1036,8 +1056,7 @@ The thread/message is specified with the mark(\\[set-mark-command])."
 (defun mew-summary-change-parent-id (fld msg id)
   (set-buffer fld)
   (save-excursion
-    (goto-char (point-min))
-    (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+    (when (mew-summary-search-msg msg)
       (mew-thread-change-parent-id id))))
 
 (defun mew-thread-change-parent-id (id)
@@ -1049,7 +1068,7 @@ The thread/message is specified with the mark(\\[set-mark-command])."
 
 (defun mew-thread-save-xref (fld msg id)
   (with-temp-buffer
-    (let ((file (mew-expand-folder fld msg)))
+    (let ((file (mew-expand-msg fld msg)))
       (mew-insert-file-contents2 file)
       (mew-header-delete-lines (list mew-x-mew-ref:))
       (goto-char (point-min))
@@ -1060,7 +1079,7 @@ The thread/message is specified with the mark(\\[set-mark-command])."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2005 Mew developing team.
+;; Copyright (C) 2000-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
