@@ -221,10 +221,6 @@ See 'mew-imap-spam-field'."
   :type '(choice (const :tag "Not use" nil)
 		 (string :tag "Spam folder")))
 
-;; must be here
-(defvar mew-basic-folders
-  (list mew-inbox-folder mew-draft-folder mew-queue-folder mew-postq-folder))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Hooks
@@ -437,7 +433,7 @@ on a draft buffer. If nil, this value is automatically generated from
 
 (defcustom mew-mail-domain-list nil
   "*Your e-mail address domain list like 
-(\"Mew.org\" \"iijlab.net\").
+(\"example.org\" \"example.jp\").
 They are used for mail-domain completion in Draft mode(C-cTAB). If
 nil, this value is automatically generated from 'mew-config-alist'."
   :group 'mew-draft
@@ -450,6 +446,26 @@ also 'mew-from'."
   :group 'mew-draft
   :type '(choice (const nil) (repeat string)))
 
+(defcustom mew-warn-addresses nil
+  "A list of addresses to be warned."
+  :group 'mew-draft
+  :type '(choice (const nil) (repeat string)))
+
+(defcustom mew-safe-addresses nil
+  "A list of addresses not to be warned."
+  :group 'mew-draft
+  :type '(choice (const nil) (repeat string)))
+
+(defcustom mew-warn-domains nil
+  "A list of domains to be warned."
+  :group 'mew-draft
+  :type '(choice (const nil) (repeat string)))
+
+(defcustom mew-safe-domains nil
+  "A list of domains not to be warned."
+  :group 'mew-draft
+  :type '(choice (const nil) (repeat string)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
 ;;; Mail Addresses
@@ -555,13 +571,14 @@ use a string. (e.g. \"25\")"
   :type 'boolean)
 
 (defcustom mew-smtp-ssl-port "465" ;; no universal consensus
-  "*The port for SMTP over SSL/TLS."
+  "*The port for SMTP over SSL/TLS. Set this to \"smtp\" if you 
+want to use TLS."
   :group 'mew-smtp
   :type 'string)
 
 (defcustom mew-smtp-user nil
-  "*The user name on your SMTP server. If non-nil, authentication is
-used."
+  "*The user name on your SMTP server. If not configured, 
+your e-mail address is automatically set"
   :group 'mew-smtp
   :type 'string)
 
@@ -720,15 +737,15 @@ If non-nil, only header is retrieved."
   :type 'boolean)
 
 (defcustom mew-pop-proxy-server nil
-  "*The name of PROXY server connecting to POP server."
+  "*The name of POP proxy server."
   :group 'mew-pop
   :type '(choice (const :tag "Not use" nil)
 		 (string :tag "Pop proxy server")))
 
 (defcustom mew-pop-proxy-port nil
-  "*The port of PROXY proxy."
+  "*The port for POP proxy."
   :group 'mew-pop
-  :type 'string)
+  :type '(choice string (const nil)))
 
 (defvar mew-pop-timeout-time 4)
 
@@ -837,6 +854,17 @@ Mail/inbox.
   :group 'mew-imap
   :type '(choice (const nil) (repeat string)))
 
+(defcustom mew-imap-proxy-server nil
+  "*The name of IMAP proxy server."
+  :group 'mew-imap
+  :type '(choice (const :tag "Not use" nil)
+		 (string :tag "Imap proxy server")))
+
+(defcustom mew-imap-proxy-port nil
+  "*The port for IMAP proxy."
+  :group 'mew-imap
+  :type '(choice string (const nil)))
+
 (defvar mew-imap-timeout-time 4)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1195,9 +1223,9 @@ element must be 'name, 'comment, 'address, 'nickname, or appropriate
 
 Consider the following examples:
 
-	A: Kazu Yamamoto <kazu@mew.org>
-	B: kazu@mew.org (Kazu Yamamoto)
-	C: Kazuhiko Yamamoto (Kazu) <kazu@mew.org>
+	A: Kazu Yamamoto <kazu@example.org>
+	B: kazu@example.org (Kazu Yamamoto)
+	C: Kazuhiko Yamamoto (Kazu) <kazu@example.org>
 
 Each element returns the following value:
 
@@ -1210,9 +1238,9 @@ Each element returns the following value:
 				B: Kazu Yamamoto
 				C: (Kazu)
 	'address	Address part
-				A: kazu@mew.org
-				B: kazu@mew.org
-				C: kazu@mew.org
+				A: kazu@example.org
+				B: kazu@example.org
+				C: kazu@example.org
 
 	'nickname       One element of personal information in
                         Addrbook according to 'mew-addrbook-for-summary'.
@@ -1239,6 +1267,16 @@ instead of the 'D' mark."
 		 (const :tag "Not use" nil)
 		 (character :tag "Use another character")))
 
+(defcustom mew-summary-form-mark-spam nil
+  "*If non-nil, the 'D' mark automatically is put onto
+SPAM messages. A message is considered a SPAM message 
+if the MD5 checksum of its body is duplicated in a scan.
+If a character, use it as a mark instead of the 'D' mark."
+  :group 'mew-summary
+  :type '(choice (const :tag "Use mew-mark-delete" t)
+		 (const :tag "Not use" nil)
+		 (character :tag "Use another character")))
+
 (defcustom mew-summary-form-mark-review nil
   "*If non-nil, the '*' mark automatically is put onto
 messages destined to me. If a character, use it as a mark
@@ -1263,19 +1301,13 @@ quantization by 'mew-summary-form-size-unit'."
 
 (defcustom mew-spam: "X-Bogosity:"
   "*The field name which your spam filter inserts. 
-This variable MUST be set in \".emacs\".
 E.g.
 	\"X-Spam-Flag:\" for 'spamassassin'
 	\"X-Bogosity:\" for 'bogofilter'"
   :group 'mew-summary
   :type 'string)
 
-(defcustom mew-scan-fields
-  (list "Folder:" "Filename:" 
-	mew-subj: mew-date: mew-from: mew-to: mew-cc:
-	mew-ct: mew-cte: mew-x-mew-uidl:
-	mew-message-id: mew-in-reply-to: mew-references: mew-x-mew-ref:
-	mew-spam: "Body")
+(defcustom mew-scan-fields nil
   "*A list which specifies mewl's output.
 The first element MUST be \"Folder:\".
 The second element MUST be \"Filename:\".
@@ -1286,11 +1318,7 @@ The last MUST be \"Body\"."
                (cons (const "Filename:")
                      (repeat string))))
 
-(defvar mew-scan-fields-alias
-  '("FLD" "NUM" "SUBJ" "DATE" "FROM" "TO" "CC"
-    "CT" "CTE" "UID"
-    "ID" "IRT" "REF" "XREF"
-    "SPAM" "BODY")
+(defvar mew-scan-fields-alias nil
   "*A list of aliases for 'mew-scan-fields'.
 Functions called MEW-FOO will be defined according to this variable.")
 
@@ -1299,6 +1327,10 @@ Functions called MEW-FOO will be defined according to this variable.")
 
 (defvar mew-scan-decode-fields (list mew-subj: mew-from: mew-to:))
 
+(defvar mew-draft-address-warning-fields
+  (list mew-to: mew-cc: mew-dcc: mew-bcc:
+	mew-resent-to: mew-resent-cc: mew-resent-dcc: mew-resent-bcc:))
+
 (defcustom mew-thread-column 28
   "A position where thread indentation is inserted
 for indentation of thread. This valuable is for backward compatibility
@@ -1395,6 +1427,13 @@ to this file."
   :group 'mew-env
   :type 'integer)
 
+(defcustom mew-scan-body-length 10
+  "*Body length which Mew treats.
+Increase this value if 'mew-summary-form-mark-spam' put
+the 'D' mark onto different messages."
+  :group 'mew-env
+  :type 'integer)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Ask?
@@ -1463,11 +1502,6 @@ invalid. Otherwise, 'update is used for range in the case."
   :group 'mew-summary
   :type 'boolean)
 
-(defcustom mew-ask-virtual-folder-name nil
-  "*If *non-nil*, ask a virtual folder name."
-  :group 'mew-summary
-  :type 'boolean)
-
 (defcustom mew-ask-charset nil
   "*Mew asks a user whether or not the charset chosen on composing
 is appropriate.
@@ -1588,6 +1622,11 @@ created (if the feature is provided)."
   :group 'mew-privacy
   :type 'boolean)
 
+(defcustom mew-use-master-passwd nil
+  "*if non-nil, a master password is used."
+  :group 'mew-privacy
+  :type 'boolean)
+
 (defcustom mew-use-8bit nil
   "*If non-nil, the 8bit-clean charset mechanism is used for 8bit
 charsets, such as ISO-8859-1, in Draft mode. If nil, an appropriate
@@ -1636,7 +1675,7 @@ your POP server becomes non-zero. To use this feature,
 (defcustom mew-use-sender nil
   "*If From: contains multiple addresses, Mew adds Sender: anyway. 
 If From: contains one address and SMTP MAIL FROM
-('mew-smtp-mail-from') is different from it, this variable effects.
+\('mew-smtp-mail-from') is different from it, this variable effects.
 If non-nil, Mew adds Sender: and specifies SMTP MAIL FROM to its value.
 Otherwise, Mew does not add Sender:."
   :group 'mew-draft
@@ -1731,6 +1770,32 @@ the unread mark is deleted."
   :group 'mew-summary
   :type 'boolean)
 
+(defcustom mew-use-smtp-auth t
+  "*If nil, SMTP AUTH is ignored even an SMTP server
+returns the AUTH capability."
+  :group 'mew-smtp
+  :type 'boolean)
+
+(defcustom mew-smtp-auth-plain-authorize-id nil
+  "*If nil, SMTP AUTH PLAIN is created as follow:
+	NUL authenticate-id NUL password
+If non-nil, it is created as follow:
+	authorize-id NUL authenticate-id NUL password
+For more information, see RFC 2595."
+  :group 'mew-smtp
+  :type 'boolean)
+
+(defcustom mew-use-punycode t
+  "*If non-nil, punycode strings are decoded into IDN strings"
+  :group 'mew-message
+  :type 'boolean)
+
+(defcustom mew-use-async-write nil
+  "*If non-nil, message retrieval of POP/IMAP/NNTP on Unix uses
+asynchronous write for each message and fsync in their sentinel."
+  :group 'mew-net
+  :type 'boolean)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Appearance
@@ -1796,6 +1861,12 @@ See also 'mew-summary-show-direction'."
   :group 'mew-summary
   :type '(choice (const next) (const up) (const down) (const stop)))
 
+(defcustom mew-summary-show-pause nil
+  "*Pause on going to next message if *non-nil*.
+See also 'mew-summary-show-direction' and 'mew-summary-mark-direction'."
+  :group 'mew-summary
+  :type 'boolean)
+
 (defcustom mew-summary-mark-duplicated-skip 'first
   "*This value decides which duplicated message should not be
 marked with 'mew-mark-duplicated' when 'mew-summary-mark-duplicated'
@@ -1918,7 +1989,7 @@ not work resulting that no garbage file remains."
 (defcustom mew-header-alist nil
   "*Alist of key and value pair for header field to be inserted on draft.
 '((\"X-fingerprint:\" . \"6B 63 38 88 67 5E 96 8E  CE A4 62 73 3F 11 64 94\")
-  (\"X-URL:\" . \"http://www.Mew.org/~kazu/\"))"
+  (\"X-URL:\" . \"http://www.example.org/\"))"
   :group 'mew-draft
   :type 'sexp)
 
@@ -1961,8 +2032,16 @@ See also 'mew-config-alist'."
   :group 'mew-draft
   :type '(choice string (const nil)))
 
+(defcustom mew-bcc nil
+  "*A value inserted into Bcc: field in Draft mode if *non-nil*.
+See also 'mew-config-alist'."
+  :group 'mew-draft
+  :type '(choice string (const nil)))
+
 (defcustom mew-dcc nil
   "*A value inserted into Dcc: field in Draft mode if *non-nil*.
+If the value is 'me, your address is automatically inserted
+according to the case.
 See also 'mew-config-alist'."
   :group 'mew-draft
   :type '(choice string (const nil)))
@@ -2272,7 +2351,8 @@ circular domain completion."
 	    mew-date: mew-message-id: mew-mv: "Content-"
 	    "Precedence:" mew-organization: "X-"
 	    "Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:"
-	    "DomainKey-Signature:"))
+	    "DomainKey-Signature:" "DKIM-Signature:"
+	    "Authentication-Results:"))
   "*A field list to be deleted when edit again."
   :group 'mew-draft
   :type '(choice (const nil)
@@ -2282,7 +2362,7 @@ circular domain completion."
   (list mew-received: "Return-Path:" "Delivery-Date:" "Delivered-To:"
 	mew-to: mew-cc: mew-from: mew-organization:
 	"Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:"
-	"DomainKey-Signature:")
+	"DomainKey-Signature:" "DKIM-Signature:")
   "*A field list to be deleted when send with different To: and Cc:."
   :group 'mew-draft
   :type '(choice (const nil)
@@ -2368,10 +2448,10 @@ on '\\<mew-summary-mode-map>\\[mew-summary-save]'. Otherwise overwrote."
 ;;; Folders
 ;;;
 
-(defcustom mew-regex-folder-candidate "^[^.0-9]\\|^[0-9].*[^0-9]"
+(defcustom mew-regex-folder-candidate "^[^.#0-9]\\|^[0-9].*[^0-9]"
   "*Regular expression used in 'mew-dir-list-with-link-count'
 and 'mew-dir-list-without-link-count' to list up folder
-candidates. The default value \"^[^.0-9]\\|^[0-9].*[^0-9]\" lists up 
+candidates. The default value \"^[^.#0-9]\\\\|^[0-9].*[^0-9]\" lists up 
 folders whose name is alphabet only."
   :group 'mew-env
   :type 'regexp)
@@ -2693,7 +2773,13 @@ in Summary/Virtual mode."
   :type 'face)
 
 (defcustom mew-regex-url
-  "\\b\\(s?https?\\|ftp\\|file\\|gopher\\|news\\|telnet\\|wais\\|mailto\\):\\(//[-a-zA-Z0-9_.]+:[0-9]*\\)?[-a-zA-Z0-9_=?#$@~`%&*+|\\/.,:;]*[-a-zA-Z0-9_=#$@~`%&*+|\\/]"
+  (concat
+   "\\b\\("
+   "\\(\\(file\\|news\\|mailto\\):\\)"
+   "\\|"
+   "\\(\\(s?https?\\|ftp\\|gopher\\|telnet\\|wais\\)://\\)"
+   "\\)"
+   "[^ $B!!(B\t\n>)\"]+")
   "*Regular expression to find URL."
   :group 'mew-highlight
   :type 'regexp)
@@ -2802,7 +2888,10 @@ appropriate for this value."
                  (const pgp-encryption)
                  (const pgp-signature-encryption)
 		 (const pgp-encryption-signature)
-		 (const smime-signature) ;; xxx
+		 (const smime-signature)
+                 (const smime-encryption)
+                 (const smime-signature-encryption)
+		 (const smime-encryption-signature)
                  (other :tag "nil" nil)))
 
 (defcustom mew-protect-privacy-encrypted nil
@@ -2931,9 +3020,6 @@ your public key as well as receivers' one."
 (defvar mew-prog-grep "grep")
 (defvar mew-prog-grep-opts '("-l" "-e"))
 
-(defvar mew-prog-vgrep "grep")
-(defvar mew-prog-vgrep-opts '("-l" "-e"))
-
 (defvar mew-prog-grep-max-msgs 10000)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -2994,6 +3080,15 @@ w3 is used."
 (defvar mew-mode-name-summary "Summary")
 (defvar mew-mode-name-virtual "Virtual")
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Types
+;;;
+
+
+(defvar mew-file-type "MewX")
+(defvar mew-file-creator "EMAx")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; For broken MUAs
@@ -3006,10 +3101,10 @@ w3 is used."
 
 Good example:
 	From: Kazu Yamamoto (=?ISO-2022-JP?B?GyRCOzNLXE9CSScbKEI=?=) 
-		<kazu@iijlab.net>
+		<kazu@example.org>
 Bad example:
 	From: Kazu Yamamoto (\"=?ISO-2022-JP?B?GyRCOzNLXE9CSScbKEI=?=\") 
-		<kazu@iijlab.net>
+		<kazu@example.org>
 
 (2) Decode raw non-ASCII text, which violates RFC 822.
 This is typically found in Subject: field.
@@ -3091,11 +3186,24 @@ Level 2: syntax error."
   :group 'mew-draft
   :type 'hook)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Preventing waring messages when byte-compiling
+;;;
+
+(defvar mew-regex-message-files  nil)
+(defvar mew-regex-message-files2 nil)
+(defvar mew-regex-message-files3 nil)
+(defvar mew-regex-message-files4 nil)
+(defvar mew-regex-message-files5 nil)
+
+(defvar mew-basic-folders nil)
+
 (provide 'mew-vars)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
