@@ -232,6 +232,7 @@ in draft buffer.")
     ("Resent-Cc:"   . mew-complete-address)
     ("Resent-Dcc:"  . mew-complete-address)
     ("Resent-Bcc:"  . mew-complete-address)
+    ("Resent-Fcc:"  . mew-complete-local-folder)
     ("Newsgroups:"  . mew-complete-newsgroups)
     ("Followup-To:" . mew-complete-newsgroups))
   "*Completion function alist concerned with the key."
@@ -279,110 +280,7 @@ in draft buffer.")
 (defvar mew-content-type mew-ct-txt
   "*The default Content-Type: for a file whose suffix is unknown.")
 
-(defvar mew-mime-content-type
-  '(("multipart"    nil         nil     nil            mew-icon-multipart)
-    ;;
-    ("audio/basic"  "\\.au$"    mew-b64 mew-prog-audio mew-icon-audio)
-    ("audio/x-wav"  "\\.wav$"   mew-b64 mew-prog-audio mew-icon-audio)
-    ("audio/x-aiff" "\\.aif?f$" mew-b64 mew-prog-audio mew-icon-audio)
-    ("audio/x-midi" "\\.midi?$" mew-b64 mew-prog-audio mew-icon-audio)
-    ("audio/x-mpeg" "\\.mpga$\\|\\.mp[23]$" mew-b64 mew-prog-audio mew-icon-audio)
-    ;;
-    ("image/gif"   "\\.gif$"   mew-b64 mew-prog-image mew-icon-image gif)
-    ("image/tiff"  "\\.tif?f$" mew-b64 mew-prog-image mew-icon-image tiff)
-    ("image/jpeg"  "\\.jpe?g$" mew-b64 mew-prog-image mew-icon-image jpeg)
-    ("image/png"   "\\.png$"   mew-b64 mew-prog-image mew-icon-image png)
-    ("image/x-xwd" "\\.xwd$"   mew-b64 mew-prog-image mew-icon-image xwd)
-    ("image/x-xbm" "\\.xbm$"   mew-b64 mew-prog-image mew-icon-image xbm)
-    ("image/x-xpm" "\\.xpm$"   mew-b64 mew-prog-image mew-icon-image xpm)
-    ("image/x-bmp" "\\.bmp$"   mew-b64 mew-prog-image mew-icon-image bmp)
-    ("image/x-pcx" "\\.pcx$"   mew-b64 mew-prog-image mew-icon-image PCX)
-    ("image/x-tga" "\\.tga$"   mew-b64 mew-prog-image mew-icon-image TGA)
-    ("image"       "^$"        mew-b64 mew-prog-image mew-icon-image)
-    ;;
-    ("model/iges" "\\.ige?s$" mew-b64 mew-prog-iges  mew-icon-image) ;; xxx
-    ("model/vrml" "\\.wrl$"   mew-b64 mew-prog-vrml  mew-icon-image)
-    ("model/mesh" "\\.me?sh$" mew-b64 mew-prog-mesh  mew-icon-image)
-    ("model"      "^$"        mew-b64 mew-prog-model mew-icon-image)
-    ;;
-    ("video/mpeg"      "\\.mpe?g$" mew-b64 mew-prog-video mew-icon-video)
-    ("video/quicktime" "\\.mov$"   mew-b64 mew-prog-video mew-icon-video)
-    ("video/x-msvideo" "\\.avi$"   mew-b64 mew-prog-video mew-icon-video)
-    ;;
-    ("message/rfc822"          "^[0-9]+$" nil
-     mew-prog-rfc822           mew-icon-message/rfc822)
-    ("message/external-body"   "\\.ext$"  nil
-     mew-prog-external-body    mew-icon-message/external-body)
-    ("message/delivery-status" "^$"       nil
-     mew-prog-delivery-status  mew-icon-text)
-    ;;
-    ("application/postscript"        "\\.e?ps$" mew-qp
-     mew-prog-postscript             mew-icon-application/postscript)
-    ("application/pdf"               "\\.pdf$"  mew-b64
-     mew-prog-pdf                    mew-icon-application/postscript)
-    ("application/msword"            "\\.doc$"  mew-b64
-     mew-prog-msword                 mew-icon-text)
-    ("application/vnd.ms-excel"      "\\.xl[st]$" mew-b64
-     mew-prog-msexcel                mew-icon-text)
-    ("application/vnd.ms-powerpoint" "\\.ppt$"  mew-b64
-     mew-prog-mspowerpoint           mew-icon-text)
-    ("application/vnd.visio"         "\\.vsd$"  mew-b64
-     mew-prog-visio                  mew-icon-text)
-    ("application/vnd.sun.xml.writer"           "\\.sxw$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.writer.template"  "\\.stw$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.calc"             "\\.sxc$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.calc.template"    "\\.stc$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.draw"             "\\.sxd$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.draw.template"    "\\.std$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.impress"          "\\.sxi$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.impress.template" "\\.sti$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.writer.global"    "\\.sxg$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.math"             "\\.sxm$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/ms-tnef"           "\\.dat$" mew-b64
-     mew-prog-mstnef                 mew-icon-unknown)
-    ("application/rtf"               "\\.rtf$" mew-b64
-     mew-prog-rtf                    mew-icon-text)
-    ("application/pgp-keys"          "\\.pka$"  nil
-     mew-prog-pgp-keys               mew-icon-unknown)
-    ("application/x-pkcs7-signature" "\\.p7s$"  mew-b64
-     nil			     mew-icon-unknown)
-    ("application/vnd.fujitsu.oasys" "\\.oas$"  mew-b64
-     mew-prog-oasys                  mew-icon-text)
-    ("application/vnd.fujitsu.oasys2" "\\.oa2$" mew-b64
-     mew-prog-oasys                  mew-icon-text)
-    ("application/vnd.fujitsu.oasys3" "\\.oa3$" mew-b64
-     mew-prog-oasys                  mew-icon-text)
-    ("application/octet-stream"
-     "\\.tar$\\|\\.tar\\.\\|\\.gz$\\|\\.Z$\\|\\.taz$\\|\\.tgz$\\|\\.tbz$\\|\\.bz2?$\\|\\.lzh$\\|\\.zip$\\|\\.bin$\\|\\.pgp$\\|\\.gpg$\\|\\.exe$\\|\\.dll$"
-      mew-b64 mew-prog-octet-stream mew-icon-application/octet-stream)
-    ;;
-    ("text/html"     "\\.html?$" nil     mew-prog-html      mew-icon-text)
-    ("text/enriched" "\\.rtf$"   nil     mew-prog-enriched  mew-icon-text)
-    ("text/css"      "\\.css$"   nil     mew-prog-text      mew-icon-text)
-    ("text/sgml"     "\\.sgml$"  nil     mew-prog-text      mew-icon-text)
-    ("text/plain"    "\\.txt$\\|\\.c$\\|\\.h$\\|\\.el$\\|\\.diff$\\|\\.patch$"
-                                 nil     mew-prog-plain     mew-icon-text)
-    ("text/xml"      "\\.xml$"   nil     mew-prog-xml       mew-icon-text)
-    ("text/rfc822-headers" "\\.hdr$" nil
-     mew-prog-rfc822-headers     mew-icon-message/rfc822)
-    ("text"         "^$"        nil     mew-prog-text      mew-icon-text)
-    ;; Unknown CT: matches here.
-    ("application/xml"               "\\.xml$"  mew-b64
-     mew-prog-xml2                   mew-icon-text)
-    (t               "^$"       nil   mew-prog-octet-stream mew-icon-unknown)
-    ;; Unknown suffix matches here and return the entry specified 
-    ;; by mew-content-type.
-    (nil             ".*"))
+(defvar mew-mime-content-type nil
   "\\<mew-summary-mode-map>Database for MIME content type.
 This is a list of entries which consist of <content-type>, 
 <regular expression of file suffix>, <content-transfer-encoding>, 
@@ -479,6 +377,10 @@ the next version of Mew.")
 (defsubst mew-mime-image-format-name (ct)
   (nth 5 (mew-ctdb-by-ct ct)))
 
+(defvar mew-mime-content-type-for-ooffice nil
+  "A list of (content-type suffix) for OpenOffice.org to be prepended
+'mew-mime-content-type'")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; MIME Content-Disposition:
@@ -513,7 +415,12 @@ the next version of Mew.")
      ((,mew-ct-mls ,mew-ct-pgs) (,mew-ct-mle ,mew-ct-pge)) "PSPE")
     (pgp-encryption-signature
      ((,mew-ct-mle ,mew-ct-pge) (,mew-ct-mls ,mew-ct-pgs)) "PEPS")
-    (smime-signature  ((,mew-ct-mls ,mew-ct-sms)) "SS"))
+    (smime-signature  ((,mew-ct-smm ,mew-ct-smm-sig)) "SS")
+    (smime-encryption ((,mew-ct-smm ,mew-ct-smm-enc)) "SE")
+    (smime-signature-encryption
+     ((,mew-ct-smm ,mew-ct-smm-sig) (,mew-ct-smm ,mew-ct-smm-enc)) "SSSE")
+    (smie-encryption-signature
+     ((,mew-ct-smm ,mew-ct-smm-enc) (,mew-ct-smm ,mew-ct-smm-sig)) "SESS"))
   "*Alist of key, a list of privacy Content-Type, and its mark."
   :group 'mew-privacy
   :type 'sexp)
@@ -578,7 +485,8 @@ the next version of Mew.")
     (,mew-in-reply-to:   unstruct   unstruct)
     (,mew-x-face:	 unstruct   unstruct)
     (,mew-face:          unstruct   unstruct)
-    ("Domainkey-Signature:" unstruct unstruct)) ;; capitalized
+    ("Domainkey-Signature:" unstruct unstruct) ;; capitalized
+    ("Dkim-Signature:"      unstruct unstruct)) ;; capitalized
   "(field enc dec)")
 
 (defsubst mew-field-type-for-encoding (key)
@@ -592,109 +500,8 @@ the next version of Mew.")
 ;;; Theme
 ;;;
 
-(defvar mew-theme-file "mew-theme"
-  "The file name which contains theme.
-This variable should be set in \".emacs\"(not in \".mew\").")
-
-(defface mew-face-header-subject nil
-  "*Face to highlight the value of Subject:"
-  :group 'mew-highlight)
-
-(defface mew-face-header-from nil
-  "*Face to highlight the value of From:"
-  :group 'mew-highlight)
-
-(defface mew-face-header-to nil
-  "*Face to highlight the value of To:"
-  :group 'mew-highlight)
-
-(defface mew-face-header-date nil
-  "*Face to highlight the value of Date:"
-  :group 'mew-highlight)
-
-(defface mew-face-header-key nil
-  "*Face to highlight by default"
-  :group 'mew-highlight)
-
-(defface mew-face-header-private nil
-  "*Face to highlight private field-keys"
-  :group 'mew-highlight)
-
-(defface mew-face-header-important nil
-  "*Face to highlight important field-keys"
-  :group 'mew-highlight)
-
-(defface mew-face-header-marginal nil
-  "*Face to highlight marginal field-values"
-  :group 'mew-highlight)
-
-(defface mew-face-header-xmew nil
-  "*Face to highlight the value of X-Mew:"
-  :group 'mew-highlight)
-
-(defface mew-face-header-xmew-bad nil
-  "*Face to highlight the value of X-Mew: in bad cases"
-  :group 'mew-highlight)
-
-(defface mew-face-body-url nil
-  "*Face to highlight URL in Message/Draft mode"
-  :group 'mew-highlight)
-  
-(defface mew-face-body-comment nil
-  "*Face to highlight comments in a body"
-  :group 'mew-highlight)
-
-(defface mew-face-body-cite1 nil
-  "*Face to highlight the first citation"
-  :group 'mew-highlight)
-
-(defface mew-face-body-cite2 nil
-  "*Face to highlight the second citation"
-  :group 'mew-highlight)
-
-(defface mew-face-body-cite3 nil
-  "*Face to highlight the third citation"
-  :group 'mew-highlight)
-
-(defface mew-face-body-cite4 nil
-  "*Face to highlight the forth citation"
-  :group 'mew-highlight)
-
-(defface mew-face-body-cite5 nil
-  "*Face to highlight the fifth citation"
-  :group 'mew-highlight)
-
-(defface mew-face-mark-review nil
-  "*Face to highlight the review mark"
-  :group 'mew-highlight)
-
-(defface mew-face-mark-escape nil
-  "*Face to highlight the escape mark"
-  :group 'mew-highlight)
-
-(defface mew-face-mark-delete nil
-  "*Face to highlight the delete mark"
-  :group 'mew-highlight)
-
-(defface mew-face-mark-unlink nil
-  "*Face to highlight the unlink mark"
-  :group 'mew-highlight)
-
-(defface mew-face-mark-refile nil
-  "*Face to highlight the refile mark"
-  :group 'mew-highlight)
-
-(defface mew-face-mark-unread nil
-  "*Face to highlight the unread mark"
-  :group 'mew-highlight)
-
-(defface mew-face-eof-message nil
-  "*Face to highlight the 'end of message' string"
-  :group 'mew-highlight)
-
-(defface mew-face-eof-part nil
-  "*Face to highlight the 'end of part' string"
-  :group 'mew-highlight)
+(defvar mew-theme-file nil
+  "The file name which contains theme.")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -734,7 +541,9 @@ This variable should be set in \".emacs\"(not in \".mew\").")
     ("^NNTP-Posting-" nil)
     ("^\\(Message-Id\\|Posted\\|In-Reply-To\\|References\\|Precedence\\):$" nil)
     ("^DomainKey-Signature:$" nil)
-    ("^Authentication-Results:$" t
+    ("^DKIM-Signature:$" nil)
+    ("^Authentication-Results:$"
+     mew-authentication-results
      mew-face-header-important
      mew-face-header-xmew
      ("fail" mew-face-header-xmew-bad
@@ -785,6 +594,10 @@ present, mew-face-header-marginal is used."
 (defsubst mew-nspec-extraface (nspec)
   (nth 5 nspec))
 
+(defun mew-authentication-results (val)
+  (save-match-data
+    (or (string-match "=fail" val) (string-match "=softfail" val))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Highlight body
@@ -950,7 +763,7 @@ function which returns nil or a mark or a string.
 If you are using 'spamassassin' which adds the \"X-Spam-Flag:\" field
 to messages and puts them in your mailbox, configure as follows:
 
-(setq mew-spam: \"X-Spam-Flag:\") ;; must be in \".emacs\"
+(setq mew-spam: \"X-Spam-Flag:\")
 
 (defun mew-spam-assassin (val)
   (let ((case-fold-search t))
@@ -967,7 +780,7 @@ value is \"Yes\" are automatically marked with 'D' since the
 If you are using 'bogofilter' which adds the \"X-Bogosity:\" field
 to messages and puts them to your mailbox, configure as follows:
 
-(setq mew-spam: \"X-Bogosity:\") ;; must be in \".emacs\" but default
+(setq mew-spam: \"X-Bogosity:\")
 
 (defun mew-spam-bogofilter (val)
   (let ((case-fold-search t))
@@ -1053,7 +866,7 @@ If you want to use \"lynx\" instead of \"mozilla\", configure like this:
   (or proto (setq proto (mew-proto case)))
   (cond
    ((mew-folder-popp  proto) mew-pop-inbox-folder)
-   ((mew-folder-imapp proto) mew-imap-inbox-folder)
+   ((mew-folder-imapp proto) (mew-imap-inbox-folder case))
    ((mew-folder-nntpp proto) (mew-nntp-newsgroup case))
    (t ;; local 
     (mew-inbox-folder case))))
@@ -1111,13 +924,16 @@ Currently, the following keys are supported:
 \"imap-user\", \"imap-auth\", \"imap-auth-list\", 
 \"imap-size\", \"imap-header-only\", \"imap-delete\", 
 \"imap-trash-folder\", \"imap-queue-folder\", \"imap-spam-field\", \"imap-spam-word\",
+\"imap-proxy-server\", \"imap-proxy-port\",
 \"nntp-server\", \"nntp-port\", \"nntp-ssh-server\", \"nntp-ssl\", \"nntp-ssl-port\", 
 \"nntp-user\", \"nntp-size\", \"nntp-header-only\", 
 \"nntp-msgid-user\", \"nntp-msgid-domain\", 
+\"ssl-cert-directory\", \"ssl-verify-level\", 
 \"inbox-folder\", \"queue-folder\", \"postq-folder\", 
 \"mailbox-type\", \"mbox-command\", \"mbox-command-arg\", 
-\"use-retrieve\", \"signature-file\", \"content-type\", \"refile-guess-alist\",
-\"spam-prog\", \"spam-prog-args\", \"ham-prog\", \"ham-prog-args\".
+\"signature-file\", \"content-type\", \"refile-guess-alist\",
+\"spam-prog\", \"spam-prog-args\", \"ham-prog\", \"ham-prog-args\",
+\"use-old-pgp\".
 
 from = name <user@mail-domain>
 message-id = *random*.smtp-msgid-user@smtp-msgid-domain
@@ -1127,15 +943,15 @@ An example is as follows:
 
 (setq mew-config-alist
       '((\"mew\"
-	 (\"mail-domain\"  . \"mew.org\")
+	 (\"mail-domain\"  . \"example.org\")
 	 (\"inbox-folder\" . \"+inbox-mew\"))
 	(\"keio\"
-	 (\"cc\"           . \"kazu@iijlab.net\")
+	 (\"cc\"           . \"kazu@example.jp\")
 	 (\"user\"         . \"pooh\")
-	 (\"mail-domain\"  . \"sfc.keio.ac.jp\"))
+	 (\"mail-domain\"  . \"example.net\"))
 	(\"default\"
 	 (\"name\"         . \"Kazu Yamamoto\")
-	 (\"mail-domain\"  . \"iijlab.net\"))))
+	 (\"mail-domain\"  . \"example.jp\"))))
 "
   :group 'mew-env
   :type '(alist :key-type string
@@ -1150,7 +966,7 @@ An example is as follows:
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2005 Mew developing team.
+;; Copyright (C) 2000-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
