@@ -52,127 +52,111 @@ For more information, see the document of 'mew-summary-mode'."
   (mew-highlight-cursor-line)
   (run-hooks 'mew-virtual-mode-hook))
 
-(defun mew-summary-virtual (&optional ext)
-  "Create Virtual mode with specified pattern.
-If called with '\\[universal-argument]',
-execute 'mew-summary-virtual-with-external'. Otherwise, 
-'mew-summary-virtual-with-internal' is called."
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Making Virtual mode
+;;;
+
+(defun mew-summary-virtual (&optional regionp)
+  "Making Virtual mode according to a specified pick pattern.
+'mewl' or 'grep' is called as a picking command. If called with
+'\\[universal-argument]', the target is the region."
   (interactive "P")
-  (if (not (mew-summary-or-virtual-p))
-      (message "This command cannot be used in this mode")
-    (if ext
-	(if mew-summary-virtual-with-external-function
-	    (mew-summary-virtual-with-external)
-	  (message "mew-summary-virtual-with-external-function is nil"))
-      (mew-summary-virtual-with-internal))))
+  (mew-pickable
+   (mew-summary-with-mewl
+    (let* ((folder (mew-pickable-folder))
+	   (msgs (mew-summary-pick-msgs folder regionp))
+	   (prompt (format "%s/%s virtual" mew-prog-mewl mew-prog-grep))
+	   (prog mew-prog-grep)
+	   (opts mew-prog-grep-opts)
+	   mew-inherit-pick-mewlp
+	   grepp pattern prog-opts-pat
+	   pfolder vfolder lra)
+      (if (not msgs)
+	  (message "No message")
+	(setq pattern (mew-input-pick-pattern prompt))
+	(cond
+	 ((string= pattern "")
+	  (setq prog-opts-pat (mew-input-pick-command prog opts))
+	  (mew-set '(prog opts pattern) prog-opts-pat)
+	  (setq grepp t))
+	 (t
+	  (setq pattern (mew-pick-canonicalize-pattern pattern))
+	  (unless mew-inherit-pick-mewlp (setq grepp t))))
+	(if (and grepp (not (mew-which-exec prog)))
+	    (message "'%s' not found" prog)
+	  (setq pfolder (mew-expand-folder2 folder))
+	  (setq lra (list (cons pfolder folder)))
+	  (setq vfolder (mew-folder-to-virtual folder))
+	  (mew-summary-switch-to-folder vfolder)
+	  (when (mew-summary-exclusive-p)
+	    (mew-vinfo-set-flds (list folder))
+	    (cond
+	     (grepp
+	      (mew-sinfo-set-find-key pattern)
+	      (message "Picking messages in %s..." folder)
+	      (mew-summary-virtual-with-grep prog opts pattern folder msgs pfolder lra))
+	     (t
+	      (mew-sinfo-set-find-key nil)
+	      (message "Picking messages in %s..." folder)
+	      (mew-summary-virtual-with-mewl pattern folder msgs pfolder lra))))))))))
 
-(defun mew-summary-get-vfolder (flds)
-  (let ((vfolder (mew-folder-to-virtual (mapconcat 'identity flds ","))))
-    (when mew-ask-virtual-folder-name
-      (setq vfolder (mew-input-string
-		     "Virtual folder name %s(%s): " "" vfolder)))
-    (unless (mew-folder-virtualp vfolder)
-      (setq vfolder (mew-folder-to-virtual vfolder)))
-    vfolder))
+(defun mew-summary-virtual-with-mewl (pattern folder src-msgs pfolder lra)
+  "Create Virtual mode with 'mewl'"
+  (let ((opts (list "-a" "-p" pattern "-b" mew-mail-path))
+	(range (mew-summary-pick-range src-msgs)))
+    (setq pfolder (mew-scan-mewl-folder pfolder))
+    (if range
+	(setq opts (nconc opts (list pfolder range)))
+      (setq opts (nconc opts (list pfolder))))
+    (mew-local-retrieve 'vir opts nil lra)))
 
-(defun mew-summary-virtual-with-internal ()
-  "Create Virtual mode with mewl."
-  (let* ((flds (mew-input-folders (mew-summary-physical-folder)))
-	 folders vfolder pattern opts lra)
-    (when flds
-      (setq folders (mapcar 'mew-expand-folder2 flds))
-      (setq pattern (mew-input-pick-pattern))
-      (setq vfolder (mew-summary-get-vfolder flds))
-      (mew-summary-switch-to-folder vfolder)
-      (when (mew-summary-exclusive-p)
-	(mew-vinfo-set-flds flds)
-	(while folders
-	  (setq opts (cons (car folders) (cons "-s" opts)))
-	  (setq lra (cons (cons (car folders) (car flds)) lra))
-	  (setq flds (cdr flds))
-	  (setq folders (cdr folders)))
-	(setq opts (cons "-a" (cons "-p" (cons pattern (nreverse opts)))))
-	(mew-local-retrieve 'vir opts nil lra)))))
-
-(defvar mew-summary-virtual-with-external-function
-  'mew-summary-virtual-with-grep
-  "*A function to be called by '\\[universal-argument] \\<mew-summary-mode-map>\\[mew-summary-virtual]'.
-This function MUST returns a file name which contains message numbers.")
-
-(defun mew-summary-virtual-with-external ()
-  "Create Virtual mode with 'mew-summary-virtual-with-external-function'."
+(defun mew-summary-virtual-with-grep (prog opts pattern folder msgs pfolder lra)
+  "Create Virtual mode with 'grep'"
   (interactive)
-  (let* ((flds (mew-input-folders (mew-summary-physical-folder)))
-	 folders vfolder pattern file func opts lra)
-    (when flds
-      (setq folders (mapcar 'mew-expand-folder2 flds))
-      (setq pattern (mew-input-grep-pattern))
-      (mew-sinfo-set-find-key pattern) ;; xxx necessary?
-      (setq vfolder (mew-summary-get-vfolder flds))
-      (mew-summary-switch-to-folder vfolder)
-      (when (mew-summary-exclusive-p)
-	(with-temp-buffer
-	  (mew-piolet
-	   mew-cs-text-for-read mew-cs-text-for-write
-	   (mew-alet
-	    (setq file (funcall mew-summary-virtual-with-external-function
-				folders pattern)))))
-	(mew-vinfo-set-flds flds)
-	(setq func `(lambda () (mew-delete-file ,file)))
-	(setq opts (list "-i" file))
-	(while folders
-	  (setq lra (cons (cons (car folders) (car flds)) lra))
-	  (setq flds (cdr flds))
-	  (setq folders (cdr folders)))
-	(mew-local-retrieve 'vir opts func lra)))))
+  (let ((file-rttl (mew-summary-virtual-with-grep1 prog opts pattern pfolder msgs))
+	file rttl func args)
+    (mew-set '(file rttl) file-rttl)
+    (setq func `(lambda () (mew-delete-file ,file)))
+    (setq args (list "-i" file))
+    (mew-local-retrieve 'vir args func lra nil rttl)))
 
-(defun mew-summary-virtual-with-grep (flds pattern)
-  (setq pattern (mew-cs-encode-arg pattern))
-  (let ((file (mew-make-temp-name))
-	picks fld dir msgs nxt)
-    (while flds
-      (setq dir (mew-expand-folder (car flds)))
-      (let ((default-directory dir))
-	(setq msgs (mew-dir-messages "."))
-	(if (= (length msgs) 1) (setq msgs (cons null-device msgs)))
-	(cd dir)
-	(mew-erase-buffer)
-	(message "Picking in %s..." (car flds))
+(defun mew-summary-virtual-with-grep1 (prog opts pattern folder msgs)
+  (let ((dir (mew-expand-folder folder))
+	(file (mew-make-temp-name))
+	(rttl 0) nxt)
+    (if (= (length msgs) 1) (setq msgs (cons null-device msgs)))
+    (if pattern (setq pattern (mew-cs-encode-arg pattern)))
+    (with-temp-buffer
+      (mew-set-buffer-multibyte t)
+      (cd dir)
+      (mew-piolet
+       mew-cs-text-for-read mew-cs-text-for-write
+       (mew-alet ;; xxx
 	(while msgs
 	  (goto-char (point-max))
 	  (setq nxt (nthcdr mew-prog-grep-max-msgs msgs))
 	  (if nxt (setcdr (nthcdr (1- mew-prog-grep-max-msgs) msgs) nil))
-	  (apply 'call-process
-		 mew-prog-vgrep nil t nil
-		 (append mew-prog-vgrep-opts (list pattern) msgs))
-	  (setq msgs nxt))
-	(message "Picking in %s...done" (car flds)))
-      (goto-char (point-min))
-      (setq msgs nil)
-      (while (not (eobp))
-	(if (looking-at mew-regex-message-files2)
-	    (setq msgs (cons (mew-match-string 0) msgs)))
-	(forward-line))
-      (setq msgs (mew-uniq-list msgs))
-      (setq msgs (mapcar 'string-to-number msgs))
-      (setq msgs (sort msgs '<))
-      (setq msgs (mapcar 'number-to-string msgs))
-      (if msgs (setq picks (cons (cons dir msgs) picks)))
-      (setq flds (cdr flds)))
-    (mew-erase-buffer)
-    (setq picks (nreverse picks))
-    (while picks
-      (setq fld (mew-path-to-folder (car (car picks))))
-      (insert "CD: " fld "\n")
-      (setq msgs (cdr (car picks)))
-      (while msgs
-	(insert (car msgs) "\n")
-	(setq msgs (cdr msgs)))
-      (setq picks (cdr picks)))
-    (mew-frwlet
-     mew-cs-text-for-read mew-cs-text-for-write
-     (write-region (point-min) (point-max) file nil 'no-msg))
-    file))
+	  (apply 'call-process prog nil t nil
+		 (append opts (and pattern (list pattern)) msgs))
+	  (setq msgs nxt)))
+       (setq msgs nil)
+       (goto-char (point-min))
+       (while (re-search-forward mew-regex-message-files2 nil t)
+	 (setq msgs (cons (mew-match-string 1) msgs))
+	 (forward-line))
+       (setq msgs (mew-uniq-list msgs))
+       (setq msgs (mapcar 'string-to-number msgs))
+       (setq msgs (sort msgs '<))
+       (setq msgs (mapcar 'number-to-string msgs)))
+      (mew-erase-buffer)
+      (setq rttl (length msgs))
+      (insert "CD: " folder "\n")
+      (mapcar (lambda (x) (insert (mew-msg-get-filename x) "\n")) msgs)
+      (mew-frwlet
+       mew-cs-text-for-read mew-cs-text-for-write
+       (write-region (point-min) (point-max) file nil 'no-msg))
+      (list file rttl))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -183,12 +167,13 @@ This function MUST returns a file name which contains message numbers.")
   "Making Virtual mode for messages marked with '*'.
 If called with '\\[universal-argument]', you can specify a target mark."
   (interactive "P")
-  (if (not (mew-summary-p))
-      (message "This command cannot be used in this mode")
-    (let* ((folder (mew-summary-folder-name 'ext))
-	   (vfolder (mew-summary-get-vfolder (list folder)))
+  (if (not (mew-pickable))
+      (message "This command cannot be used in this folder")
+    (let* ((folder (mew-pickable-folder))
+	   (vfolder (mew-folder-to-virtual folder))
 	   (mark mew-mark-review)
 	   (start (point))
+	   (case-fold-search nil)
 	   beg line med regex)
       (if ask-mark (setq mark (mew-input-mark)))
       (setq regex (mew-mark-regex mark))
@@ -235,7 +220,7 @@ If called with '\\[universal-argument]', you can specify a target mark."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
