@@ -2,12 +2,11 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1994
-;; Revised: Jun  3, 2005
+;; Revised: Dec  3, 2007
 
 ;;; Commentary:
 
 ;; The updated version is available from:
-;;	ftp://ftp.Mew.org/pub/Mew/mew-current.tar.gz
 ;;	http://www.Mew.org/
 ;;
 ;; See info for configuring a site file/.emacs/.mew.el.
@@ -19,7 +18,7 @@
 ;;; Mew version
 ;;;
 
-(defconst mew-version-number "4.2.52"
+(defconst mew-version-number "5.2.52"
   "Version number for this version of Mew.")
 (defconst mew-version (format "Mew version %s" mew-version-number)
   "Version string for this version of Mew.")
@@ -29,19 +28,15 @@
 (require 'mew-func)
 (require 'mew-vars)
 (require 'mew-vars2)
+(require 'mew-vars3)
 (cond
- ((eq system-type 'windows-nt)
+ ((memq system-type '(windows-nt cygwin))
   (require 'mew-win32))
  ((eq system-type 'darwin)
   (require 'mew-darwin))
  (t
   (require 'mew-unix)))
 
-(eval-when-compile
-  (unless (fboundp 'mew-scan-setup)
-    (require 'mew-scan))
-  (mew-scan-setup))
-
 (defun mew-version-show ()
   "Show mew-version in minibuffer."
   (interactive)
@@ -72,7 +67,7 @@
   "Execute Mew first unless Mew is running. And retrieve arrived
 messages or just visit to the default folder.
 
-'proto' is determined by 'mew-proto' and 'mew-input-case'.  
+'proto' is determined by 'mew-proto' and 'mew-case'.  
 
 If 'proto' is '+' (ie a local folder), a mailbox is determined
 according to 'mew-mailbox-type'. Otherwise (ie a remote folder), an
@@ -130,13 +125,20 @@ A Draft buffer is prepared according to SWITCH-FUNCTION.
 CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored."
   (unless mew-init-p (mew-init))
   (let* ((draft (mew-folder-new-message mew-draft-folder))
-	 (attachdir (mew-attachdir draft)))
+	 (attachdir (mew-attachdir draft))
+	 cc asked)
+    (when (and mew-ask-to (null to))
+      (setq to (mew-input-address (concat mew-to: " ")))
+      (setq asked t))
+    (when mew-ask-cc
+      (setq cc (mew-input-address (concat mew-cc: " ")))
+      (setq asked t))
     (mew-current-set-window-config)
     (mew-window-configure 'draft)
     (mew-summary-prepare-draft
      (mew-draft-find-and-switch draft switch-function)
      (mew-delete-directory-recursively attachdir)
-     (mew-draft-header subject nil to nil nil nil nil other-headers)
+     (mew-draft-header subject nil to cc nil nil nil other-headers asked)
      (mew-draft-mode)
      (run-hooks 'mew-draft-mode-newdraft-hook))))
 
@@ -146,21 +148,23 @@ CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored."
 ;;;
 
 (defun mew-init ()
-  (with-temp-buffer
+  (let ((buf (get-buffer-create "*Mew hello*")))
     (run-hooks 'mew-env-hook)
-    ;; need to load the theme file before .mew.el
-    (if mew-theme-file (load mew-theme-file 'no-err 'no-msg))
     (load mew-rc-file 'no-err 'no-msg)
-    (switch-to-buffer (current-buffer))
+    (require 'mew-varsx)
+    (if mew-theme-file (load mew-theme-file 'no-err 'no-msg))
+    (switch-to-buffer buf)
     (mew-window-configure '(1 0)) ;; Using mew-mode-name-message
     (mew-hello)
     (message "Setting up Mew world...")
     (mew-set-environment)
     (run-hooks 'mew-init-hook)
+    (mew-transit-folder)
     (mew-status-update t)
     (mew-passwd-setup)
     (mew-highlight-timer-setup)
     (setq mew-init-p t)
+    (mew-kill-buffer buf)
     (message "Setting up Mew world...done")))
 
 (defun mew-set-environment (&optional no-dir)
@@ -281,9 +285,10 @@ If executed with '3', information about IMAP folders is updated."
 (mew-info-defun "mew-current-" mew-current-info-list)
 
 (defun mew-current-set (fld msg part)
-  (mew-current-set-fld (mew-frame-id) fld)
-  (mew-current-set-msg (mew-frame-id) msg)
-  (mew-current-set-part (mew-frame-id) part))
+  (let ((fid (mew-frame-id)))
+    (mew-current-set-fld fid fld)
+    (mew-current-set-msg fid msg)
+    (mew-current-set-part fid part)))
 
 (defun mew-current-clean-up ()
   (if window-system
@@ -644,6 +649,7 @@ Mew remain, so you can resume with buffer operations."
     (mew-sinfo-clean-up)
     (mew-buffers-clean-up) ;; Summary mode and Virtual mode
     ;;
+    (mew-passwd-clean-up) ;; should be before dir clean up
     (mew-temp-dir-clean-up)
     ;;
     (run-hooks 'mew-quit-hook)
@@ -654,7 +660,6 @@ Mew remain, so you can resume with buffer operations."
     (mew-refile-clean-up)
     (mew-current-clean-up)
     (mew-addrbook-clean-up)
-    (mew-passwd-clean-up)
     (mew-highlight-timer-clean-up)
     (mew-net-clean-up)
     (mew-biff-clean-up)
@@ -667,6 +672,38 @@ Mew remain, so you can resume with buffer operations."
     (setq mew-init-p nil)
     (message nil))) ;; just clear
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Transit
+;;;
+
+;; Spotlight does not handle Files/Directories starting with ".".
+(defvar mew-transit-folder-alist
+  '(("+.pop" "+#pop") ("+.imap" "+#imap") ("+.nntp" "+#nntp")))
+
+(defun mew-transit-folder ()
+  (let ((alist mew-transit-folder-alist)
+	ent old new dirs subold subnew)
+    (with-temp-buffer
+      (while alist
+	(setq ent (car alist))
+	(setq alist (cdr alist))
+	(setq old (mew-expand-folder (nth 0 ent)))
+	(setq new (mew-expand-folder (nth 1 ent)))
+	(when (and (file-directory-p old) (not (file-exists-p new)))
+	  (rename-file old new)
+	  (cd new)
+	  (setq dirs (directory-files "."))
+	  (while dirs
+	    (setq subold (car dirs))
+	    (setq dirs (cdr dirs))
+	    ;; % is a special character for URL.
+	    ;; And "open" does not convert % to %25, sigh.
+	    (when (string-match "@[^%]+\\(%\\)" subold)
+	      (setq subnew (copy-sequence subold))
+	      (aset subnew (match-beginning 1) ?#)
+	      (rename-file subold subnew))))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Load Mew libraries
@@ -689,7 +726,9 @@ Mew remain, so you can resume with buffer operations."
 (require 'mew-bq)
 (require 'mew-syntax)
 (require 'mew-scan)
+(require 'mew-passwd)
 (require 'mew-pick)
+(require 'mew-search)
 (require 'mew-summary)
 (require 'mew-summary2)
 (require 'mew-summary3)
@@ -720,7 +759,7 @@ Mew remain, so you can resume with buffer operations."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994-2005 Mew developing team.
+;; Copyright (C) 1994-2007 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
